/*
 *  ./acpica/busmgr/bm.h - <TBD>
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef __BM_H__
#define __BM_H__

#include "busmgr.h"



/*
 * Control Module Registration:
 * ----------------------------
 */

/* Notification Callback Function */
typedef 
ACPI_STATUS (*BM_NOTIFY_CALLBACK) (
    BM_NOTIFY               NotifyType,
    BM_HANDLE               DeviceHandle,
    void                    **Context);

/* Request Callback Function */ 
typedef 
ACPI_STATUS (*BM_REQUEST_CALLBACK) (
    BM_REQUEST_INFO         *RequestInfo,
    void                    *Context);

/* Structure containing Control Module callback functions */
typedef struct
{
    BM_NOTIFY_CALLBACK      Notify;
    BM_REQUEST_CALLBACK     Request;
    void                    *Context;
} BM_CONTROL_INFO;


/* 
 * Internal Device:
 * ----------------
 * Structure for internal representation of an ACPI Bus Manager 'device'
 * and maintaining the device hierarchy.
 */
typedef struct BmDevice
{
    BM_DEVICE_INFO          Info;
    BM_CONTROL_INFO         Control;
    struct BmDevice         *Parent;
    struct BmDevice         *Next;
    struct 
    {
        struct BmDevice         *Head;
        struct BmDevice         *Tail;
    } Scope;
} BM_INTERNAL_DEVICE;


/*
 * Internal Device List:
 * ---------------------
 * Structure containing a list of pointers to <internal> devices.
 */
typedef struct
{
    UINT32                  Count;
    BM_INTERNAL_DEVICE      *Devices[BM_MAX_HANDLES];
} BM_INTERNAL_DEVICE_LIST;



ACPI_STATUS
BmPrintDevice (
    BM_INTERNAL_DEVICE      *Device);

ACPI_STATUS
BmCalculateDeviceStatus (
    BM_INTERNAL_DEVICE      *Device);

ACPI_STATUS
BmCalculateDeviceInfo (
    ACPI_OBJECT_TYPE        AcpiType,
    BM_INTERNAL_DEVICE      *Device);

ACPI_STATUS
BmAddDevice (
    ACPI_HANDLE             AcpiHandle,
    ACPI_OBJECT_TYPE        AcpiType,
    BM_INTERNAL_DEVICE      *ParentDevice,
    BM_INTERNAL_DEVICE      **ChildDevice);

ACPI_STATUS
BmEnumerateNamespace(void);

ACPI_STATUS
BmGetDeviceStatus (
    BM_HANDLE               DeviceHandle,
    BM_DEVICE_STATUS        *DeviceStatus);
    
ACPI_STATUS
BmGetDeviceInfo(
    BM_HANDLE               DeviceHandle,
    BM_DEVICE_INFO          **DeviceInfo);

ACPI_STATUS
BmGetDeviceHandle(
    ACPI_HANDLE             AcpiHandle,
    BM_HANDLE               *DeviceHandle);

ACPI_STATUS
BmGetDeviceContext(
    BM_HANDLE               DeviceHandle,
    void                    **Context);

ACPI_STATUS
BmSearch(
    BM_HANDLE               StartDeviceHandle,
    BM_DEVICE_INFO          *Criteria,
    BM_HANDLE_LIST          *Results);

ACPI_STATUS
BmRegisterControl(
    BM_DEVICE_INFO          *Criteria,
    BM_CONTROL_INFO         *Control);

ACPI_STATUS
BmUnregisterControl(
    BM_DEVICE_INFO          *Criteria,
    BM_CONTROL_INFO         *Control);


#endif /* __BM_H__ */