/*
 *  ./acpica/busmgr/busmgr.h - <TBD>
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef __BUSMGR_H__
#define __BUSMGR_H__

/* ACPI Core Subsystem Includes */
#include <actypes.h>
#include <acexcep.h>



#ifndef ARRAY_ANYSIZE
#define ARRAY_ANYSIZE                   1
#endif

#define BM_MAX_HANDLES                  256

/* 
 * Commands:
 * ---------
 */
typedef UINT32                          BM_COMMAND;

#define BM_COMMAND_UNKNOWN              ((BM_COMMAND) 0x00000000)

#define BM_COMMAND_SEARCH               ((BM_COMMAND) 0x00000001)
#define BM_COMMAND_GET_INFO             ((BM_COMMAND) 0x00000002)
#define BM_COMMAND_GET_STATUS           ((BM_COMMAND) 0x00000003)

#define BM_COMMAND_GET_CURRENT_RESOURCES ((BM_COMMAND) 0x00000010)
#define BM_COMMAND_GET_POSSIBLE_RESOURCES ((BM_COMMAND) 0x00000011)
#define BM_COMMAND_SET_CURRENT_RESOURCES ((BM_COMMAND) 0x00000012)

#define BM_COMMAND_DEVICE_SPECIFIC      ((BM_COMMAND) 0x00000080)

/*
 * Notifications:
 * --------------
 */
typedef UINT32                          BM_NOTIFY;

#define BM_NOTIFY_UNKNOWN               ((BM_NOTIFY) 0x00000000)

#define BM_NOTIFY_DEVICE_ADDED          ((BM_NOTIFY) 0x00000001)
#define BM_NOTIFY_DEVICE_REMOVED        ((BM_NOTIFY) 0x00000002)

/* 
 * Device Handles:
 * ---------------
 */
typedef UINT32                          BM_HANDLE;

#define BM_HANDLE_UNKNOWN               ((BM_HANDLE) 0x00000000)
#define BM_HANDLE_ROOT                  ((BM_HANDLE) 0x00000000)

/*
 * Device Classes:
 * ---------------
 */
typedef UINT32                          BM_DEVICE_CLASS;

#define BM_CLASS_UNKNOWN                ((BM_DEVICE_CLASS) 0x00000000)

#define BM_CLASS_SCOPE                  ((BM_DEVICE_CLASS) 0x00000001)
#define BM_CLASS_PROCESSOR              ((BM_DEVICE_CLASS) 0x00000002)
#define BM_CLASS_THERMAL_ZONE           ((BM_DEVICE_CLASS) 0x00000004)
#define BM_CLASS_POWER_RESOURCE         ((BM_DEVICE_CLASS) 0x00000008)
#define BM_CLASS_DEVICE                 ((BM_DEVICE_CLASS) 0x00000010)

/*
 * Device HIDs:
 * ------------
 */
typedef char                          BM_DEVICE_HID[9];

#define BM_HID_UNKNOWN                  '\0'

#define BM_HID_EMBEDDED_CONTROLLER      "PNP0C09"
#define BM_HID_CM_BATTERY               "PNP0C0A"

/*
 * Device Flags:
 * -------------
 * The encoding of BM_DEVICE_FLAGS is illustrated below.
 * Note that a set bit (1) indicates the property is TRUE
 * (e.g. if bit 0 is set then the device has dynamic status).
 * +-----------+-+-+-+-+-+-+
 * | Bits 31:6 |5|4|3|2|1|0|    
 * +-----------+-+-+-+-+-+-+
 *       |      | | | | | |
 *       |      | | | | | +- Dynamic status?
 *       |      | | | | +--- Identifiable?
 *       |      | | | +----- Configurable?
 *       |      | | +------- Power Manageable?
 *       |      | +--------- Ejectable?
 *       |      +----------- Docking Station?
 *       +------------------ <Reserved>
 *
 * Dynamic Status:  Device has a _STA object.
 * Identifiable:    Device has a _HID and/or _ADR and possibly
 *                  other identification objects defined.
 * Configurable:    Device has a _CRS and possibly other
 *                  configuration objects defined.
 * Power Manageable:Device has a _PR0 and/or _PS0 and possibly
 *                  other power management objects defined.
 * Ejectable:       Device has an _EJD and/or _EJx and possibly
 *                  other dynamic insertion/removal objects defined.
 * Docking Station: Device has a _DCK object defined.
 *
 */
typedef UINT32                          BM_DEVICE_FLAGS;

#define BM_FLAGS_UNKNOWN                ((BM_DEVICE_FLAGS) 0x00000000)

#define BM_FLAGS_DYNAMIC_STATUS         ((BM_DEVICE_FLAGS) 0x00000001)
#define BM_FLAGS_IDENTIFIABLE           ((BM_DEVICE_FLAGS) 0x00000002)
#define BM_FLAGS_CONFIGURABLE           ((BM_DEVICE_FLAGS) 0x00000004)
#define BM_FLAGS_POWER_MANAGEABLE       ((BM_DEVICE_FLAGS) 0x00000008)
#define BM_FLAGS_EJECTABLE              ((BM_DEVICE_FLAGS) 0x00000010)
#define BM_FLAGS_DOCKING_STATION        ((BM_DEVICE_FLAGS) 0x00000020)
#define BM_FLAGS_CONTROL                ((BM_DEVICE_FLAGS) 0x80000000)


/*
 * Device Status:
 * --------------
 * The encoding of BM_DEVICE_STATUS is illustrated below.
 * Note that a set bit (1) indicates the property is TRUE
 * (e.g. if bit 0 is set then the device is present).
 * +-----------+-+-+-+-+
 * | Bits 31:4 |3|2|1|0|    
 * +-----------+-+-+-+-+
 *       |      | | | |
 *       |      | | | +- Present?
 *       |      | | +--- Enabled?
 *       |      | +----- Show in UI?
 *       |      +------- Functioning?
 *       +-------------- <Reserved>
 */
typedef UINT32                          BM_DEVICE_STATUS;

#define BM_STATUS_UNKNOWN               ((BM_DEVICE_STATUS) 0x00000000)
#define BM_STATUS_PRESENT               ((BM_DEVICE_STATUS) 0x00000001)
#define BM_STATUS_ENABLED               ((BM_DEVICE_STATUS) 0x00000002)
#define BM_STATUS_SHOW_UI               ((BM_DEVICE_STATUS) 0x00000004)
#define BM_STATUS_FUNCTIONING           ((BM_DEVICE_STATUS) 0x00000008)
#define BM_STATUS_DEFAULT               ((BM_DEVICE_STATUS) 0x0000000F)

/*
 * Device Information:
 * -------------------
 * Structure containing basic device information.
 */
typedef struct
{
    BM_HANDLE               DeviceHandle;
    ACPI_HANDLE             AcpiHandle;
    char                    Name[5];
    BM_DEVICE_CLASS         Class;
    BM_DEVICE_HID           HardwareId;
    BM_DEVICE_FLAGS         Flags;
    BM_DEVICE_STATUS        Status;
} BM_DEVICE_INFO;

/*
 * Handle List:
 * ------------
 * Structure containing a list of device handles.
 */
typedef struct
{
    UINT32                  Count;
    BM_HANDLE               Handles[BM_MAX_HANDLES];
} BM_HANDLE_LIST;

/*
 * Search Info:
 * ------------
 * Structure containing seach criteria and results.
 */
typedef struct
{
    BM_DEVICE_INFO          Criteria;
    BM_HANDLE_LIST          Results;
} BM_SEARCH_INFO;

/* 
 * Request:
 * --------
 * Structure for sending a request (command) to the ACPI Bus Manager.
 */
typedef struct 
{
    ACPI_STATUS             Status;
    BM_COMMAND              Command;
    BM_HANDLE               DeviceHandle;
    ACPI_BUFFER             Buffer;
} BM_REQUEST_INFO;



ACPI_STATUS
BmInitialize(void);

ACPI_STATUS
BmTerminate(void);

void
BmNotify (
    ACPI_HANDLE         AcpiHandle,
    UINT32              NotifyValue,
    void                *Context);

ACPI_STATUS
BmRequest(
    BM_REQUEST_INFO         *RequestInfo);


#endif  /* __BUSMGR_H__ */
