/*
 *  ./acpica/control/battery/battery.c - 
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __BATTERY_C__

// TODO: remove...
#include <KernelExport.h>

#include <acpi.h>

#include <bm.h>
#include <utility.h>

#include "battery.h"

#define _COMPONENT          BATTERY
        MODULE_NAME         ("battery");




ACPI_STATUS
BtGetInfo(
    BM_DEVICE_INFO          *DeviceInfo,
    ACPI_BUFFER             *Buffer)
{
    ACPI_STATUS             Status = AE_OK;
    ACPI_OBJECT             *Package = NULL;
    ACPI_BUFFER             TempBuffer;
    ACPI_BUFFER             PackageFormat;

    FUNCTION_TRACE("BtGetInfo");

    if (!DeviceInfo || !Buffer)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    TempBuffer.Length = 0;
    TempBuffer.Pointer = NULL;

    /*
     * Evalute _BIF:
     * -------------
     * Note: The first attempt is just to get the size of the _BIF package.
     */
    Status = AcpiEvaluateObject(DeviceInfo->AcpiHandle, "_BIF", NULL, &TempBuffer);

    if (Status == AE_OK)
    {
        DEBUG_PRINT(ACPI_WARN, ("BtGetInfo: Invalid _BIF.\n"));
        return_ACPI_STATUS(AE_ERROR);
    }

    if (Status != AE_BUFFER_OVERFLOW)
    {
        return_ACPI_STATUS(Status);
    }

    TempBuffer.Pointer = OsdCallocate(TempBuffer.Length);
    if (!TempBuffer.Pointer)
    {
        return_ACPI_STATUS(AE_NO_MEMORY);
    }

    if (ACPI_FAILURE(Status = AcpiEvaluateObject(DeviceInfo->AcpiHandle, "_BIF", NULL, &TempBuffer)))
    {
        goto Cleanup;
    }

    /*
     * Extract Package Data:
     * ---------------------
     * Convert the data from an ACPI_TYPE_Package to the more user-friendly
     * BT_BATTERY_INFO type.
     */
    if (ACPI_FAILURE(Status = UtCastBuffer(&TempBuffer, (void**)&Package, sizeof(ACPI_OBJECT))))
    {
        goto Cleanup;
    }

    PackageFormat.Length = sizeof("NNNNNNNNNSSSS");
    PackageFormat.Pointer = "NNNNNNNNNSSSS";

    if (ACPI_FAILURE(Status = UtExtractPackageData(Package, &PackageFormat, Buffer)))
    {
        goto Cleanup;
    }

Cleanup:

    OsdFree(TempBuffer.Pointer);

    return_ACPI_STATUS(Status);
}



ACPI_STATUS
BtGetStatus(
    BM_DEVICE_INFO          *DeviceInfo,
    ACPI_BUFFER             *Buffer)
{
    ACPI_STATUS             Status = AE_OK;
    ACPI_OBJECT             *Package = NULL;
    ACPI_BUFFER             TempBuffer;
    ACPI_BUFFER             PackageFormat;

    FUNCTION_TRACE("BtGetStatus");

    if (!DeviceInfo || !Buffer)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    TempBuffer.Length = 0;
    TempBuffer.Pointer = NULL;

    /*
     * Evalute _BST:
     * -------------
     * Note: The first attempt is just to get the size of the _BST package.
     */
    Status = AcpiEvaluateObject(DeviceInfo->AcpiHandle, "_BST", NULL, &TempBuffer);

    if (Status == AE_OK)
    {
        DEBUG_PRINT(ACPI_WARN, ("BtGetInfo: Invalid _BST.\n"));
        return_ACPI_STATUS(AE_ERROR);
    }

    if (Status != AE_BUFFER_OVERFLOW)
    {
        return_ACPI_STATUS(Status);
    }

    TempBuffer.Pointer = OsdCallocate(TempBuffer.Length);
    if (!TempBuffer.Pointer)
    {
        return_ACPI_STATUS(AE_NO_MEMORY);
    }

    if (ACPI_FAILURE(Status = AcpiEvaluateObject(DeviceInfo->AcpiHandle, "_BST", NULL, &TempBuffer)))
    {
        goto Cleanup;
    }

    /*
     * Extract Package Data:
     * ---------------------
     * Convert the data from an ACPI_TYPE_Package to the more user-friendly
     * BT_BATTERY_STATUS type.
     */
    if (ACPI_FAILURE(Status = UtCastBuffer(&TempBuffer, (void**)&Package, sizeof(ACPI_OBJECT))))
    {
        goto Cleanup;
    }

    PackageFormat.Length = sizeof("NNNN");
    PackageFormat.Pointer = "NNNN";

    if (ACPI_FAILURE(Status = UtExtractPackageData(Package, &PackageFormat, Buffer)))
    {
        goto Cleanup;
    }

Cleanup:

    OsdFree(TempBuffer.Pointer);

    return_ACPI_STATUS(Status);
}



ACPI_STATUS
BtSetTripPoint(
    BM_DEVICE_INFO          *DeviceInfo,
    ACPI_BUFFER             *Buffer)
{
    ACPI_STATUS             Status = AE_OK;
    UINT32                  *TripPoint = NULL;
    ACPI_OBJECT_LIST        ArgList;
    ACPI_OBJECT             Arg0;

    FUNCTION_TRACE("BtSetTripPoint");

    if (!DeviceInfo || !Buffer)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    /*
     * 'Extract' the new trip point value from the ACPI buffer and check.
     */
    if (ACPI_FAILURE(Status = UtCastBuffer(Buffer, (void**)&TripPoint, sizeof(UINT32))))
    {
        return_ACPI_STATUS(Status);
    }

    if ((*TripPoint) > BT_MAX_TRIP_POINT_VALUE)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    MEMSET(&ArgList, 0, sizeof(ACPI_OBJECT));
    ArgList.Count = 1;
    ArgList.Pointer = &Arg0;

    MEMSET(&Arg0, 0, sizeof(ACPI_OBJECT));
    Arg0.Type = ACPI_TYPE_Number;
    Arg0.Number.Value = *TripPoint;

    Status = AcpiEvaluateObject(DeviceInfo->AcpiHandle, "_BTP", &ArgList, NULL);

    return_ACPI_STATUS(Status);
}



ACPI_STATUS
BtGetDevicesPowered(
    BM_DEVICE_INFO          *DeviceInfo,
    ACPI_BUFFER             *Buffer)
{
    ACPI_STATUS             Status = AE_OK;
    ACPI_HANDLE             AcpiHandle = NULL;
    BM_HANDLE_LIST          HandleList;

    FUNCTION_TRACE("BtGetDevicesPowered");

    if (!DeviceInfo || !Buffer)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    MEMSET(&HandleList, 0, sizeof(BM_HANDLE_LIST));

    if (ACPI_FAILURE(AcpiGetHandle(DeviceInfo->AcpiHandle, "_PCL", &AcpiHandle)))
    {
        return_ACPI_STATUS(Status);
    }

    if (ACPI_FAILURE(UtEvaluateReferenceList(AcpiHandle, &HandleList)))
    {
        return_ACPI_STATUS(Status);
    }

    Status = UtCopyToBuffer(Buffer, &HandleList, sizeof(BM_HANDLE_LIST));

    return_ACPI_STATUS(Status);
}




ACPI_STATUS
BtInitialize(void)
{
    ACPI_STATUS             Status = AE_OK;
    BM_DEVICE_INFO          Criteria;
    BM_CONTROL_INFO         Control;

    FUNCTION_TRACE("BtInitialize");

    MEMSET(&Criteria, 0, sizeof(BM_DEVICE_INFO));
    MEMSET(&Control, 0, sizeof(BM_CONTROL_INFO));

    MEMCPY(Criteria.HardwareId, BM_HID_CM_BATTERY, sizeof(BM_HID_CM_BATTERY));

    Control.Notify = &BtNotify;
    Control.Request = &BtRequest;

    Status = BmRegisterControl(&Criteria, &Control);

    return_ACPI_STATUS(Status);
}



ACPI_STATUS
BtTerminate(void)
{
    ACPI_STATUS             Status = AE_OK;
    BM_DEVICE_INFO          Criteria;
    BM_CONTROL_INFO         Control;

    FUNCTION_TRACE("BtTerminate");

    MEMSET(&Criteria, 0, sizeof(BM_DEVICE_INFO));
    MEMSET(&Control, 0, sizeof(BM_CONTROL_INFO));

    MEMCPY(Criteria.HardwareId, BM_HID_CM_BATTERY, sizeof(BM_HID_CM_BATTERY));

    Control.Notify = &BtNotify;
    Control.Request = &BtRequest;

    Status = BmUnregisterControl(&Criteria, &Control);

    return_ACPI_STATUS(Status);
}



ACPI_STATUS 
BtNotify (
    BM_NOTIFY               NotifyType,
    BM_HANDLE               DeviceHandle,
    void                    **Context)
{
    ACPI_STATUS             Status = AE_OK;

    FUNCTION_TRACE("BtNotify");

    switch (NotifyType)
    {
    case BM_NOTIFY_DEVICE_ADDED:
        DEBUG_PRINT(ACPI_INFO, ("BtNotify: Adding battery device [0x%08X].\n", DeviceHandle));
        break;

    case BM_NOTIFY_DEVICE_REMOVED:
        DEBUG_PRINT(ACPI_INFO, ("BtNotify: Removing battery device [0x%08X].\n", DeviceHandle));
        break;

    case BT_NOTIFY_STATUS_CHANGE:
        // TODO...
        DEBUG_PRINT(ACPI_INFO, ("BtNotify: Battery status (_BST) change event detected.\n"));
        break;

    case BT_NOTIFY_INFORMATION_CHANGE:
        // TODO...
        DEBUG_PRINT(ACPI_INFO, ("BtNotify: Battery information (_BIF) change event detected.\n"));
        break;

    default:
        Status = AE_SUPPORT;
        break;
    }

    return_ACPI_STATUS(Status);
}    



ACPI_STATUS
BtRequest (
    BM_REQUEST_INFO         *RequestInfo,
    void                    *Context)
{
    ACPI_STATUS             Status = AE_OK;
    BM_DEVICE_INFO          *DeviceInfo = NULL;

    FUNCTION_TRACE("BtRequest");

    /*
     * Must have a valid request structure.  Don't care about context, as
     * device-specific information is not needed for this control module.
     */
    if (!RequestInfo || !RequestInfo->Buffer.Pointer)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    /*
     * Get details on this device...
     */
    if (ACPI_FAILURE(Status = BmGetDeviceInfo(RequestInfo->DeviceHandle, &DeviceInfo)))
    {
        return_ACPI_STATUS(Status);
    }

    /*
     * Handle Request:
     * ---------------
     */
    switch (RequestInfo->Command)
    {
    case BT_COMMAND_GET_INFO:
        Status = BtGetInfo(DeviceInfo, &(RequestInfo->Buffer));
        break;

    case BT_COMMAND_GET_STATUS:
        Status = BtGetStatus(DeviceInfo, &(RequestInfo->Buffer));
        break;

    case BT_COMMAND_SET_TRIP_POINT:
        Status = BtSetTripPoint(DeviceInfo, &(RequestInfo->Buffer));
        break;

    case BT_COMMAND_GET_DEVICES_POWERED:
        Status = BtGetDevicesPowered(DeviceInfo, &(RequestInfo->Buffer));
        break;

    default:
        Status = AE_SUPPORT;
        break;
    }

    RequestInfo->Status = Status;

    return_ACPI_STATUS(Status);
}
