/*
 *  ./acpica/control/battery/battery.h - 
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef __BATTERY_H__
#define __BATTERY_H__

#include <busmgr.h>



#define BT_MAX_TRIP_POINT_VALUE         0x7FFFFFFF

/* 
 * Battery Commands:
 * -----------------
 */
#define BT_COMMAND_GET_INFO             ((BM_COMMAND) 0x80)
#define BT_COMMAND_GET_STATUS           ((BM_COMMAND) 0x81)
#define BT_COMMAND_SET_TRIP_POINT       ((BM_COMMAND) 0x82)
#define BT_COMMAND_GET_DEVICES_POWERED  ((BM_COMMAND) 0x83)

/* 
 * Battery Notifications:
 * ----------------------
 */
#define BT_NOTIFY_STATUS_CHANGE         ((BM_NOTIFY) 0x80)
#define BT_NOTIFY_INFORMATION_CHANGE    ((BM_NOTIFY) 0x81)

/* 
 * BT_CM_BATTERY_INFO:
 * -------------------
 */
typedef struct {
 UINT32   PowerUnit;
 UINT32   DesignCapacity;
 UINT32   LastFullChargeCapacity;
 UINT32   BatteryTechnology;
 UINT32   DesignVoltage;
 UINT32   DesignCapacityOfWarning;
 UINT32   DesignCapacityOfLow;
 UINT32   DesignCapacityGranularity1;
 UINT32   DesignCapacityGranularity2;
 ACPI_STRING  ModelNumber;
 ACPI_STRING  SerialNumber;
 ACPI_STRING  BatteryType;
 ACPI_STRING  OEMInformation;
} BT_BATTERY_INFO;

/* 
 * BT_CM_BATTERY_STATUS:
 * ---------------------
 */
typedef struct {
 UINT32   State;
 UINT32   PresentRate;
 UINT32   RemainingCapacity;
 UINT32   PresentVoltage;
} BT_BATTERY_STATUS;



ACPI_STATUS
BtInitialize(void);

ACPI_STATUS
BtTerminate(void);

ACPI_STATUS 
BtNotify (
    BM_NOTIFY               NotifyType,
    BM_HANDLE               DeviceHandle,
    void                    **Context);

ACPI_STATUS
BtRequest(
    BM_REQUEST_INFO         *RequestInfo,
    void                    *Context);


#endif  /* __BATTERY_H__ */
