/*
 *  ./acpica/control/ec/ectransx.c - <TBD>
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __ECTRANSX_C__

#include <acpi.h>

#include "ec.h"

#define _COMPONENT          EMBEDDED_CONTROLLER
        MODULE_NAME         ("ectransx");




ACPI_STATUS
EcWaitEvent (
    EC_INFO                 *EmbeddedController,
    EC_EVENT                Event)
{
    EC_STATUS               EcStatus = 0;
    UINT32                  i = 0;

    /*FUNCTION_TRACE("EcWaitEvent");*/

    if (!EmbeddedController || (Event != EC_EVENT_OUTPUT_BUFFER_FULL && Event != EC_EVENT_INPUT_BUFFER_EMPTY))
    {
        /*return_ACPI_STATUS(AE_BAD_PARAMETER);*/
        return(AE_BAD_PARAMETER);
    }

    /*
     * Stall 1us:
     * ----------
     * Stall for 1 microsecond before reading the status register
     * for the first time.  This allows the EC to set the IBF/OBF
     * bit to its proper state.
     */
    OsdStall(1);

    EcStatus = OsdIn8(EmbeddedController->StatusPort);

    /*
     * Wait For Event:
     * ---------------
     * Poll the EC status register to detect completion of the last
     * command.  Wait up to 1ms (in 10us chunks) for this to occur.
     */
    for (i = 0; i < 100; i++)
    {
        EcStatus = OsdIn8(EmbeddedController->StatusPort);

        if (Event == EC_EVENT_OUTPUT_BUFFER_FULL)
        {
            if (EcStatus & EC_FLAG_OUTPUT_BUFFER)
            {
                /*return_ACPI_STATUS(AE_OK);*/
                return(AE_OK);
                break;
            }
        }

        else /* Event == EC_EVENT_INPUT_BUFFER_EMPTY */
        {
            if (!(EcStatus & EC_FLAG_INPUT_BUFFER))
            {
                /*return_ACPI_STATUS(AE_OK);*/
                return(AE_OK);
            }
        }

        OsdStall(10);
    }

    /*return_ACPI_STATUS(AE_ERROR);*/
    return(AE_ERROR);
}    



ACPI_STATUS
EcRead (
    EC_INFO                 *EmbeddedController,
    UINT8                   Address,
    UINT8                   *Data)
{
    ACPI_STATUS             Status = AE_OK;

    FUNCTION_TRACE("EcRead");

    if (!EmbeddedController || !Data)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    //EcBurstEnable(EmbeddedController);

    OsdOut8(EmbeddedController->CommandPort, EC_COMMAND_READ);
    if (ACPI_FAILURE(Status = EcWaitEvent(EmbeddedController, EC_EVENT_INPUT_BUFFER_EMPTY)))
    {
        DEBUG_PRINT(ACPI_WARN, ("EcRead: Failed waiting for EC to process read command.\n"));
        return_ACPI_STATUS(Status);
    }

    OsdOut8(EmbeddedController->DataPort, Address);
    if (ACPI_FAILURE(Status = EcWaitEvent(EmbeddedController, EC_EVENT_OUTPUT_BUFFER_FULL)))
    {
        DEBUG_PRINT(ACPI_WARN, ("EcRead: Failed waiting for EC to send data.\n"));
        return_ACPI_STATUS(Status);
    }

    (*Data) = OsdIn8(EmbeddedController->DataPort);

    //EcBurstDisable(EmbeddedController);

    DEBUG_PRINT(ACPI_INFO, ("EcRead: Address[0x%02x] Data[0x%02x]\n", Address, (*Data)));

    return_ACPI_STATUS(Status);
}    



ACPI_STATUS
EcWrite (
    EC_INFO                 *EmbeddedController,
    UINT8                   Address,
    UINT8                   *Data)
{
    ACPI_STATUS             Status = AE_OK;

    FUNCTION_TRACE("EcWrite");

    if (!EmbeddedController || !Data)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    //EcBurstEnable(EmbeddedController);

    OsdOut8(EmbeddedController->CommandPort, EC_COMMAND_WRITE);
    if (ACPI_FAILURE(Status = EcWaitEvent(EmbeddedController, EC_EVENT_INPUT_BUFFER_EMPTY)))
    {
        DEBUG_PRINT(ACPI_WARN, ("EcWrite: Failed waiting for EC to process write command.\n"));
        return_ACPI_STATUS(Status);
    }

    OsdOut8(EmbeddedController->DataPort, Address);
    if (ACPI_FAILURE(Status = EcWaitEvent(EmbeddedController, EC_EVENT_INPUT_BUFFER_EMPTY)))
    {
        DEBUG_PRINT(ACPI_WARN, ("EcWrite: Failed waiting for EC to process address.\n"));
        return_ACPI_STATUS(Status);
    }

    OsdOut8(EmbeddedController->DataPort, (*Data));
    if (ACPI_FAILURE(Status = EcWaitEvent(EmbeddedController, EC_EVENT_INPUT_BUFFER_EMPTY)))
    {
        DEBUG_PRINT(ACPI_WARN, ("EcWrite: Failed waiting for EC to process data.\n"));
        return_ACPI_STATUS(Status);
    }

    //EcBurstDisable(EmbeddedController);

    DEBUG_PRINT(ACPI_INFO, ("EcWrite: Address[0x%02x] Data[0x%02x]\n", Address, (*Data)));

    return_ACPI_STATUS(Status);
}    



ACPI_STATUS
EcQuery (
    EC_INFO                 *EmbeddedController,
    UINT8                   *Data)
{
    ACPI_STATUS             Status = AE_OK;

    FUNCTION_TRACE("EcQuery");

    if (!EmbeddedController || !Data)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    OsdOut8(EmbeddedController->CommandPort, EC_COMMAND_QUERY);

    if (ACPI_FAILURE(Status = EcWaitEvent(EmbeddedController, EC_EVENT_OUTPUT_BUFFER_FULL)))
    {
        DEBUG_PRINT(ACPI_WARN, ("EcQuery: Failed waiting for EC to send data.\n"));
        return_ACPI_STATUS(Status);
    }

    (*Data) = OsdIn8(EmbeddedController->DataPort);

    DEBUG_PRINT(ACPI_INFO, ("EcQuery: Data[0x%02x]\n", (*Data)));

    return_ACPI_STATUS(Status);
}    



ACPI_STATUS
EcBurstEnable(
    EC_INFO                 *EmbeddedController)
{
    ACPI_STATUS             Status = AE_OK;
    UINT8                   Data = 0;

    /*FUNCTION_TRACE("EcBurstEnable");*/

    OsdOut8(EmbeddedController->CommandPort, EC_COMMAND_BURST_ENABLE);

    if (ACPI_FAILURE(Status = EcWaitEvent(EmbeddedController, EC_EVENT_OUTPUT_BUFFER_FULL)))
    {
        DEBUG_PRINT(ACPI_WARN, ("EcBurstEnable: Failed waiting for burst enable acknowledge byte.\n"));
        /*return_ACPI_STATUS(Status);*/
        return(Status);
    }

    Data = OsdIn8(EmbeddedController->DataPort);
    if (Data != EC_BURST_ENABLE_ACKNOWLEDGE)
    {
        DEBUG_PRINT(ACPI_WARN, ("EcBurstEnable: Invalid acknowledge byte received [0x%02x].\n", Data));
        /*return_ACPI_STATUS(Status);*/
        return(Status);
    }

    /*return_ACPI_STATUS(Status);*/
    return(Status);
}    



ACPI_STATUS
EcBurstDisable(
    EC_INFO                 *EmbeddedController)
{
    ACPI_STATUS             Status = AE_OK;
    EC_STATUS               EcStatus = 0;

    /*FUNCTION_TRACE("EcBurstDisable");*/

    EcStatus = OsdIn8(EmbeddedController->StatusPort);
    if (!(EcStatus & EC_FLAG_BURST_MODE))
    {
        DEBUG_PRINT(ACPI_INFO, ("EcBurstDisable: Burst mode already disabled.\n"));
        /*return_ACPI_STATUS(AE_OK);*/
        return(AE_OK);
    }

    OsdOut8(EmbeddedController->CommandPort, EC_COMMAND_BURST_DISABLE);

    if (ACPI_FAILURE(Status = EcWaitEvent(EmbeddedController, EC_EVENT_INPUT_BUFFER_EMPTY)))
    {
        DEBUG_PRINT(ACPI_WARN, ("EcBurstDisable: Failed waiting for EC to consume burst disable command.\n"));
        /*return_ACPI_STATUS(Status);*/
        return(Status);
    }

    /*return_ACPI_STATUS(Status);*/
    return(Status);
}    



ACPI_STATUS
EcTransaction (
    EC_INFO                 *EmbeddedController,
    EC_REQUEST              *EcRequest)
{
    ACPI_STATUS             Status = AE_OK;

    FUNCTION_TRACE("EcTransaction");

    if (!EmbeddedController || !EcRequest)
    {
        OsdPrintf(">>> EcTransaction: Invalid EmbeddedController | EcRequest\n");
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    /*
     * Disable EC GPE:
     * ---------------
     * Disable EC interrupts (GPEs) from occuring during this transaction.
     * This is done here as EcTransaction() is also called by the EC GPE
     * handler -- where disabling/re-enabling the EC GPE is automatically
     * handled by the ACPI Core Subsystem.
     */ 
    if (ACPI_FAILURE(AcpiDisableEvent(EmbeddedController->GpeBit, EVENT_GPE)))
    {
        DEBUG_PRINT(ACPI_ERROR, ("EcRequest: Unable to disable the EC GPE.\n"));
    }

    /*
     * Perform the transaction.
     */
    switch (EcRequest->Command)
    {
    case EC_COMMAND_READ:
        Status = EcRead(EmbeddedController, EcRequest->Address, &(EcRequest->Data));
        break;

    case EC_COMMAND_WRITE:
        Status = EcWrite(EmbeddedController, EcRequest->Address, &(EcRequest->Data));
        break;

    default:
        Status = AE_SUPPORT;
        break;
    }

    /*
     * Clear & Re-Enable the EC GPE:
     * -----------------------------
     * 'Consume' any EC GPE events that we generated while performing
     * the transaction (e.g. IBF/OBF).  Clearing the GPE here shouldn't
     * have an adverse affect on outstanding EC-SCI's, as the source
     * (EC-SCI) will still be high and thus should trigger the GPE
     * immediately after we re-enabling it.
     */
    if (ACPI_FAILURE(AcpiClearEvent(EmbeddedController->GpeBit, EVENT_GPE)))
    {
        DEBUG_PRINT(ACPI_ERROR, ("EcRequest: Unable to clear the EC GPE.\n"));
    }
    if (ACPI_FAILURE(AcpiEnableEvent(EmbeddedController->GpeBit, EVENT_GPE)))
    {
        DEBUG_PRINT(ACPI_ERROR, ("EcRequest: Unable to re-enable the EC GPE.\n"));
    }

    return_ACPI_STATUS(Status);
}


