/*
 *  ./acpica/control/power/power.h - <TBD>
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef __POWER_H__
#define __POWER_H__

#include <busmgr.h>


/* 
 * Power Resource Commands:
 * ------------------------
 */
#define PW_COMMAND_GET_INFO             ((BM_COMMAND) 0x80)
#define PW_COMMAND_GET_POWER_STATE      ((BM_COMMAND) 0x81)
#define PW_COMMAND_SET_POWER_STATE      ((BM_COMMAND) 0x82)



/*
 * PW_POWER_STATE:
 * ---------------
 */
typedef UINT32                          PW_POWER_STATE;

#define PW_POWER_STATE_UNKNOWN          ((PW_POWER_STATE) 0xFFFFFFFF)

#define PW_SYSTEM_POWER_STATE_S0        ((PW_POWER_STATE) 0x00000000)
#define PW_SYSTEM_POWER_STATE_S1        ((PW_POWER_STATE) 0x00000001)
#define PW_SYSTEM_POWER_STATE_S2        ((PW_POWER_STATE) 0x00000002)
#define PW_SYSTEM_POWER_STATE_S3        ((PW_POWER_STATE) 0x00000003)
#define PW_SYSTEM_POWER_STATE_S4        ((PW_POWER_STATE) 0x00000004)
#define PW_SYSTEM_POWER_STATE_S5        ((PW_POWER_STATE) 0x00000005)

#define PW_SYSTEM_POWER_STATE_COUNT     6

#define PW_DEVICE_POWER_STATE_D0        ((PW_POWER_STATE) 0x00000000)
#define PW_DEVICE_POWER_STATE_D1        ((PW_POWER_STATE) 0x00000001)
#define PW_DEVICE_POWER_STATE_D2        ((PW_POWER_STATE) 0x00000002)
#define PW_DEVICE_POWER_STATE_D3        ((PW_POWER_STATE) 0x00000003)

#define PW_DEVICE_POWER_STATE_COUNT     4

#define PW_RESOURCE_POWER_STATE_ON      ((PW_POWER_STATE) 0x00000000)
#define PW_RESOURCE_POWER_STATE_OFF     ((PW_POWER_STATE) 0x00000003)

#define PW_RESOURCE_POWER_STATE_COUNT   2

/*
 * PW_DEVICE_FLAGS:
 * ----------------
 * The encoding of PW_DEVICE_FLAGS is illustrated below.
 * Note that a set bit (1) indicates the property is TRUE (e.g.
 * if bit 0 is set then the device definition includes _PSC).
 * +-----------+-+-+-+
 * | Bits 31:3 |2|1|0|    
 * +-----------+-+-+-+
 *       |      | | |
 *       |      | | +- _PSC Present?
 *       |      | +--- _IRC Present?
 *       |      +----- _PRW/_PSW Present?
 *       +------------ <Reserved>
 */
typedef UINT32                          PW_DEVICE_FLAGS;

#define PW_FLAGS_UNKNOWN                ((PW_DEVICE_FLAGS) 0x00000000)

#define PW_FLAGS_PSC                    ((PW_DEVICE_FLAGS) 0x00000001)
#define PW_FLAGS_IRC                    ((PW_DEVICE_FLAGS) 0x00000002)
#define PW_FLAGS_PRW                    ((PW_DEVICE_FLAGS) 0x00000004)

/*
 * PW_STATE_FLAGS:
 * ---------------
 * The encoding of PW_STATE_FLAGS is illustrated below.
 * Note that a set bit (1) indicates the property is TRUE (e.g.
 * if bit 0 is set then the device supports D0).
 * +-----------+-+-+-+-+
 * | Bits 31:4 |3|2|1|0|    
 * +-----------+-+-+-+-+
 *       |      | | | |
 *       |      | | | +- D0 Supported?
 *       |      | | +--- D1 Supported?
 *       |      | +----- D2 Supported?
 *       |      +------- D3 Supported?
 *       +-------------- <Reserved>
 */
typedef UINT32                          PW_STATE_FLAGS;

#define PW_FLAGS_D0                     ((PW_STATE_FLAGS) 0x00000001)
#define PW_FLAGS_D1                     ((PW_STATE_FLAGS) 0x00000002)
#define PW_FLAGS_D2                     ((PW_STATE_FLAGS) 0x00000004)
#define PW_FLAGS_D3                     ((PW_STATE_FLAGS) 0x00000008)

/*
 * PW_DEVICE_INFO:
 * ---------------
 * This structure holds all device-specific power management information.
 */
typedef struct
{
    BM_HANDLE           DeviceHandle;
    ACPI_HANDLE         AcpiHandle;
    PW_POWER_STATE      PowerState;
    PW_DEVICE_FLAGS     Flags;
    UINT32              DxSupported[PW_SYSTEM_POWER_STATE_COUNT];
} PW_DEVICE_INFO;


/*
 * PW_RESOURCE_INFO:
 * -----------------
 * This structure holds all resource-specific power management information.
 */
typedef struct
{
    BM_HANDLE           DeviceHandle;
    ACPI_HANDLE         AcpiHandle;
    PW_POWER_STATE      SystemLevel;
    UINT32              ResourceOrder;
    PW_POWER_STATE      PowerState;
    UINT32              ReferenceCount;
} PW_RESOURCE_INFO;



/* power.h */

ACPI_STATUS
PwInitialize(void);

ACPI_STATUS
PwTerminate(void);

/* pwresource.h */

ACPI_STATUS
PwResourcePrint(
    PW_RESOURCE_INFO        *PowerResource);

ACPI_STATUS
PwResourceGetInfo(
    PW_RESOURCE_INFO        *PowerResource,
    ACPI_BUFFER             *Buffer);

ACPI_STATUS
PwResourceListGetPowerState(
    BM_HANDLE_LIST          *PowerResourceList,
    PW_POWER_STATE          *PowerState);

ACPI_STATUS
PwResourceListTransition(
    PW_DEVICE_INFO          *PowerDevice,
    BM_HANDLE_LIST          *CurrentResourceList,
    BM_HANDLE_LIST          *TargetResourceList);

ACPI_STATUS
PwResourceGetPowerState(
    BM_HANDLE               DeviceHandle,
    PW_POWER_STATE          *PowerState);

ACPI_STATUS
PwResourceSetPowerState(
    PW_RESOURCE_INFO        *PowerResource,
    PW_POWER_STATE          TargetPowerState);

ACPI_STATUS
PwResourceAdd(
    BM_HANDLE               DeviceHandle,
    void                    **Context);

ACPI_STATUS
PwResourceRemove(
    void                    **Context);

ACPI_STATUS
PwResourceInitialize(void);

ACPI_STATUS
PwResourceTerminate(void);

ACPI_STATUS 
PwResourceNotify (
    BM_NOTIFY               NotifyType,
    BM_HANDLE               DeviceHandle,
    void                    **Context);

ACPI_STATUS
PwResourceRequest (
    BM_REQUEST_INFO         *RequestInfo,
    void                    *Context);

/* pwdevice.h */

ACPI_STATUS
PwDevicePrint(
    PW_DEVICE_INFO          *Device);

ACPI_STATUS
PwDeviceGetInfo(
    PW_DEVICE_INFO          *Device,
    ACPI_BUFFER             *Buffer);

ACPI_STATUS
PwDeviceGetPowerState(
    PW_DEVICE_INFO          *Device,
    PW_POWER_STATE          *PowerState);

ACPI_STATUS
PwDeviceSetPowerState(
    PW_DEVICE_INFO          *Device,
    PW_POWER_STATE          CurrentPowerState,
    PW_POWER_STATE          TargetPowerState);

ACPI_STATUS
PwDeviceAdd(
    BM_HANDLE               DeviceHandle,
    void                    **Context);

ACPI_STATUS
PwDeviceRemove(
    void                    **Context);

ACPI_STATUS
PwDeviceInitialize(void);

ACPI_STATUS
PwDeviceTerminate(void);

ACPI_STATUS 
PwDeviceNotify (
    BM_NOTIFY               NotifyType,
    BM_HANDLE               DeviceHandle,
    void                    **Context);

ACPI_STATUS
PwDeviceRequest (
    BM_REQUEST_INFO         *RequestInfo,
    void                    *Context);


#endif  /* __POWER_H__ */
