/*
 *  ./acpica/control/power/pwresource.c - <TBD>
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __PWRESOURCE_C__

// TODO: remove...
#include <KernelExport.h>

#include <acpi.h>

#include <utility.h>

#include "power.h"


#define _COMPONENT          POWER_CONTROL
        MODULE_NAME         ("pwresource");






ACPI_STATUS
PwResourcePrint(
    PW_RESOURCE_INFO        *PowerResource)
{
    ACPI_BUFFER             Buffer;

    if (!PowerResource)
    {
        return(AE_BAD_PARAMETER);
    }

    Buffer.Length = 256;
    Buffer.Pointer = OsdCallocate(Buffer.Length);
    if (!Buffer.Pointer)
    {
        return(AE_NO_MEMORY);
    }

    AcpiGetName(PowerResource->AcpiHandle, ACPI_FULL_PATHNAME, &Buffer);

    OsdPrintf("POWER RESOURCE: Handle[0x%08X] AcpiHandle[0x%08X] Name[%s] \n", PowerResource->DeviceHandle, PowerResource->AcpiHandle, Buffer.Pointer);
    OsdPrintf("  SystemLevel[%d] ResourceOrder[%d] PowerState[%d] ReferenceCount[%d]\n", PowerResource->SystemLevel, PowerResource->ResourceOrder, PowerResource->PowerState, PowerResource->ReferenceCount);

    OsdFree(Buffer.Pointer);

    return(AE_OK);
}



ACPI_STATUS
PwResourceGetInfo(
    PW_RESOURCE_INFO        *PowerResource,
    ACPI_BUFFER             *Buffer)
{
    ACPI_STATUS             Status = AE_OK;

    FUNCTION_TRACE("PwResourceGetInfo");

    if (!PowerResource || !Buffer)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    Status = UtCopyToBuffer(Buffer, PowerResource, sizeof(PW_RESOURCE_INFO));

    return_ACPI_STATUS(Status);
}



ACPI_STATUS
PwResourceListGetPowerState(
    BM_HANDLE_LIST          *PowerResourceList,
    PW_POWER_STATE          *PowerState)
{
    ACPI_STATUS             Status = AE_OK;
    UINT32                  i = 0;

    FUNCTION_TRACE("PwResourceListGetPowerState");

    if (!PowerResourceList || !PowerState)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    (*PowerState) = PW_POWER_STATE_UNKNOWN;

    /*
     * Calculate Current PowerState:
     * -----------------------------
     * The current state of a list of power resources is PW_RESOURCE_POWER_STATE_ON
     * if all power resources are currently in the ON state.  In other words,
     * if any power resource in the list is OFF then the collection isn't 
     * fully ON.
     */
    for (i = 0; i < PowerResourceList->Count; i++)
    {
        if (ACPI_FAILURE(Status = PwResourceGetPowerState(PowerResourceList->Handles[i], PowerState)))
        {
            (*PowerState) = PW_POWER_STATE_UNKNOWN;
            break;
        }

        if (*PowerState != PW_RESOURCE_POWER_STATE_ON)
        {
            break;
        }
    }

    return_ACPI_STATUS(Status);
}



ACPI_STATUS
PwResourceListTransition(
    PW_DEVICE_INFO          *PowerDevice,
    BM_HANDLE_LIST          *CurrentResourceList,
    BM_HANDLE_LIST          *TargetResourceList)
{
    ACPI_STATUS             Status = AE_OK;
    PW_RESOURCE_INFO        *PowerResource = NULL;
    UINT32                  i = 0;

    FUNCTION_TRACE("PwResourceListTransition");

    if (!PowerDevice || !CurrentResourceList || !TargetResourceList)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    /*
     * Reference Target:
     * -----------------
     * Reference all resources for the target power state first (so the
     * device doesn't get turned off while transitioning).  Power resources
     * that aren't on (new reference count of 1) are turned on.
     */
    for (i = 0; i < TargetResourceList->Count; i++)
    {
        if (ACPI_SUCCESS(BmGetDeviceContext(TargetResourceList->Handles[i], (void**)(&PowerResource))))
        {
            if (++PowerResource->ReferenceCount == 1)
            {
                // TODO: Ordered _ON...
                if (ACPI_FAILURE(Status = PwResourceSetPowerState(PowerResource, PW_RESOURCE_POWER_STATE_ON)))
                {
                    // TODO: Punt or continue?
                    DEBUG_PRINT(ACPI_ERROR, ("Unable to change power state for power resource [0x%08X].\n", TargetResourceList->Handles[i]));
                }
            }

            // TODO: Remove
            PwResourcePrint(PowerResource);
        }
        else
        {
            // TODO: Remove
            PwResourcePrint(PowerResource);
            // TODO: Punt or continue?
            DEBUG_PRINT(ACPI_ERROR, ("Invalid power resource entry [0x%08X].\n", TargetResourceList->Handles[i]));
        }
    }

    /*
     * Dereference Current:
     * --------------------
     * Dereference all resources for the current power state.  Power 
     * resources no longer referenced (new reference count of 0) are turned 
     * off.
     */
    for (i = 0; i < CurrentResourceList->Count; i++)
    {
        if (ACPI_SUCCESS(BmGetDeviceContext(CurrentResourceList->Handles[i], (void**)(&PowerResource))))
        {
            if (--PowerResource->ReferenceCount == 0)
            {
                // TODO: Ordered _OFF...
                if (ACPI_FAILURE(Status = PwResourceSetPowerState(PowerResource, PW_RESOURCE_POWER_STATE_OFF)))
                {
                    // TODO: Punt or continue?
                    DEBUG_PRINT(ACPI_ERROR, ("Unable to change power state for power resource [0x%08X].\n", CurrentResourceList->Handles[i]));
                }
            }

            // TODO: Remove
            PwResourcePrint(PowerResource);
        }
        else
        {
            // TODO: Remove
            PwResourcePrint(PowerResource);
            // TODO: Punt or continue?
            DEBUG_PRINT(ACPI_ERROR, ("Invalid power resource entry [0x%08X].\n", CurrentResourceList->Handles[i]));
        }
    }

    return_ACPI_STATUS(Status);
}



ACPI_STATUS
PwResourceGetPowerState(
    BM_HANDLE               DeviceHandle,
    PW_POWER_STATE          *PowerState)
{
    ACPI_STATUS             Status = AE_OK;
    BM_DEVICE_STATUS        DeviceStatus = BM_STATUS_UNKNOWN;

    FUNCTION_TRACE("PwResourceGetPowerState");

    if (!PowerState)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    if (ACPI_FAILURE(Status = BmGetDeviceStatus(DeviceHandle, &DeviceStatus)))
    {
        (*PowerState) = PW_POWER_STATE_UNKNOWN;
        return_ACPI_STATUS(Status);
    }

    switch (DeviceStatus)
    {
    case 0:
        (*PowerState) = PW_RESOURCE_POWER_STATE_OFF;
        break;

    case 1:
        (*PowerState) = PW_RESOURCE_POWER_STATE_ON;
        break;

    default:
        (*PowerState) = PW_POWER_STATE_UNKNOWN;
        DEBUG_PRINT(ACPI_WARN, ("Invalid status [0x%08X] returned for power resource [0x%08X].", DeviceStatus, DeviceHandle));
        break;
    }

    return_ACPI_STATUS(Status);
}



ACPI_STATUS
PwResourceSetPowerState(
    PW_RESOURCE_INFO        *PowerResource,
    PW_POWER_STATE          TargetPowerState)
{
    ACPI_STATUS             Status = AE_OK;
    PW_POWER_STATE          CurrentPowerState = PW_POWER_STATE_UNKNOWN;

    FUNCTION_TRACE("PwResourceSetPowerState");

    if (!PowerResource)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    if (ACPI_FAILURE(Status = PwResourceGetPowerState(PowerResource->DeviceHandle, &CurrentPowerState)))
    {
        return_ACPI_STATUS(Status);
    }

    if (TargetPowerState == CurrentPowerState)
    {
        DEBUG_PRINT(ACPI_INFO, ("Power resource [0x%08X] already at target power state.\n", PowerResource->DeviceHandle));
        return_ACPI_STATUS(AE_OK);
    }

    switch (TargetPowerState)
    {
    case PW_RESOURCE_POWER_STATE_ON:
        DEBUG_PRINT(ACPI_INFO, ("Turning power resource [0x%08X] ON.\n", PowerResource->DeviceHandle));
        Status = AcpiEvaluateObject(PowerResource->AcpiHandle, "_ON", NULL, NULL);
        break;

    case PW_RESOURCE_POWER_STATE_OFF:
        DEBUG_PRINT(ACPI_INFO, ("Turning power resource [0x%08X] OFF.\n", PowerResource->DeviceHandle));
        Status = AcpiEvaluateObject(PowerResource->AcpiHandle, "_OFF", NULL, NULL);
        break;

    default:
        Status = AE_BAD_PARAMETER;
        break;
    }

    return_ACPI_STATUS(Status);
}



ACPI_STATUS
PwResourceCalculateInfo(
    PW_RESOURCE_INFO        *PowerResource)
{
    ACPI_STATUS             Status = AE_OK;

    FUNCTION_TRACE("PwResourceCalculateInfo");

    if (!PowerResource)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    // TODO: Need API in core to expose this information?

    PowerResource->SystemLevel = PW_POWER_STATE_UNKNOWN;
    PowerResource->ResourceOrder = 0;
    PowerResource->PowerState = PW_POWER_STATE_UNKNOWN;
    //PowerResource->References = 0;

    Status = PwResourceGetPowerState(PowerResource->DeviceHandle, &(PowerResource->PowerState));

    return_ACPI_STATUS(Status);
}



ACPI_STATUS
PwResourceAdd(
    BM_HANDLE               DeviceHandle,
    void                    **Context)
{
    ACPI_STATUS             Status = AE_OK;
    PW_RESOURCE_INFO        *PowerResource = NULL;
    BM_DEVICE_INFO          *DeviceInfo = NULL;

    FUNCTION_TRACE("PwResourceAdd");

    DEBUG_PRINT(ACPI_INFO, ("PwResourceAdd: Power resource [0x%08X].\n", DeviceHandle));

    if (!Context || *Context)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    /*
     * Get details on this device...
     */
    if (ACPI_FAILURE(Status = BmGetDeviceInfo(DeviceHandle, &DeviceInfo)))
    {
        return_ACPI_STATUS(Status);
    }

    /*
     * Allocate a new PW_RESOURCE_INFO structure.
     */
    PowerResource = OsdCallocate(sizeof(PW_RESOURCE_INFO));
    if (!PowerResource)
    {
        return_ACPI_STATUS(AE_NO_MEMORY);
    }

    PowerResource->DeviceHandle = DeviceInfo->DeviceHandle;
    PowerResource->AcpiHandle = DeviceInfo->AcpiHandle;

    /*
     * Get Information on this power resource...
     */
    Status = PwResourceCalculateInfo(PowerResource);
    
    if (ACPI_FAILURE(Status))
    {
        OsdFree(PowerResource);
    }
    else
    {
        *Context = PowerResource;
    }

    return_ACPI_STATUS(AE_OK);
}



ACPI_STATUS
PwResourceRemove(
    void                    **Context)
{
    ACPI_STATUS             Status = AE_OK;
    PW_RESOURCE_INFO        *PowerResource = NULL;

    FUNCTION_TRACE("PwResourceRemove");

    if (!Context || !*Context)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    /*
     * Context contains information specific to this power resource.
     */ 
    PowerResource = (PW_RESOURCE_INFO*)*Context;

    DEBUG_PRINT(ACPI_INFO, ("PwResourceRemove: Power resource [0x%08X].\n", PowerResource->DeviceHandle));

    OsdFree(PowerResource);

    return_ACPI_STATUS(Status);
}




ACPI_STATUS
PwResourceInitialize(void)
{
    ACPI_STATUS             Status = AE_OK;
    BM_DEVICE_INFO          Criteria;
    BM_CONTROL_INFO         Control;

    FUNCTION_TRACE("PwResourceInitialize");

    MEMSET(&Criteria, 0, sizeof(BM_DEVICE_INFO));
    MEMSET(&Control, 0, sizeof(BM_CONTROL_INFO));

    Criteria.Class = BM_CLASS_POWER_RESOURCE;

    Control.Notify = &PwResourceNotify;
    Control.Request = &PwResourceRequest;

    Status = BmRegisterControl(&Criteria, &Control);

    return_ACPI_STATUS(Status);
}



ACPI_STATUS
PwResourceTerminate(void)
{
    ACPI_STATUS             Status = AE_OK;
    BM_DEVICE_INFO          Criteria;
    BM_CONTROL_INFO         Control;

    FUNCTION_TRACE("PwResourceTerminate");

    MEMSET(&Criteria, 0, sizeof(BM_DEVICE_INFO));
    MEMSET(&Control, 0, sizeof(BM_CONTROL_INFO));

    Criteria.Class = BM_CLASS_POWER_RESOURCE;

    Control.Notify = &PwResourceNotify;
    Control.Request = &PwResourceRequest;

    Status = BmUnregisterControl(&Criteria, &Control);

    return_ACPI_STATUS(Status);
}


ACPI_STATUS 
PwResourceNotify (
    BM_NOTIFY               NotifyType,
    BM_HANDLE               DeviceHandle,
    void                    **Context)
{
    ACPI_STATUS             Status = AE_OK;

    FUNCTION_TRACE("PwResourceNotify");

    switch (NotifyType)
    {
    case BM_NOTIFY_DEVICE_ADDED:
        Status = PwResourceAdd(DeviceHandle, Context);
        break;

    case BM_NOTIFY_DEVICE_REMOVED:
        Status = PwResourceRemove(Context);
        break;

    default:
        Status = AE_SUPPORT;
        break;
    }

    return_ACPI_STATUS(Status);
}



ACPI_STATUS
PwResourceRequest (
    BM_REQUEST_INFO         *RequestInfo,
    void                    *Context)
{
    ACPI_STATUS             Status = AE_OK;
    PW_RESOURCE_INFO        *PowerResource = NULL;

    FUNCTION_TRACE("PwResourceRequest");

    /*
     * Must have a valid request structure and Context.
     */
    if (!RequestInfo || !Context)
    {
        return_ACPI_STATUS(AE_BAD_PARAMETER);
    }

    /*
     * Context contains information specific to this power resource.
     */ 
    PowerResource = (PW_RESOURCE_INFO*)Context;

    /*
     * Handle Request:
     * ---------------
     */
    switch (RequestInfo->Command)
    {
    case PW_COMMAND_GET_INFO:
        Status = PwResourceGetInfo(PowerResource, &(RequestInfo->Buffer));
        break;

    case PW_COMMAND_GET_POWER_STATE:
        {
            PW_POWER_STATE PowerState = PW_POWER_STATE_UNKNOWN;

            if (ACPI_SUCCESS(Status = PwResourceGetPowerState(PowerResource->DeviceHandle, &PowerState)))
            {
                Status = UtCopyToBuffer(&(RequestInfo->Buffer), &PowerState, sizeof(PW_POWER_STATE));
            }
        }
        break;

    case PW_COMMAND_SET_POWER_STATE:
        {
            PW_POWER_STATE *PowerState = NULL;

            if (ACPI_SUCCESS(Status = UtCastBuffer(&(RequestInfo->Buffer), (void**)&PowerState, sizeof(PW_POWER_STATE))))
            {
                Status = PwResourceSetPowerState(PowerResource, *PowerState);
            }
        }
        break;

    default:
        Status = AE_SUPPORT;
        break;
    }

    RequestInfo->Status = Status;

    return_ACPI_STATUS(Status);
}


