/*
 *  ./acpica/control/processor/processor.h - 
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef __PROCESSOR_H__
#define __PROCESSOR_H__

#include <busmgr.h>



// TODO: Need UINT64 for PMTimer/Microsecond conversions...
typedef unsigned long long    UINT64;

#define PR_MAX_POWER_STATES             4
#define PR_MAX_PERFORMANCE_STATES       8
#define PR_MAX_THROTTLING_STATES        8

/* 
 * Processor Commands:
 * -------------------
 */
#define PR_COMMAND_GET_INFO             ((BM_COMMAND) 0x80)
#define PR_COMMAND_SET_CX_STATE_INFO    ((BM_COMMAND) 0x81)
#define PR_COMMAND_GET_THROTTLING_STATE ((BM_COMMAND) 0x82)
#define PR_COMMAND_SET_THROTTLING_STATE ((BM_COMMAND) 0x83)
#define PR_COMMAND_GET_PERF_STATE       ((BM_COMMAND) 0x84)
#define PR_COMMAND_SET_PERF_STATE       ((BM_COMMAND) 0x85)
#define PR_COMMAND_GET_CURRENT_FREQ     ((BM_COMMAND) 0x86)

/*
 * PR_POWER_STATE:
 * ---------------
 */
typedef UINT32                          PR_POWER_STATE;

#define PR_POWER_STATE_UNKNOWN          ((PR_POWER_STATE) 0xFFFFFFFF)

#define PR_POWER_STATE_C0               ((PR_POWER_STATE) 0x00000000)
#define PR_POWER_STATE_C1               ((PR_POWER_STATE) 0x00000001)
#define PR_POWER_STATE_C2               ((PR_POWER_STATE) 0x00000002)
#define PR_POWER_STATE_C3               ((PR_POWER_STATE) 0x00000003)

/* 
 * Processor Notifications:
 * ------------------------
 */
#define PR_NOTIFY_PERF_STATES_CHANGE    ((BM_NOTIFY) 0x80)
#define PR_NOTIFY_POWER_STATES_CHANGE   ((BM_NOTIFY) 0x81)


typedef struct
{
 UINT32     TimeThreshold;
 UINT32     CountThreshold;
 UINT32     Count;
 PR_POWER_STATE   TargetState;
} PR_POLICY_VALUES;
 
/* 
 * PR_CX_STATE_INFO:
 * -----------------
 */
typedef struct
{
    UINT32                  Latency;
 UINT64     Utilization;
    PR_POLICY_VALUES        PromotionPolicy;
    PR_POLICY_VALUES        DemotionPolicy;
} PR_CX_STATE_INFO;

/* 
 * PR_POWER_INFO:
 * --------------
 */
typedef struct
{
    UINT32                  Count;
 PR_POWER_STATE   ActiveState;
 PR_CX_STATE_INFO  Info[PR_MAX_POWER_STATES];
} PR_POWER_INFO;

/* 
 * PR_PERFORMANCE_INFO:
 * --------------------
 */
typedef struct
{
    UINT32                  Count;
 // TODO...
} PR_PERFORMANCE_INFO;

/* 
 * PR_THROTTLING_INFO:
 * -------------------
 */
typedef struct
{
    UINT32                  Count;
    UINT32                  Percentage[PR_MAX_THROTTLING_STATES];
} PR_THROTTLING_INFO;

/* 
 * PR_INFO:
 * --------
 */
typedef struct
{
    BM_HANDLE               DeviceHandle;
    ACPI_HANDLE             AcpiHandle;
 PR_POWER_INFO   PowerStates;
    PR_PERFORMANCE_INFO     PerformanceStates;
    PR_THROTTLING_INFO      ThrottlingStates;
} PR_INFO;



/* processor.c */

ACPI_STATUS
PrInitialize(void);

ACPI_STATUS
PrTerminate(void);

ACPI_STATUS 
PrNotify (
    BM_NOTIFY               NotifyType,
    BM_HANDLE               DeviceHandle,
    void                    **Context);

ACPI_STATUS
PrRequest(
    BM_REQUEST_INFO         *RequestInfo,
    void                    *Context);

/* prpolicy.c */

ACPI_STATUS
PrPolicyInitialize(
 PR_INFO                 *Processor);

ACPI_STATUS
PrPolicyTerminate(
 PR_INFO                 *Processor);


#endif  /* __PROCESSOR_H__ */
