
/*
 *  ./acpica/subsystem/common/cmglobal.c - Global variables for the ACPI subsystem
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __CMGLOBAL_C__
#define DEFINE_ACPI_GLOBALS

#include <acpi.h>
#include <events.h>
#include <namesp.h>
#include <interp.h>


#define _COMPONENT          MISCELLANEOUS
        MODULE_NAME         ("cmglobal");




/* 
 * We want the debug switches statically initialized so they 
 * are already set when the debugger is entered.
 */

    /* Debug switch - level and trace mask */

#ifdef ACPI_DEBUG
UINT32                      DebugLevel = DEBUG_DEFAULT;
#else
UINT32                      DebugLevel = NORMAL_DEFAULT;
#endif

    /* Debug switch - layer (component) mask */

UINT32                      DebugLayer = ALL_COMPONENTS;
UINT32                      Gbl_NestingLevel = 0;


/* System flags */

UINT32                      Gbl_SystemFlags = 0;
UINT32                      Gbl_StartupFlags = 0;
BOOLEAN      Gbl_Shutdown = TRUE;  /* System starts unitialized! */





/* 
 * Names built-in to the interpreter
 *
 * Initial values are currently supported only for types String and Number.
 * To avoid type punning, both are specified as strings in this table.
 */

PREDEFINED_NAMES            Gbl_PreDefinedNames[] = 
{
    {"_GPE",    INTERNAL_TYPE_DefAny},
    {"_PR_",    INTERNAL_TYPE_DefAny},
    {"_SB_",    INTERNAL_TYPE_DefAny},
    {"_SI_",    INTERNAL_TYPE_DefAny},
    {"_TZ_",    INTERNAL_TYPE_DefAny},
    {"_REV",    ACPI_TYPE_Number, "2"},
    {"_OS_",    ACPI_TYPE_String, ACPI_OS_NAME},
    {"_GL_",    ACPI_TYPE_Mutex, "0"},

    /* Table terminator */

    {NULL,      ACPI_TYPE_Any}
};


/* 
 * Properties of the ACPI Object Types, both internal and external.
 * 
 * Elements of NsProperties are bit significant
 * and the table is indexed by values of ACPI_OBJECT_TYPE
 */

UINT8                       Gbl_NsProperties[] = 
{
    NSP_NORMAL,                 /* 00 Any              */
    NSP_NORMAL,                 /* 01 Number           */
    NSP_NORMAL,                 /* 02 String           */
    NSP_NORMAL,                 /* 03 Buffer           */
    NSP_LOCAL,                  /* 04 Package          */
    NSP_NORMAL,                 /* 05 FieldUnit        */
    NSP_NEWSCOPE | NSP_LOCAL,   /* 06 Device           */
    NSP_LOCAL,                  /* 07 Event            */
    NSP_NEWSCOPE | NSP_LOCAL,   /* 08 Method           */
    NSP_LOCAL,                  /* 09 Mutex            */
    NSP_LOCAL,                  /* 10 Region           */
    NSP_NEWSCOPE | NSP_LOCAL,   /* 11 Power            */
    NSP_NEWSCOPE | NSP_LOCAL,   /* 12 Processor        */
    NSP_NEWSCOPE | NSP_LOCAL,   /* 13 Thermal          */
    NSP_NORMAL,                 /* 14 BufferField      */
    NSP_NORMAL,                 /* 15 DdbHandle        */
    NSP_NORMAL,                 /* 16 reserved         */
    NSP_NORMAL,                 /* 17 reserved         */
    NSP_NORMAL,                 /* 18 reserved         */
    NSP_NORMAL,                 /* 19 reserved         */
    NSP_NORMAL,                 /* 20 reserved         */
    NSP_NORMAL,                 /* 21 reserved         */
    NSP_NORMAL,                 /* 22 reserved         */
    NSP_NORMAL,                 /* 23 reserved         */
    NSP_NORMAL,                 /* 24 reserved         */
    NSP_NORMAL,                 /* 25 DefField         */
    NSP_NORMAL,                 /* 26 BankField        */
    NSP_NORMAL,                 /* 27 IndexField       */
    NSP_NORMAL,                 /* 28 DefFieldDefn     */
    NSP_NORMAL,                 /* 29 BankFieldDefn    */
    NSP_NORMAL,                 /* 30 IndexFieldDefn   */
    NSP_NORMAL,                 /* 31 If               */
    NSP_NORMAL,                 /* 32 Else             */
    NSP_NORMAL,                 /* 33 While            */
    NSP_NEWSCOPE,               /* 34 Scope            */
    NSP_LOCAL,                  /* 35 DefAny           */
    NSP_NORMAL,                 /* 36 Reference           */
    NSP_NORMAL,                 /* 37 Alias            */
    NSP_NORMAL,                 /* 38 Notify           */
    NSP_NORMAL,                 /* 39 Address Handler  */
    NSP_NORMAL                  /* 40 Invalid          */
};




ACPI_TABLE_DESC             Gbl_AcpiTables[NUM_ACPI_TABLES];


ACPI_TABLE_SUPPORT          Gbl_AcpiTableData[NUM_ACPI_TABLES] =
{
                 /* Name,   Signature,  Signature size,    How many allowed?,   Supported?  Global typed pointer */

    /* RSDP 0 */ {"RSDP",   RSDP_SIG, sizeof (RSDP_SIG)-1, ACPI_TABLE_SINGLE,   AE_OK,      NULL},
    /* APIC 1 */ {APIC_SIG, APIC_SIG, sizeof (APIC_SIG)-1, ACPI_TABLE_SINGLE,   AE_OK,      (void **) &Gbl_APIC},
    /* DSDT 2 */ {DSDT_SIG, DSDT_SIG, sizeof (DSDT_SIG)-1, ACPI_TABLE_SINGLE,   AE_OK,      (void **) &Gbl_DSDT},
    /* FACP 3 */ {FACP_SIG, FACP_SIG, sizeof (FACP_SIG)-1, ACPI_TABLE_SINGLE,   AE_OK,      (void **) &Gbl_FACP},
    /* FACS 4 */ {FACS_SIG, FACS_SIG, sizeof (FACS_SIG)-1, ACPI_TABLE_SINGLE,   AE_OK,      (void **) &Gbl_FACS},
    /* PSDT 5 */ {PSDT_SIG, PSDT_SIG, sizeof (PSDT_SIG)-1, ACPI_TABLE_MULTIPLE, AE_OK,      NULL},
    /* RSDT 6 */ {RSDT_SIG, RSDT_SIG, sizeof (RSDT_SIG)-1, ACPI_TABLE_SINGLE,   AE_OK,      NULL},
    /* SSDT 7 */ {SSDT_SIG, SSDT_SIG, sizeof (SSDT_SIG)-1, ACPI_TABLE_MULTIPLE, AE_OK,      NULL},
    /* SBST 8 */ {SBST_SIG, SBST_SIG, sizeof (SBST_SIG)-1, ACPI_TABLE_SINGLE,   AE_OK,      (void **) &Gbl_SBST},
    /* BOOT 9 */ {BOOT_SIG, BOOT_SIG, sizeof (BOOT_SIG)-1, ACPI_TABLE_SINGLE,   AE_SUPPORT, NULL}
};

ACPI_INIT_DATA Gbl_AcpiInitData;




#ifdef ACPI_DEBUG

char                        *MsgAcpiErrorBreak = "*** Break on ACPI_ERROR ***\n";



char *
CmGetMutexName (
    UINT32                  MutexId)
{

    if (MutexId > MAX_MTX)
    {
        return ("Invalid Mutex ID");
    }

    return (Gbl_MutexNames[MutexId]);
}


/* 
 * Elements of Gbl_NsTypeNames below must match
 * one-to-one with values of ACPI_OBJECT_TYPE
 *
 * The type ACPI_TYPE_Any (Untyped) is used as a "don't care" when searching; when 
 * stored in a table it really means that we have thus far seen no evidence to 
 * indicatewhat type is actually going to be stored for this entry.
 */

static char                 Gbl_BadType[] = "UNDEFINED";
#define TYPE_NAME_LENGTH    9                       /* Maximum length of each string */

static char                 *Gbl_NsTypeNames[] =    /* printable names of ACPI types */
{
    "Untyped",
    "Number",
    "String",
    "Buffer",
    "Package",
    "FieldUnit",
    "Device",
    "Event",
    "Method",
    "Mutex",
    "Region",
    "Power",
    "Processor",
    "Thermal",
    "BufferFld",
    "DdbHandle", 
     Gbl_BadType, 
     Gbl_BadType, 
     Gbl_BadType, 
     Gbl_BadType,
     Gbl_BadType, 
     Gbl_BadType, 
     Gbl_BadType, 
     Gbl_BadType, 
     Gbl_BadType,
    "DefField",
    "BnkField",
    "IdxField",
    "DefFldDfn",
    "BnkFldDfn",
    "IdxFldDfn",
    "If",
    "Else",
    "While",
    "Scope",
    "DefAny",
    "Reference",
    "Alias",
    "Notify", 
    "AddrHndlr", 
    "Invalid"
};




char *
CmGetTypeName (
    UINT32                  Type)
{

    if (Type > INTERNAL_TYPE_Invalid)
    {
        return (Gbl_BadType);
    }

    return (Gbl_NsTypeNames[Type]);
}

#endif




BOOLEAN
CmValidObjectType (
    UINT32                  Type)
{
    
    if (Type > ACPI_TYPE_MAX)
    {
        if ((Type < INTERNAL_TYPE_BEGIN) ||
            (Type > INTERNAL_TYPE_MAX))
        {
            return FALSE;
        }
    }

    return TRUE;
}




char *
CmFormatException (
    ACPI_STATUS             Status)
{

    if (Status > ACPI_MAX_STATUS)
    {
        return "UNKNOWN_STATUS";
    }

    return (Gbl_ExceptionNames [Status]);
}



ACPI_OWNER_ID
CmAllocateOwnerId (
    UINT32                  IdType)
{
    ACPI_OWNER_ID           OwnerId = 0xFFFF;


    FUNCTION_TRACE ("CmAllocateOwnerId");


    CmAcquireMutex (MTX_CACHES);

    switch (IdType)
    {
    case OWNER_TYPE_TABLE:

        OwnerId = Gbl_NextTableOwnerId;
        Gbl_NextTableOwnerId++;

        if (Gbl_NextTableOwnerId == FIRST_METHOD_ID)
        {
            Gbl_NextTableOwnerId = FIRST_TABLE_ID;
        }
        break;


    case OWNER_TYPE_METHOD:

        OwnerId = Gbl_NextMethodOwnerId;
        Gbl_NextMethodOwnerId++;

        if (Gbl_NextMethodOwnerId == FIRST_TABLE_ID)
        {
            Gbl_NextMethodOwnerId = FIRST_METHOD_ID;
        }
        break;
    }


    CmReleaseMutex (MTX_CACHES);

    return_VALUE (OwnerId);
}



void 
CmInitGlobals (ACPI_INIT_DATA *InitData)
{
    UINT32                  i;

    FUNCTION_TRACE ("CmInitGlobals");


    if (InitData)
    {
        MEMCPY (&Gbl_AcpiInitData, InitData, sizeof (ACPI_INIT_DATA));
    } 

    else 
    {
        MEMSET (&Gbl_AcpiInitData, 0, sizeof (ACPI_INIT_DATA));
    }

    /* ACPI table structure */

    for (i = 0; i < ACPI_TABLE_MAX; i++)
    {
        Gbl_AcpiTables[i].Prev          = &Gbl_AcpiTables[i];
        Gbl_AcpiTables[i].Next          = &Gbl_AcpiTables[i];
        Gbl_AcpiTables[i].Pointer       = NULL;
        Gbl_AcpiTables[i].Length        = 0;
        Gbl_AcpiTables[i].Allocation    = ACPI_MEM_NOT_ALLOCATED;
        Gbl_AcpiTables[i].Count         = 0;
    }
      

    /* Address Space handler array */

    for (i = 0; i < ACPI_MAX_ADDRESS_SPACE; i++)
    {
        Gbl_AddressSpaces[i].Handler    = NULL;
        Gbl_AddressSpaces[i].Context    = NULL;
    }

    /* Mutex locked flags */

    for (i = 0; i < NUM_MTX; i++)
    {
        Gbl_AcpiMutexInfo[i].Mutex      = NULL;
        Gbl_AcpiMutexInfo[i].Locked     = FALSE;
        Gbl_AcpiMutexInfo[i].UseCount   = 0;
    }

    /* Global notify handlers */

    Gbl_SysNotify.Handler       = NULL;
    Gbl_DrvNotify.Handler       = NULL;

    /* Global "typed" ACPI table pointers */

    Gbl_RSDP                    = NULL;
    Gbl_RSDT                    = NULL;
    Gbl_FACS                    = NULL;
    Gbl_FACP                    = NULL;
    Gbl_APIC                    = NULL;
    Gbl_DSDT                    = NULL;
    Gbl_SBST                    = NULL;


    /* Global Lock support */

    Gbl_GlobalLockAcquired      = FALSE;
    Gbl_GlobalLockThreadCount   = 0;

    /* Miscellaneous variables */

    Gbl_SystemFlags             = 0;
    Gbl_StartupFlags            = 0;
    Gbl_GlobalLockSet           = FALSE;
    Gbl_RsdpOriginalLocation    = 0;
    Gbl_WhenToParseMethods      = METHOD_PARSE_CONFIGURATION;
    Gbl_CmSingleStep            = FALSE;
    Gbl_DbTerminateThreads      = FALSE;
    Gbl_Shutdown                = FALSE;
    Gbl_NsLookupCount           = 0;
    Gbl_PsFindCount             = 0;
    Gbl_AcpiHardwarePresent     = TRUE;
    Gbl_NextTableOwnerId        = FIRST_TABLE_ID;
    Gbl_NextMethodOwnerId       = FIRST_METHOD_ID;
    Gbl_DebuggerConfiguration   = DEBUGGER_THREADING;

    /* Cache of small "state" objects */

    Gbl_GenericStateCache       = NULL;
    Gbl_GenericStateCacheDepth  = 0;
    Gbl_StateCacheRequests      = 0;
    Gbl_StateCacheHits          = 0;

    Gbl_ParseCache              = NULL;
    Gbl_ParseCacheDepth         = 0;
    Gbl_ParseCacheRequests      = 0;
    Gbl_ParseCacheHits          = 0;

    Gbl_ObjectCache             = NULL;
    Gbl_ObjectCacheDepth        = 0;
    Gbl_ObjectCacheRequests     = 0;
    Gbl_ObjectCacheHits         = 0;

    Gbl_WalkStateCache          = NULL;
    Gbl_WalkStateCacheDepth     = 0;
    Gbl_WalkStateCacheRequests  = 0;
    Gbl_WalkStateCacheHits      = 0;

    /* Interpreter */

    Gbl_BufSeq                  = 0;
    Gbl_NamedObjectErr          = FALSE;

    /* Parser */

    Gbl_ParsedNamespaceRoot     = NULL;

    /* Hardware oriented */

    Gbl_Gpe0EnableRegisterSave  = NULL;
    Gbl_Gpe1EnableRegisterSave  = NULL;
    Gbl_OriginalMode            = SYS_MODE_UNKNOWN;   /*  original ACPI/legacy mode   */
    Gbl_GpeRegisters            = NULL;
    Gbl_GpeInfo                 = NULL;

    /* Namespace */

    Gbl_RootObject              = &Gbl_RootObjStruct;

    Gbl_RootObject->DataType    = DESC_TYPE_NTE;
    Gbl_RootObject->Type        = ACPI_TYPE_Any;
    Gbl_RootObject->Fill1       = 0;
    Gbl_RootObject->Name        = * (UINT32 *) NS_ROOT;
    Gbl_RootObject->Scope       = NULL;
    Gbl_RootObject->ParentEntry = NULL;
    Gbl_RootObject->NextEntry   = NULL;
    Gbl_RootObject->PrevEntry   = NULL;
    Gbl_RootObject->Object      = NULL;
    
    /* Memory allocation metrics - compiled out in non-debug mode. */

    INITIALIZE_ALLOCATION_METRICS();

    return_VOID;
}   


