/*
 *  ./acpica/subsystem/debugger/dbfileio.c - Debugger file I/O commands.  These can't usually
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include <acpi.h>
#include <acapi.h>
#include <debugger.h>
#include <namesp.h>
#include <parser.h>
#include <events.h>
#include <tables.h>

#ifdef ACPI_DEBUG

#define _COMPONENT          DEBUGGER
        MODULE_NAME         ("dbfileio");

ACPI_GENERIC_OP         *root;

#ifdef ACPI_APPLICATION
#include <stdio.h>
FILE                    *DebugFile = NULL;
#endif



/* NOTE: this is here for lack of a better place.  It is used in all flavors of the debugger, need LCD file */


INT32
DbMatchArgument (
    char                    *UserArgument,
    ARGUMENT_INFO           *Arguments)
{
    UINT32                  i;


    if (!UserArgument || UserArgument[0] == 0)
    {
        return -1;
    }

    for (i = 0; Arguments[i].Name; i++)
    {
        if (STRSTR (Arguments[i].Name, UserArgument) == Arguments[i].Name)
        {
            return i;
        }
    }

    /* Argument not recognized */

    return -1;
}





void
DbCloseDebugFile (
    void)
{

#ifdef ACPI_APPLICATION

    if (DebugFile)
    {
       fclose (DebugFile);
       DebugFile = NULL;
       OutputToFile = FALSE;
       OsdPrintf ("Debug output file %s closed\n", DebugFilename);
    }
#endif

}



void
DbOpenDebugFile (
    char                    *Name)
{

#ifdef ACPI_APPLICATION

    DbCloseDebugFile ();
    DebugFile = fopen (Name, "w+");
    if (DebugFile)
    {
        OsdPrintf ("Debug output file %s opened\n", Name);
        STRCPY (DebugFilename, Name);
        OutputToFile = TRUE;
    }

#endif
}


#ifdef ACPI_APPLICATION

ACPI_STATUS
DbLoadTable(
    FILE                    *fp, 
    char                    **TablePtr, 
    UINT32                  *TableLength)
{
    ACPI_TABLE_HEADER       TableHeader;
    char                    *AmlPtr;
    UINT32                  AmlLength;


    /* Read the table header */

    if (fread (&TableHeader, 1, sizeof (TableHeader), fp) != sizeof (TableHeader))
    {
        OsdPrintf ("Couldn't read the table header\n");
        return (AE_BAD_SIGNATURE);
    }

    /* Get and validate the table length */

    *TableLength = TableHeader.Length;
    if (!*TableLength)
    {
        OsdPrintf ("Found a table length of zero!\n");
        return (AE_ERROR);
    }

    /* Allocate a buffer for the table */

    *TablePtr = (char *) malloc ((size_t) *TableLength);
    if (!*TablePtr)
    {
        OsdPrintf ("Could not allocate memory for the table (size=0x%X)\n", TableHeader.Length);
        return (AE_NO_MEMORY);
    }
            

    AmlPtr      = *TablePtr + sizeof (TableHeader);
    AmlLength   = *TableLength - sizeof (TableHeader);

    /* Copy the header to the buffer */

    MEMCPY (*TablePtr, &TableHeader, sizeof (TableHeader));

    /* Get the rest of the table */

    if ((UINT32) fread (AmlPtr, 1, (size_t) AmlLength, fp) == AmlLength)
    {
        return AE_OK;
    }

    OsdPrintf ("Error reading the table\n");
    free (*TablePtr);
    *TablePtr = NULL;
    *TableLength = 0;

    return AE_ERROR;
}
#endif



ACPI_STATUS
DbLoadAcpiTable (
    char                    *Filename)
{
#ifdef ACPI_APPLICATION
    FILE                    *fp;
    ACPI_STATUS             Status;
    char                    *TablePtr;
    UINT32                  TableLength;
    ACPI_TABLE_DESC         TableInfo;


    fp = fopen (Filename, "rb");
    if (!fp)
    {
        OsdPrintf ("Could not open file %s\n", Filename);
        return AE_ERROR;
    }

    OsdPrintf ("Loading Acpi table from file %s\n", Filename);
    Status = DbLoadTable (fp, &TablePtr, &TableLength);
    fclose(fp);

    if (ACPI_FAILURE (Status))
    {
        OsdPrintf ("Couldn't get table from the file\n");
        return Status;
    }


    /* Attempt to recognize and install the table */

    TableInfo.Pointer = (ACPI_TABLE_HEADER *) TablePtr;                       /* TBD: Fix redundant parameter */
    Status = TbInstallTable (TablePtr, &TableInfo);
    if (ACPI_FAILURE (Status))
    {
        if (Status == AE_EXIST)
        {
            OsdPrintf ("Table is already installed/loaded\n");
        }
        else
        {
            OsdPrintf ("Could not install table, %s\n", CmFormatException (Status));
        }
        free (TablePtr);
        return Status;
    }


    OsdPrintf ("%s successfully loaded and installed at %p\n", 
                                Gbl_AcpiTableData[TableInfo.Type].Name, TablePtr);

    Gbl_AcpiHardwarePresent = FALSE;

#endif  /* ACPI_APPLICATION */  
    return AE_OK;
}


#endif  /* ACPI_DEBUG */

