/*
 *  ./acpica/subsystem/debugger/dbhistry.c - debugger HISTORY command
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include <acpi.h>
#include <parser.h>
#include <dispatch.h>
#include <amlcode.h>
#include <namesp.h>
#include <parser.h>
#include <events.h>
#include <interp.h>
#include <debugger.h>
#include <tables.h>

#ifdef ACPI_DEBUG

#define _COMPONENT          DEBUGGER
        MODULE_NAME         ("dbhistry");




typedef struct HistoryInfo
{
    char                    Command[80];
    UINT32                  CmdNum;

} HISTORY_INFO;


#define HI_NO_HISTORY       0
#define HI_RECORD_HISTORY   1

#define HISTORY_SIZE        20
HISTORY_INFO                HistoryBuffer[HISTORY_SIZE];
UINT32                      LoHistory = 0;
UINT32                      NumHistory = 0;
UINT32                      NextHistoryIndex = 0;
UINT32                      NextCmdNum = 1;


void
DbAddToHistory (
    char                    *CommandLine)
{


    STRCPY (HistoryBuffer[NextHistoryIndex].Command, CommandLine);
    HistoryBuffer[NextHistoryIndex].CmdNum = NextCmdNum;

    if ((NumHistory == HISTORY_SIZE) &&
        (NextHistoryIndex == LoHistory))
    {
        LoHistory++;
        if (LoHistory >= HISTORY_SIZE)
        {
            LoHistory = 0;
        }
    }

    NextHistoryIndex++;
    if (NextHistoryIndex >= HISTORY_SIZE)
    {
        NextHistoryIndex = 0;
    }


    NextCmdNum++;
    if (NumHistory < HISTORY_SIZE)
    {
        NumHistory++;
    }

}



void
DbDisplayHistory (void)
{
    UINT32                  i;
    UINT32                  HistoryIndex;


    HistoryIndex = LoHistory;
    for (i = 0; i < NumHistory; i++)
    {
        OsdPrintf ("%d  %s\n", HistoryBuffer[HistoryIndex].CmdNum, &HistoryBuffer[HistoryIndex].Command);

        HistoryIndex++;
        if (HistoryIndex >= HISTORY_SIZE)
        {
            HistoryIndex = 0;
        }
    }
}



char *
DbGetFromHistory (
    char                    *CommandNumArg)
{
    UINT32                  i;
    UINT32                  HistoryIndex;
    UINT32                  CmdNum;


    if (CommandNumArg == NULL)
    {
        CmdNum = NextCmdNum - 1;
    }

    else
    {
        CmdNum = STRTOUL (CommandNumArg, NULL, 0);
    }

    HistoryIndex = LoHistory;
    for (i = 0; i < NumHistory; i++)
    {
        if (HistoryBuffer[HistoryIndex].CmdNum == CmdNum)
        {
            return (HistoryBuffer[HistoryIndex].Command);
        }


        HistoryIndex++;
        if (HistoryIndex >= HISTORY_SIZE)
        {
            HistoryIndex = 0;
        }
    }

    OsdPrintf ("Invalid history number: %d\n", HistoryIndex);
    return NULL;
}
    

#endif

