
/*
 *  ./acpica/subsystem/dispatcher/dswload1.c - Dispatcher namespace load callbacks
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __DSWLOAD1_C__

#include <acpi.h>
#include <parser.h>
#include <amlcode.h>
#include <dispatch.h>
#include <interp.h>
#include <namesp.h>
#include <events.h>


#define _COMPONENT          DISPATCHER
        MODULE_NAME         ("dswload2");




ACPI_STATUS
DsLoad1BeginOp (
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *Op)
{
    NAME_TABLE_ENTRY        *NewEntry;
    ACPI_STATUS             Status;
    ACPI_OBJECT_TYPE        DataType; 
    char                    Buffer[8];
    char                    *BufferPtr;



    DEBUG_PRINT (TRACE_DISPATCH, ("Load1BeginOp: Op=%p State=%p\n", Op, WalkState));


    /* We are only interested in opcodes that have an associated name */

    if (!PsIsNamedOp (Op->Opcode))
    {
        return AE_OK;
    }


    /* Check if this object has already been installed in the namespace */

    if (Op->NameTableEntry)
    {
        return AE_OK;
    }

    /* Convert the name to a null-terminated string */

    *(UINT32 *) Buffer = (((ACPI_NAMED_OP *)Op)->Name);
    Buffer[4] = 0;
    BufferPtr = Buffer;


    /* Map the raw opcode into an internal object type */

    DataType = DsMapNamedOpcodeToDataType (Op->Opcode);

    /* Attempt to type a NAME opcode by examining the argument */

    /* TBD: is this the right place to do this? */

    if (Op->Opcode == AML_NameOp)
    {
        if (Op->Value.Arg)
        {
            
            DataType = DsMapOpcodeToDataType ((Op->Value.Arg)->Opcode, NULL);
        }
    }

    DEBUG_PRINT (TRACE_DISPATCH, ("Load1BeginOp: State=%p Op=%p Type=%x\n", WalkState, Op, DataType));


    /* 
     * Enter the named type into the internal namespace.  We enter the name
     * as we go downward in the parse tree.  Any necessary subobjects that involve
     * arguments to the opcode must be created as we go back up the parse tree later.
     */
    Status = NsLookup (WalkState->ScopeInfo, BufferPtr, DataType, IMODE_LoadPass1, 
                                NS_NO_UPSEARCH, WalkState, &(NewEntry));

    if (ACPI_SUCCESS (Status))
    {
        /* 
         * Put the NTE in the "op" object that the parser uses, so we
         * can get it again quickly when this scope is closed
         */
        Op->NameTableEntry = NewEntry;
    }


    return (Status);
}





ACPI_STATUS
DsLoad1EndOp (
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *Op)
{
    ACPI_OBJECT_TYPE        DataType; 



    DEBUG_PRINT (TRACE_DISPATCH, ("Load1EndOp: Op=%p State=%p\n", Op, WalkState));

    /* We are only interested in opcodes that have an associated name */

    if (!PsIsNamedOp (Op->Opcode))
    {
        return AE_OK;
    }

    /* TBD: can this be removed? */

    if (Op->Opcode == AML_ScopeOp)
    {
        DEBUG_PRINT (TRACE_DISPATCH, ("Load1EndOp: ending scope Op=%p State=%p\n", Op, WalkState));
        if (((ACPI_NAMED_OP *)Op)->Name == -1)
        {
            DEBUG_PRINT (ACPI_ERROR, ("Load1EndOp: Un-named scope! Op=%p State=%p\n", Op, WalkState));
            return AE_OK;
        }
    }


    DataType = DsMapNamedOpcodeToDataType (Op->Opcode);

    /* Pop the scope stack */

    if (NsOpensScope (DataType))
    {

        DEBUG_PRINT (TRACE_DISPATCH, ("AmlEndNamespaceScope/%s: Popping scope for Op %p\n",
                                        CmGetTypeName (DataType), Op));
        DsScopeStackPop (WalkState);
    }

    return AE_OK;

}




