
/*
 *  ./acpica/subsystem/dispatcher/dswscope.c - Scope stack manipulation
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __DSWSCOPE_C__

#include <acpi.h>
#include <interp.h>
#include <dispatch.h>


#define _COMPONENT          NAMESPACE
        MODULE_NAME         ("dswscope");


#define STACK_POP(head) head




void
DsScopeStackClear (
 ACPI_WALK_STATE   *WalkState)
{
    ACPI_GENERIC_STATE      *ScopeInfo;


    while (WalkState->ScopeInfo)
    {
        /* Pop a scope off the stack */

     ScopeInfo = WalkState->ScopeInfo;
     WalkState->ScopeInfo = ScopeInfo->Scope.Next;

        DEBUG_PRINT (TRACE_EXEC, ("Popped object type 0x%X\n", ScopeInfo->Common.Value));
        CmDeleteGenericState (ScopeInfo);
    }
}




ACPI_STATUS
DsScopeStackPush (
    NAME_TABLE_ENTRY        *NewScope, 
    ACPI_OBJECT_TYPE        Type,
 ACPI_WALK_STATE   *WalkState)
{
    ACPI_GENERIC_STATE      *ScopeInfo;


    FUNCTION_TRACE ("DsScopeStackPush");


    if (!NewScope)
    {
        /*  invalid scope   */

        REPORT_ERROR ("DsScopeStackPush: null scope passed");
        return_ACPI_STATUS (AE_BAD_PARAMETER);
    }

    /* Make sure object type is valid */

    if (!AmlValidateObjectType (Type))
    {
        REPORT_WARNING ("DsScopeStackPush: type code out of range");
    }


    /* Allocate a new scope object */

    ScopeInfo = CmCreateGenericState ();
    if (!ScopeInfo)
    {
        return_ACPI_STATUS (AE_NO_MEMORY);
    }

    /* Init new scope object */

    ScopeInfo->Scope.Entry  = NewScope;
    ScopeInfo->Common.Value = (UINT16) Type;

    /* Push new scope object onto stack */

    CmPushGenericState (&WalkState->ScopeInfo, ScopeInfo);

    return_ACPI_STATUS (AE_OK);
}



ACPI_STATUS
DsScopeStackPop (
 ACPI_WALK_STATE   *WalkState)
{
    ACPI_GENERIC_STATE      *ScopeInfo;


    FUNCTION_TRACE ("DsScopeStackPop");

    /*
     * Pop scope info object off the stack.
     */

    ScopeInfo = CmPopGenericState (&WalkState->ScopeInfo);
    if (!ScopeInfo)
 {
  return_ACPI_STATUS (AE_STACK_UNDERFLOW);
 }

    DEBUG_PRINT (TRACE_EXEC, ("Popped object type 0x%X\n", ScopeInfo->Common.Value));

    CmDeleteGenericState (ScopeInfo);

 return_ACPI_STATUS (AE_OK);
}



