
/*
 *  ./acpica/subsystem/hardware/hwmode.c - Functions for ACPI<->Legacy mode switching logic
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __HWMODE_C__

#include <acpi.h>
#include <hardware.h>


#define _COMPONENT          HARDWARE
        MODULE_NAME         ("hwmode");



ACPI_STATUS
HwSetMode (
    UINT32                  Mode)
{

    ACPI_STATUS             Status = AE_ERROR;

    FUNCTION_TRACE ("HwSetMode");

    
    if (Mode == SYS_MODE_ACPI)
    {
        /* BIOS should have disabled ALL fixed and GP events */
        
        OsdOut8 (Gbl_FACP->SmiCmd, Gbl_FACP->AcpiEnable);
        DEBUG_PRINT (ACPI_INFO, ("Attempting to enable ACPI mode\n"));
    }

    else if (Mode == SYS_MODE_LEGACY)
    {
        /* 
         * BIOS should clear all fixed status bits and restore fixed event
         * enable bits to default
         */

        OsdOut8 (Gbl_FACP->SmiCmd, Gbl_FACP->AcpiDisable);
        DEBUG_PRINT (ACPI_INFO, ("Attempting to enable Legacy (non-ACPI) mode\n"));
    }

    if (HwGetMode () == Mode)
    {
        DEBUG_PRINT (ACPI_INFO, ("Mode %d successfully enabled\n", Mode));
        Status = AE_OK;
    }
        
    return_ACPI_STATUS (Status);
}



UINT32
HwGetMode (void)
{

    FUNCTION_TRACE ("HwGetMode");

    
    if (HwRegisterIO (ACPI_READ, MTX_LOCK, (INT32)SCI_EN))
    {
        return_VALUE (SYS_MODE_ACPI);
    }
    else
    {
        return_VALUE (SYS_MODE_LEGACY);
    }
}


UINT32
HwGetModeCapabilities (void)
{

    FUNCTION_TRACE ("HwGetModeCapabilities");

    
    if (!(Gbl_SystemFlags & SYS_MODES_MASK))
    {
        if (HwGetMode () == SYS_MODE_LEGACY)
        {   
            /* 
             * Assume that if this call is being made, AcpiInit has been called and
             * ACPI support has been established by the presence of the tables.
             * Therefore since we're in SYS_MODE_LEGACY, the system must support both
             * modes 
             */
            
            Gbl_SystemFlags |= (SYS_MODE_ACPI | SYS_MODE_LEGACY);
        }
        
        else
        {   
            /* TBD!!! this may be unsafe... */

            /* 
             * system is is ACPI mode, so try to switch back to LEGACY to see if
             * it is supported 
             */
            HwSetMode (SYS_MODE_LEGACY);
            
            if (HwGetMode () == SYS_MODE_LEGACY)
            {   
                /* Now in SYS_MODE_LEGACY, so both are supported */
                
                Gbl_SystemFlags |= (SYS_MODE_ACPI | SYS_MODE_LEGACY);
                HwSetMode (SYS_MODE_ACPI);
            }
            
            else
            {   
                /* Still in SYS_MODE_ACPI so this must be an ACPI only system */
                
                Gbl_SystemFlags |= SYS_MODE_ACPI;
            }
        }
    }
    
    return_VALUE (Gbl_SystemFlags & SYS_MODES_MASK);
}
