
/*
 *  ./acpica/subsystem/include/acapi.h - external interfaces to the ACPI subsystem
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#ifndef __ACAPI_H__
#define __ACAPI_H__

#include <actypes.h>
#include <actables.h>

/*
 * Global interfaces
 */

ACPI_STATUS
AcpiInitialize (
    ACPI_INIT_DATA          *InitData);

ACPI_STATUS
AcpiTerminate (
    void);

ACPI_STATUS
AcpiEnable (
 void);

ACPI_STATUS
AcpiDisable (
    void);

ACPI_STATUS
AcpiGetSystemInfo(
    ACPI_BUFFER             *RetBuffer);

ACPI_STATUS
AcpiFormatException (
    ACPI_STATUS             Exception,
    ACPI_BUFFER             *OutBuffer);


/*
 * ACPI table manipulation interfaces
 */

ACPI_STATUS
AcpiLoadFirmwareTables (
    void);

ACPI_STATUS
AcpiLoadTable (
    ACPI_TABLE_HEADER       *TablePtr);

ACPI_STATUS
AcpiUnloadTable (
    ACPI_TABLE_TYPE         TableType);

ACPI_STATUS
AcpiGetTableHeader (
    ACPI_TABLE_TYPE         TableType,
    UINT32                  Instance,
    ACPI_TABLE_HEADER       *OutTableHeader);

ACPI_STATUS
AcpiGetTable (
    ACPI_TABLE_TYPE         TableType,
    UINT32                  Instance,
    ACPI_BUFFER             *RetBuffer);


/*
 * Namespace and name interfaces
 */

ACPI_STATUS
AcpiLoadNamespace (
    void);

ACPI_STATUS
AcpiWalkNamespace (
    ACPI_OBJECT_TYPE        Type, 
    ACPI_HANDLE             StartObject, 
    UINT32                  MaxDepth,
    WALK_CALLBACK           UserFunction, 
    void                    *Context, 
    void *                  *ReturnValue);

ACPI_STATUS 
AcpiGetName (
    ACPI_HANDLE             Handle, 
    UINT32                  NameType,
    ACPI_BUFFER             *RetPathPtr);

ACPI_STATUS 
AcpiGetHandle (
    ACPI_HANDLE             Parent, 
    ACPI_STRING             Pathname, 
    ACPI_HANDLE             *RetHandle);


/* 
 * Object manipulation and enumeration
 */

ACPI_STATUS
AcpiEvaluateObject (
    ACPI_HANDLE             Object, 
    ACPI_STRING             Pathname, 
    ACPI_OBJECT_LIST        *ParameterObjects,
    ACPI_BUFFER             *ReturnObjectBuffer);

ACPI_STATUS
AcpiGetObjectInfo (
    ACPI_HANDLE             Device, 
    ACPI_DEVICE_INFO        *Info);

ACPI_STATUS
AcpiGetNextObject (
    ACPI_OBJECT_TYPE        Type, 
    ACPI_HANDLE             Parent, 
    ACPI_HANDLE             Child,
    ACPI_HANDLE             *OutHandle);

ACPI_STATUS 
AcpiGetType (
    ACPI_HANDLE             Object,
    ACPI_OBJECT_TYPE        *OutType);

ACPI_STATUS
AcpiGetParent (
    ACPI_HANDLE             Object,
    ACPI_HANDLE             *OutHandle);


/*
 * Event handler interfaces
 */

ACPI_STATUS
AcpiInstallFixedEventHandler (
    UINT32                  Event,
    FIXED_EVENT_HANDLER     Handler,
    void                    *Context);
    
ACPI_STATUS
AcpiRemoveFixedEventHandler (
    UINT32                  Event,
    FIXED_EVENT_HANDLER     Handler);

ACPI_STATUS
AcpiInstallNotifyHandler (
    ACPI_HANDLE             Device,
    UINT32                  HandlerType, 
    NOTIFY_HANDLER          Handler, 
    void                    *Context);

ACPI_STATUS
AcpiRemoveNotifyHandler (
    ACPI_HANDLE             Device, 
    UINT32                  HandlerType, 
    NOTIFY_HANDLER          Handler);

ACPI_STATUS
AcpiInstallAddressSpaceHandler (
    ACPI_HANDLE             Device, 
    ACPI_ADDRESS_SPACE_TYPE SpaceId, 
    ADDRESS_SPACE_HANDLER   Handler,
    ADDRESS_SPACE_SETUP     Setup,
    void                    *Context);

ACPI_STATUS
AcpiRemoveAddressSpaceHandler (
    ACPI_HANDLE             Device, 
    ACPI_ADDRESS_SPACE_TYPE SpaceId, 
    ADDRESS_SPACE_HANDLER   Handler);

ACPI_STATUS
AcpiInstallGpeHandler (
    UINT32                  GpeNumber,
    UINT32                  Type,
    GPE_HANDLER             Handler,
    void                    *Context);

ACPI_STATUS
AcpiRemoveGpeHandler (
    UINT32                  GpeNumber, 
    GPE_HANDLER             Handler);

ACPI_STATUS
AcpiEnableEvent (
    UINT32                  Event,
    UINT32                  Type);

ACPI_STATUS
AcpiDisableEvent (
    UINT32                  Event,
    UINT32                  Type);

ACPI_STATUS
AcpiClearEvent (
    UINT32                  Event,
    UINT32                  Type);

ACPI_STATUS
AcpiGetEventStatus (
    UINT32                  Event,
    UINT32                  Type,
    ACPI_EVENT_STATUS       *EventStatus);

#include "event.h"

/*
 * Resource interfaces
 */

ACPI_STATUS
AcpiGetCurrentResources(
    ACPI_HANDLE             DeviceHandle,
    ACPI_BUFFER             *RetBuffer);

ACPI_STATUS
AcpiGetPossibleResources(
    ACPI_HANDLE             DeviceHandle,
    ACPI_BUFFER             *RetBuffer);

ACPI_STATUS
AcpiSetCurrentResources (
    ACPI_HANDLE             DeviceHandle,
    ACPI_BUFFER             *InBuffer);

ACPI_STATUS
AcpiGetIRQRoutingTable  (
    ACPI_HANDLE             BusDeviceHandle, 
    ACPI_BUFFER             *RetBuffer);


/*
 * Hardware (ACPI device) interfaces
 */

ACPI_STATUS
AcpiSetFirmwareWakingVector (
    void                    *PhysicalAddress);

ACPI_STATUS
AcpiGetFirmwareWakingVector (
    void                    **PhysicalAddress);

ACPI_STATUS
AcpiGetProcessorThrottlingInfo (
    ACPI_HANDLE             ProcessorHandle,
    ACPI_BUFFER             *UserBuffer);

ACPI_STATUS
AcpiSetProcessorThrottlingState (
    ACPI_HANDLE             ProcessorHandle,
    UINT32                  ThrottleState);

ACPI_STATUS
AcpiGetProcessorThrottlingState (
    ACPI_HANDLE             ProcessorHandle,
    UINT32                  *ThrottleState);

ACPI_STATUS
AcpiGetProcessorCxInfo (
    ACPI_HANDLE             ProcessorHandle,
    ACPI_BUFFER             *UserBuffer);

ACPI_STATUS
AcpiSetProcessorSleepState (
    ACPI_HANDLE             ProcessorHandle,
    UINT32                  CxState);

ACPI_STATUS
AcpiProcessorSleep (
    ACPI_HANDLE             ProcessorHandle,
 UINT32     *PmTimerTicks);


#endif /* __ACAPI_H__ */
