/*
 *  ./acpica/subsystem/include/actbl32.h - ACPI tables specific to IA32
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef __ACTBL32_H__
#define __ACTBL32_H__




/* IA32 Root System Description Table */

typedef struct  
{                                                   
    ACPI_TABLE_HEADER       header;                 /* Table header */
    void                    *TableOffsetEntry [1];  /* Array of pointers to other */
                                                    /* tables' headers */
} ROOT_SYSTEM_DESCRIPTION_TABLE;


/* IA32 Firmware ACPI Control Structure */

typedef struct  
{                                           
    char                    Signature[4];           /* signature "FACS" */
    UINT32                  Length;                 /* length of structure, in bytes */
    UINT32                  HardwareSignature;      /* hardware configuration signature */
    UINT32                  FirmwareWakingVector;   /* ACPI OS waking vector */
    UINT32                  GlobalLock;             /* Global Lock */
    UINT32_BIT              S4Bios_f        : 1;    /* Indicates if S4BIOS support is present */
    UINT32_BIT              Reserved1       : 31;   /* must be 0 */
    UINT8                   Resverved3 [40];        /* reserved - must be zero */

} FIRMWARE_ACPI_CONTROL_STRUCTURE;


/* IA32 Fixed ACPI Description Table */

typedef struct  
{                                           
    ACPI_TABLE_HEADER       header;                 /* table header */
    ACPI_TBLPTR             FirmwareCtrl;           /* Physical address of FACS */
    ACPI_TBLPTR             Dsdt;                   /* Physical address of DSDT */
    UINT8                   Model;                  /* System Interrupt Model */
    UINT8                   Reserved1;              /* reserved */
    UINT16                  SciInt;                 /* System vector of SCI interrupt */
    IO_ADDRESS              SmiCmd;                 /* Port address of SMI command port */
    UINT8                   AcpiEnable;             /* value to write to smi_cmd to enable ACPI */
    UINT8                   AcpiDisable;            /* value to write to smi_cmd to disable ACPI */
    UINT8                   S4BiosReq;              /* Value to write to SMI CMD to enter S4BIOS state */
    UINT8                   Reserved2;              /* reserved - must be zero */
    IO_ADDRESS              Pm1aEvtBlk;             /* Port address of Power Mgt 1a Event Reg Blk */
    IO_ADDRESS              Pm1bEvtBlk;             /* Port address of Power Mgt 1b Event Reg Blk */
    IO_ADDRESS              Pm1aCntBlk;             /* Port address of Power Mgt 1a Control Reg Blk */
    IO_ADDRESS              Pm1bCntBlk;             /* Port address of Power Mgt 1b Control Reg Blk */
    IO_ADDRESS              Pm2CntBlk;              /* Port address of Power Mgt 2 Control Reg Blk */
    IO_ADDRESS              PmTmrBlk;               /* Port address of Power Mgt Timer Ctrl Reg Blk */
    IO_ADDRESS              Gpe0Blk;                /* Port addr of General Purpose Event 0 Reg Blk */
    IO_ADDRESS              Gpe1Blk;                /* Port addr of General Purpose Event 1 Reg Blk */
    UINT8                   Pm1EvtLen;              /* Byte Length of ports at pm1X_evt_blk */
    UINT8                   Pm1CntLen;              /* Byte Length of ports at pm1X_cnt_blk */
    UINT8                   Pm2CntLen;              /* Byte Length of ports at pm2_cnt_blk */
    UINT8                   PmTmLen;                /* Byte Length of ports at pm_tm_blk */
    UINT8                   Gpe0BlkLen;             /* Byte Length of ports at gpe0_blk */
    UINT8                   Gpe1BlkLen;             /* Byte Length of ports at gpe1_blk */
    UINT8                   Gpe1Base;               /* offset in gpe model where gpe1 events start */
    UINT8                   Reserved3;              /* reserved */
    UINT16                  PLvl2Lat;               /* worst case HW latency to enter/exit C2 state */
    UINT16                  PLvl3Lat;               /* worst case HW latency to enter/exit C3 state */
    UINT16                  FlushSize;              /* Size of area read to flush caches */
    UINT16                  FlushStride;            /* Stride used in flushing caches */
    UINT8                   DutyOffset;             /* bit location of duty cycle field in p_cnt reg */
    UINT8                   DutyWidth;              /* bit width of duty cycle field in p_cnt reg */
    UINT8                   DayAlrm;                /* index to day-of-month alarm in RTC CMOS RAM */
    UINT8                   MonAlrm;                /* index to month-of-year alarm in RTC CMOS RAM */
    UINT8                   Century;                /* index to century in RTC CMOS RAM */
    UINT8                   Reserved4;              /* reserved */
    UINT8                   Reserved4a;             /* reserved */
    UINT8                   Reserved4b;             /* reserved */
    UINT32_BIT              WBInvd          : 1;    /* wbinvd instruction works properly */
    UINT32_BIT              WBInvdFlush     : 1;    /* wbinvd flushes but does not invalidate */
    UINT32_BIT              ProcC1          : 1;    /* all processors support C1 state */
    UINT32_BIT              PLvl2Up         : 1;    /* C2 state works on MP system */
    UINT32_BIT              PwrButton       : 1;    /* Power button is handled as a generic feature */
    UINT32_BIT              SleepButton     : 1;    /* Sleep button is handled as a generic feature, or not present */
    UINT32_BIT              FixedRTC        : 1;    /* RTC wakeup stat not in fixed register space */
    UINT32_BIT              RTCS4           : 1;    /* RTC wakeup stat not possible from S4 */
    UINT32_BIT              TmrValExt       : 1;    /* tmr_val is 32 bits */
    UINT32_BIT              Reserved5       : 23;   /* reserved - must be zero */

}  FIXED_ACPI_DESCRIPTION_TABLE;




#endif /* __ACTBL32_H__ */


