
/*
 *  ./acpica/subsystem/include/common.h - prototypes for the common (subsystem-wide) procedures
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef _COMMON_H
#define _COMMON_H

#include "acobject.h"
#include "acpiosd.h"


#define REF_INCREMENT       (UINT16) 0
#define REF_DECREMENT       (UINT16) 1
#define REF_FORCE_DELETE    (UINT16) 2

/* CmDumpBuffer */

#define DB_BYTE_DISPLAY     1
#define DB_WORD_DISPLAY     2
#define DB_DWORD_DISPLAY    4
#define DB_QWORD_DISPLAY    8


/* Global initialization interfaces */

void 
CmInitGlobals (
    ACPI_INIT_DATA *InitData);

void
CmTerminate (
    void);


/*
 * CmInit - miscellaneous initialization and shutdown
 */

ACPI_STATUS
CmHardwareInitialize (
    void);

ACPI_STATUS
CmSubsystemShutdown (
    void);

/*
 * CmGlobal - Global data structures and procedures
 */

char *
CmGetMutexName (
    UINT32                  MutexId);

char *
CmGetTypeName (
    UINT32                  Type);

BOOLEAN
CmValidObjectType (
    UINT32                  Type);

ACPI_OWNER_ID
CmAllocateOwnerId (
    UINT32                  IdType);


/*
 * CmClib - Local implementations of C library functions
 */

ACPI_SIZE
CmStrlen (
    const char              *String);

char *
CmStrcpy (
    char                    *DstString, 
    const char              *SrcString);

char *
CmStrncpy (
    char                    *DstString, 
    const char              *SrcString, 
    ACPI_SIZE               Count);

UINT32
CmStrncmp (
    const char              *String1, 
    const char              *String2, 
    ACPI_SIZE               Count);

UINT32
CmStrcmp (
    const char              *String1, 
    const char              *String2);

char *
CmStrcat (
    char                    *DstString,
    const char              *SrcString);

char *
CmStrncat (
    char                    *DstString, 
    const char              *SrcString, 
    ACPI_SIZE               Count);

UINT32
CmStrtoul (
    const char              *String, 
    char                    **Terminator, 
    INT32                   Base);

char *
CmStrstr (
    char                    *String1,
    char                    *String2);

char *
CmStrupr (
    char                    *SrcString);

void *
CmMemcpy (
    void                    *Dest, 
    const void              *Src, 
    ACPI_SIZE               Count);

void *
CmMemset (
    void                    *Dest,
    INT32                   Value,
    ACPI_SIZE               Count);

INT32
CmToUpper (
    INT32                   c);

INT32
CmToLower (
    INT32                   c);


/* 
 * CmCopy - Object construction and conversion interfaces
 */

ACPI_STATUS
CmBuildSimpleObject(
    ACPI_OBJECT_INTERNAL    *Obj,
    ACPI_OBJECT             *UserObj,
    char                    *DataSpace,
    UINT32                  *BufferSpaceUsed);

ACPI_STATUS
CmBuildPackageObject (
    ACPI_OBJECT_INTERNAL    *Obj, 
    char                    *Buffer, 
    UINT32                  *SpaceUsed);

ACPI_STATUS
CmBuildExternalObject (
    ACPI_OBJECT_INTERNAL    *Obj, 
    ACPI_BUFFER             *RetBuffer);

ACPI_STATUS
CmBuildInternalSimpleObject(
    ACPI_OBJECT             *UserObj,
    ACPI_OBJECT_INTERNAL    *Obj);

ACPI_STATUS
CmBuildInternalObject (
    ACPI_OBJECT             *Obj, 
    ACPI_OBJECT_INTERNAL    *InternalObj);

ACPI_STATUS
CmCopyInternalSimpleObject (
    ACPI_OBJECT_INTERNAL    *SourceObj,
    ACPI_OBJECT_INTERNAL    *DestObj);

ACPI_STATUS
CmBuildCopyInternalPackageObject (
    ACPI_OBJECT_INTERNAL    *SourceObj, 
    ACPI_OBJECT_INTERNAL    *DestObj);


/*
 * CmCreate - Object creation
 */

ACPI_STATUS
CmUpdateObjectReference (
    ACPI_OBJECT_INTERNAL    *Object,
    UINT16                  Action);

ACPI_OBJECT_INTERNAL *
_CmCreateInternalObject (
    char                    *ModuleName, 
    INT32                   LineNumber, 
    INT32                   ComponentId,
    ACPI_OBJECT_TYPE        Type);


/* 
 * CmDebug - Debug interfaces 
 */

INT32
GetDebugLevel (
    void);

void
SetDebugLevel (
    INT32                   level);

void
FunctionTrace (
    ACPI_STRING             ModuleName, 
    INT32                   LineNumber, 
    INT32                   ComponentId, 
    ACPI_STRING             FunctionName);

void
FunctionTracePtr (
    ACPI_STRING             ModuleName, 
    INT32                   LineNumber, 
    INT32                   ComponentId, 
    ACPI_STRING             FunctionName,
    void                    *Pointer);

void
FunctionTraceU32 (
    ACPI_STRING             ModuleName, 
    INT32                   LineNumber, 
    INT32                   ComponentId, 
    ACPI_STRING             FunctionName,
    UINT32                  Integer);

void
FunctionTraceStr (
    ACPI_STRING             ModuleName, 
    INT32                   LineNumber, 
    INT32                   ComponentId, 
    ACPI_STRING             FunctionName,
    char                    *String);

void
FunctionExit (
    ACPI_STRING             ModuleName, 
    INT32                   LineNumber, 
    INT32                   ComponentId, 
    ACPI_STRING             FunctionName);

void
FunctionStatusExit (
    ACPI_STRING             ModuleName, 
    INT32                   LineNumber, 
    INT32                   ComponentId, 
    ACPI_STRING             FunctionName,
    ACPI_STATUS             Status);

void
FunctionValueExit (
    ACPI_STRING             ModuleName, 
    INT32                   LineNumber, 
    INT32                   ComponentId, 
    ACPI_STRING             FunctionName,
    NATIVE_UINT             Value);

void
FunctionPtrExit (
    ACPI_STRING             ModuleName, 
    INT32                   LineNumber, 
    INT32                   ComponentId, 
    ACPI_STRING             FunctionName,
    char                    *Ptr); 
    
void
DebugPrintPrefix (
    ACPI_STRING             ModuleName, 
    INT32                   LineNumber);

void
DebugPrint (
    ACPI_STRING             ModuleName, 
    INT32                   LineNumber, 
    INT32                   ComponentId, 
    INT32                   PrintLevel, 
    char                    *Format, ...);

void
DebugPrintRaw (
    char                    *Format, ...);

void
_ReportInfo (
    ACPI_STRING             ModuleName, 
    INT32                   LineNumber, 
    INT32                   ComponentId, 
    ACPI_STRING             Message);

void
_ReportError (
    ACPI_STRING             ModuleName, 
    INT32                   LineNumber, 
    INT32                   ComponentId, 
    ACPI_STRING             Message);

void
_ReportWarning (
    ACPI_STRING             ModuleName, 
    INT32                   LineNumber, 
    INT32                   ComponentId, 
    ACPI_STRING             Message);

void
_ReportSuccess (
    ACPI_STRING             ModuleName, 
    INT32                   LineNumber, 
    INT32                   ComponentId, 
    ACPI_STRING             Message);

void 
CmDumpBuffer (
    char                    *Buffer, 
    UINT32                  Count, 
    UINT32                  Display,
    INT32                   componentId);



/* 
 * CmDelete - Object deletion 
 */

void
CmDeleteInternalObj (
    ACPI_OBJECT_INTERNAL    *Object);

void
CmDeleteInternalPackageObject (
    ACPI_OBJECT_INTERNAL    *Object);

void
CmDeleteInternalSimpleObject (
    ACPI_OBJECT_INTERNAL    *Object);

ACPI_STATUS
CmDeleteInternalObjectList (
    ACPI_OBJECT_INTERNAL    **ObjList);



/*
 * CmError - exception interfaces
 */

char *
CmFormatException (
    ACPI_STATUS             Status);


/*
 * CmMutex - mutual exclusion interfaces
 */

ACPI_STATUS
CmMutexInitialize (
    void);

void
CmMutexTerminate (
    void);

ACPI_STATUS
CmCreateMutex (
    ACPI_MUTEX_HANDLE       MutexId);

ACPI_STATUS
CmDeleteMutex (
    ACPI_MUTEX_HANDLE       MutexId);

ACPI_STATUS
CmAcquireMutex (
    ACPI_MUTEX_HANDLE       MutexId);

ACPI_STATUS
CmReleaseMutex (
    ACPI_MUTEX_HANDLE       MutexId);


/*
 * CmObject - internal object create/delete/cache routines
 */

#define CmCreateInternalObject(t)   _CmCreateInternalObject(_THIS_MODULE,__LINE__,_COMPONENT,t)
#define CmAllocateObjectDesc()      _CmAllocateObjectDesc(_THIS_MODULE,__LINE__,_COMPONENT)

void *
_CmAllocateObjectDesc (
    char                    *ModuleName, 
    INT32                   LineNumber, 
    INT32                   ComponentId); 

void
CmDeleteObjectDesc (
    ACPI_OBJECT_INTERNAL    *Object);

BOOLEAN
CmValidInternalObject (
    void                    *Object);

    
/*
 * CmRefCnt - Object reference count management
 */

void
CmAddReference (
    ACPI_OBJECT_INTERNAL    *Object);

void
CmRemoveReference (
    ACPI_OBJECT_INTERNAL    *Object);

/*
 * CmSize - Object size routines
 */

ACPI_STATUS
CmGetSimpleObjectSize (
    ACPI_OBJECT_INTERNAL    *Obj, 
    UINT32                  *ObjLength);

ACPI_STATUS
CmGetPackageObjectSize (
    ACPI_OBJECT_INTERNAL    *Obj, 
    UINT32                  *ObjLength);

ACPI_STATUS
CmGetObjectSize(
    ACPI_OBJECT_INTERNAL    *Obj, 
    UINT32                  *ObjLength);


/* 
 * CmState - Generic state creation/cache routines
 */

void
CmPushGenericState (
    ACPI_GENERIC_STATE      **ListHead,
    ACPI_GENERIC_STATE      *State);

ACPI_GENERIC_STATE *
CmPopGenericState (
    ACPI_GENERIC_STATE      **ListHead);


ACPI_GENERIC_STATE *
CmCreateGenericState (
    void);

ACPI_GENERIC_STATE *
CmCreateUpdateState (
    ACPI_OBJECT_INTERNAL    *Object,
    UINT16                  Action);

ACPI_STATUS
CmCreateUpdateStateAndPush (
    ACPI_OBJECT_INTERNAL    *Object,
    UINT16                  Action,
    ACPI_GENERIC_STATE      **StateList);

ACPI_GENERIC_STATE *
CmCreateControlState (
    void);

void
CmDeleteGenericState (
    ACPI_GENERIC_STATE      *State);

void
CmDeleteGenericStateCache (
    void);

void
CmDeleteObjectCache (
    void);

/*
 * Cmutils
 */

BOOLEAN 
CmValidAcpiName (
    UINT32                  Name);

BOOLEAN
CmValidAcpiCharacter (
    char                    Character);



/*
 * Memory allocation functions and related macros.
 * Macros that expand to include filename and line number
 */

void *
_CmAllocate (
 UINT32     Size,
 UINT32                  Component,
 ACPI_STRING    Module,
 INT32                   Line);

void *
_CmCallocate (
 UINT32     Size,
 UINT32                  Component,
 ACPI_STRING             Module,
 INT32                   Line);

void
_CmFree (
 void     *Address,
 UINT32                  Component,
 ACPI_STRING             Module,
 INT32                   Line);

void
CmInitStaticObject (
    ACPI_OBJECT_INTERNAL    *ObjDesc);

#define CmAllocate(a)          _CmAllocate(a,_COMPONENT,_THIS_MODULE,__LINE__)
#define CmCallocate(a)          _CmCallocate(a, _COMPONENT,_THIS_MODULE,__LINE__)
#define CmFree(a)           _CmFree(a,_COMPONENT,_THIS_MODULE,__LINE__)

#ifndef ACPI_DEBUG

#define CmAddElementToAllocList(a,b,c,d,e,f)
#define CmDeleteElementFromAllocList(a,b,c,d)
#define CmDumpCurrentAllocations(a,b)
#define CmDumpAllocationInfo()

#define DECREMENT_OBJECT_METRICS(a)
#define INCREMENT_OBJECT_METRICS(a)
#define INITIALIZE_ALLOCATION_METRICS()

#else

#define INITIALIZE_ALLOCATION_METRICS() \
 Gbl_CurrentObjectCount = 0; \
 Gbl_CurrentObjectSize = 0; \
 Gbl_RunningObjectCount = 0; \
 Gbl_RunningObjectSize = 0; \
 Gbl_MaxConcurrentObjectCount = 0; \
 Gbl_MaxConcurrentObjectSize = 0; \
 Gbl_CurrentAllocSize = 0; \
 Gbl_CurrentAllocCount = 0; \
 Gbl_RunningAllocSize = 0; \
 Gbl_RunningAllocCount = 0; \
 Gbl_MaxConcurrentAllocSize = 0; \
 Gbl_MaxConcurrentAllocCount = 0

#define DECREMENT_OBJECT_METRICS(a) \
 Gbl_CurrentObjectCount--; \
 Gbl_CurrentObjectSize -= a

#define INCREMENT_OBJECT_METRICS(a) \
 Gbl_CurrentObjectCount++; \
 Gbl_RunningObjectCount++; \
 if (Gbl_MaxConcurrentObjectCount < Gbl_CurrentObjectCount) \
 { \
  Gbl_MaxConcurrentObjectCount = Gbl_CurrentObjectCount; \
 } \
 Gbl_RunningObjectSize += a; \
 Gbl_CurrentObjectSize += a; \
 if (Gbl_MaxConcurrentObjectSize < Gbl_CurrentObjectSize) \
 { \
  Gbl_MaxConcurrentObjectSize = Gbl_CurrentObjectSize; \
 }
 

void
CmDumpAllocationInfo (
 void);
 
void
CmDumpCurrentAllocations (
 UINT32     Component,
 ACPI_STRING    Module);

#endif



#endif /* _COMMON_H */
