
/*
 *  ./acpica/subsystem/include/debugger.h - ACPI/AML debugger
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef __DEBUGGER_H__
#define __DEBUGGER_H__


#define DB_MAX_ARGS             8  /* Must be max method args + 1 */

#define DB_COMMAND_PROMPT      '-'
#define DB_EXECUTE_PROMPT      '%'


extern int                      optind;
extern char                     *optarg;
extern UINT8                    *AmlPtr;
extern UINT32                   AmlLength;

extern BOOLEAN                  opt_tables;
extern BOOLEAN                  opt_disasm;
extern BOOLEAN                  opt_stats;
extern BOOLEAN                  opt_parse_jit;
extern BOOLEAN     opt_verbose;
   

extern char                     *Args[DB_MAX_ARGS];
extern char                     LineBuf[80];
extern char                     ScopeBuf[40];
extern char                     DebugFilename[40];
extern BOOLEAN                  OutputToFile;
extern char                     *Buffer;
extern char                     *Filename;
extern char         *INDENT_STRING;
extern UINT32                   Gbl_MethodBreakpoint;
extern UINT8                    Gbl_DbOutputFlags;
extern UINT32                   Gbl_DbDebugLevel;
extern UINT32                   Gbl_DbConsoleDebugLevel;

extern UINT32                   NumNames;
extern UINT32                   NumMethods;
extern UINT32                   NumRegions;
extern UINT32                   NumPackages;
extern UINT32                   NumAliases;
extern UINT32                   NumDevices;
extern UINT32                   NumFieldDefs;
extern UINT32                   NumThermalZones;
extern UINT32                   NumNamedObjects;
extern UINT32                   NumGrammarElements;
extern UINT32                   NumMethodElements ;
extern UINT32                   NumMutexes;
extern UINT32                   NumPowerResources;
extern UINT32                   NumBankFields ;
extern UINT32                   NumIndexFields;
extern UINT32                   NumEvents;

extern UINT32                   SizeOfParseTree;
extern UINT32                   SizeOfMethodTrees;
extern UINT32                   SizeOfNTEs;
extern UINT32                   SizeOfAcpiObjects;


#define BUFFER_SIZE             4196

#define DB_REDIRECTABLE_OUTPUT  0x01
#define DB_CONSOLE_OUTPUT       0x02
#define DB_DUPLICATE_OUTPUT     0x03



typedef struct CommandInfo
{
    char                    *Name;          /* Command Name */
    char                    MinArgs;        /* Minimum arguments required */

} COMMAND_INFO;


typedef struct ArgumentInfo
{
    char                    *Name;          /* Argument Name */

} ARGUMENT_INFO;



#define PARAM_LIST(pl)                  pl

#define DBTEST_OUTPUT_LEVEL(lvl)        if (opt_verbose)

#define VERBOSE_PRINT(fp)               DBTEST_OUTPUT_LEVEL(lvl) {\
                                            OsdPrintf PARAM_LIST(fp);}

#define EX_NO_SINGLE_STEP       1
#define EX_SINGLE_STEP          2


void
DbCreateExecutionThreads (
    char                    *NumThreadsArg,
    char                    *NumLoopsArg,
    char                    *MethodNameArg);

INT32
DbMatchArgument (
    char                    *UserArgument,
    ARGUMENT_INFO           *Arguments);

void
DbDumpNamespaceByOwner (
    char                    *OwnerArg,
    char                    *DepthArg);

void
DbDisplayTableInfo (
    char                    *TableArg);

void
DbUnloadAcpiTable (
    char                    *TableArg,
    char                    *InstanceArg);


void
DbGenerateStatistics (
    ACPI_GENERIC_OP         *Root,
    BOOLEAN                 IsMethod);

void
DbAddToHistory (
    char                    *CommandLine);

void
DbDisplayHistory (void);

char *
DbGetFromHistory (
    char                    *CommandNumArg);

void
DbDisplayArgumentObject (
    ACPI_OBJECT_INTERNAL    *ObjDesc);

ACPI_STATUS
DbDisplayObjects (
    char                    *ObjTypeArg,
    char                    *DisplayCountArg);

ACPI_STATUS
DbFindNameInNamespace (
    char                    *NameArg);
void
DbSetOutputDestination (
    INT32                   Where);
    
void
DbSetMethodCallBreakpoint (
    ACPI_GENERIC_OP         *Op);

int
DbInitialize (void);

ACPI_STATUS
DbCommandDispatch (
    char                    *InputBuffer,
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *Op);

void
DbExecuteThread (
    void                    *Context);

ACPI_STATUS
DbUserCommands (
    char                    Prompt,
    ACPI_GENERIC_OP         *Op);

NAME_TABLE_ENTRY *
DbLocalNsLookup (
    char                    *Name);

ACPI_STATUS
DbExecuter (
    char                    Prompt,
    ACPI_GENERIC_OP         *Op);

void
DbPrepNamestring (
    char                    *Name);

void
DbDisplayMethodInfo (
    ACPI_GENERIC_OP         *Op);

ACPI_STATUS
DbSecondPassParse (
    ACPI_GENERIC_OP         *Root);

ACPI_STATUS
DbSingleStep (
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *Op,
    UINT8                   OpType);

void
DbDisassembleAml (
    char                    *Statements,
    ACPI_GENERIC_OP         *Op);

void
DbDecodeAndDisplayObject (
    char                    *Target,
    char                    *OutputType);

void
DbDisplayResultObject (
    ACPI_OBJECT_INTERNAL    *RetDesc);

ACPI_STATUS
DbDisplayStatistics (
    char                    *TypeArg);

void
DbSetMethodBreakpoint (
    char                    *Location,
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *Op);

void
DbSendNotify (
    char                    *Name,
    UINT32                  Value);

void
DbSetMethodData (
    char                    *TypeArg,
    char                    *IndexArg,
    char                    *ValueArg);

void
DbDumpBuffer (
    UINT32                  Address);

void
DbDumpObject (
    ACPI_OBJECT             *ObjDesc,
    UINT32                  Level);

ACPI_STATUS
DbDisplayAllMethods (
    char                    *DisplayCountArg);

void 
DbDisplayInternalObject (
    ACPI_OBJECT_INTERNAL    *ObjDesc);

ACPI_STATUS
DbLoadAcpiTable (
    char                    *Filename);

void
DbDisplayArguments (void);

void
DbDisplayLocals (void);

void
DbDisplayResults (void);

void
DbDisplayCallingTree (void);

void
DbDisplayOp (
    ACPI_GENERIC_OP         *origin,
    UINT32                  NumOpcodes);

INT32
DbSafeSprintf (
    char                    *Buffer,
    INT32                   BufferSize,
    INT32                   ExpandedSize,
    char                    *Format,
    ...);

INT32
DbSprintName (
    char                    *Buffer, 
    UINT32                  BufferSize, 
    UINT32                  Name);

INT32
DbSprintNamestring (
    char                    *BufferStart, 
    INT32                   BufferSize, 
    UINT8                   *Name);

INT32
DbSprintPath (
    char                    *BufferStart,
    UINT32                  BufferSize,
    ACPI_GENERIC_OP         *Op);

INT32
DbSprintOp (
    char                    *BufferStart, 
    UINT32                  BufferSize, 
    ACPI_GENERIC_OP         *Op);


void
DbDumpNamespace (
    char                    *StartArg,
    char                    *DepthArg);

void
DbExecute (
    char                    *Name,
    char                    **Args,
    UINT32                  Flags);

void
DbSetScope (
    char                    *Name);

void
DbCloseDebugFile (
    void);

void
DbOpenDebugFile (
    char                    *Name);










#endif  /* __DEBUGGER_H__ */
