/*
 *  ./acpica/subsystem/include/dispatch.h - 
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */





#ifndef _DISPATCH_H_
#define _DISPATCH_H_


#define NAMEOF_LOCAL_NTE    "__L0"
#define NAMEOF_ARG_NTE      "__A0"


/* For DsMethodDataSetValue */

#define MTH_TYPE_LOCAL              0
#define MTH_TYPE_ARG                1




/* Common interfaces */

ACPI_STATUS
DsObjStackPush (
    void                    *Object,
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
DsObjStackPop (
    UINT32                  PopCount,
    ACPI_WALK_STATE         *WalkState);

void *
DsObjStackGetValue (
    UINT32                  Index,
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
DsObjStackPopObject (
    ACPI_OBJECT_INTERNAL    **Object,
    ACPI_WALK_STATE         *WalkState);


/* dsregion - Op region support */

ACPI_STATUS
DsGetRegionArguments (
    ACPI_OBJECT_INTERNAL    *RgnDesc);


/* dsctrl - Parser/Interpreter interface, control stack routines */

/*
ACPI_CTRL_STATE *
DsCreateControlState (void);

void
DsPushControlState (
    ACPI_CTRL_STATE         *ControlState,
    ACPI_WALK_STATE         *WalkState);

ACPI_CTRL_STATE *
DsPopControlState (
    ACPI_WALK_STATE         *WalkState);
*/

ACPI_STATUS 
DsExecBeginControlOp (
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *Op);

ACPI_STATUS 
DsExecEndControlOp (
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *Op);


/* dsexec - Parser/Interpreter interface, method execution callbacks */

ACPI_STATUS
DsExecBeginOp (
    ACPI_WALK_STATE         *State,
    ACPI_GENERIC_OP         *Op);

ACPI_STATUS
DsExecEndOp (
    ACPI_WALK_STATE         *State,
    ACPI_GENERIC_OP         *Op);


/* dsfield - Parser/Interpreter interface for AML fields */


ACPI_STATUS
DsCreateField (
    ACPI_GENERIC_OP         *Op,
    ACPI_HANDLE             Region,
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
DsCreateBankField (
    ACPI_GENERIC_OP         *Op,
    ACPI_HANDLE             Region,
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
DsCreateIndexField (
    ACPI_GENERIC_OP         *Op,
    ACPI_HANDLE             Region,
    ACPI_WALK_STATE         *WalkState);


/* dsload - Parser/Interpreter interface, namespace load callbacks */

ACPI_STATUS
DsLoad1BeginOp (
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *Op);

ACPI_STATUS
DsLoad1EndOp (
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *Op);

ACPI_STATUS
DsLoad2BeginOp (
    ACPI_WALK_STATE         *State,
    ACPI_GENERIC_OP         *Op);

ACPI_STATUS
DsLoad2EndOp (
    ACPI_WALK_STATE         *State,
    ACPI_GENERIC_OP         *Op);


/* dsmthdat - method data (locals/args) */


ACPI_STATUS
DsMethodDataDeleteAll (
    ACPI_WALK_STATE         *WalkState);

BOOLEAN
DsIsMethodValue (
    ACPI_OBJECT_INTERNAL    *ObjDesc);

ACPI_OBJECT_TYPE
DsMethodDataGetType (
    UINT32                  Type,
    UINT32                  Index);

ACPI_STATUS
DsMethodDataGetValue (
    UINT32                  Type,
    UINT32                  Index, 
    ACPI_OBJECT_INTERNAL    **ObjDesc);

ACPI_STATUS
DsMethodDataSetValue (
    UINT32                  Type,
    UINT32                  Index, 
    ACPI_OBJECT_INTERNAL    *ObjDesc); 

ACPI_STATUS
DsMethodDataDeleteValue (
    UINT32                  Type,
    UINT32                  Index);

ACPI_STATUS
DsMethodDataInitArgs (
    ACPI_OBJECT_INTERNAL    **Params,
    UINT32                  ParamCount);

NAME_TABLE_ENTRY *
DsMethodDataGetNte (
    UINT32                  Type,
    UINT32                  Index);

ACPI_STATUS
DsMethodDataInit (
    ACPI_WALK_STATE         *WalkState);


/* dsmethod - Parser/Interpreter interface - control method parsing */

ACPI_STATUS
DsParseMethod (
    ACPI_HANDLE             ObjHandle);

ACPI_STATUS
DsCallControlMethod (
    ACPI_WALK_LIST          *WalkList,
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *Op);

ACPI_STATUS
DsRestartControlMethod (
    ACPI_WALK_STATE         *WalkState,
    ACPI_OBJECT_INTERNAL    *ReturnDesc);

ACPI_STATUS
DsTerminateControlMethod (
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
DsBeginMethodExecution (
    NAME_TABLE_ENTRY        *MethodEntry,
    ACPI_OBJECT_INTERNAL    *ObjDesc);


/* dsobj - Parser/Interpreter interface - object initialization and conversion */

ACPI_STATUS
DsInitOneObject (
    ACPI_HANDLE             ObjHandle, 
    UINT32                  Level, 
    void                    *Context,
    void                    **ReturnValue);

ACPI_STATUS
DsInitializeObjects (
    ACPI_TABLE_DESC         *TableDesc,
    NAME_TABLE_ENTRY        *StartEntry);

ACPI_STATUS
DsBuildInternalPackageObj (
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *op,
    ACPI_OBJECT_INTERNAL    **ObjDesc);

ACPI_STATUS
DsBuildInternalObject (
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *op,
    ACPI_OBJECT_INTERNAL    **ObjDescPtr);

ACPI_STATUS
DsInitObjectFromOp (
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *Op,
    UINT16                  Opcode,
    ACPI_OBJECT_INTERNAL    *ObjDesc);

ACPI_STATUS
DsCreateNamedObject (
    ACPI_WALK_STATE         *WalkState,
    NAME_TABLE_ENTRY        *Entry,
    ACPI_GENERIC_OP         *Op);


/* dsregn - Parser/Interpreter interface - Op Region parsing */

ACPI_STATUS
DsEvalRegionOperands (
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *Op);

ACPI_STATUS
DsInitializeRegion (
    ACPI_HANDLE             ObjHandle);

    
/* dsutils - Parser/Interpreter interface utility routines */

void
DsDeleteResultIfNotUsed (
    ACPI_GENERIC_OP         *Op,
    ACPI_OBJECT_INTERNAL    *ResultObj,
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
DsCreateOperand (
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *Arg);

ACPI_STATUS
DsCreateOperands (
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *FirstArg);

ACPI_STATUS
DsResolveOperands (
    ACPI_WALK_STATE         *WalkState);

ACPI_OBJECT_TYPE
DsMapOpcodeToDataType (
    UINT16                  Opcode,
    UINT32                  *OutFlags);

ACPI_OBJECT_TYPE 
DsMapNamedOpcodeToDataType (
    UINT16                  Opcode);


/*
 * dswscope - Scope Stack manipulation
 */

ACPI_STATUS
DsScopeStackPush (
    NAME_TABLE_ENTRY        *NewScope, 
    ACPI_OBJECT_TYPE        Type,
 ACPI_WALK_STATE   *WalkState);


ACPI_STATUS
DsScopeStackPop (
 ACPI_WALK_STATE   *WalkState);

void
DsScopeStackClear (
 ACPI_WALK_STATE   *WalkState);


/* Dswstate - parser WALK_STATE management routines */

ACPI_WALK_STATE *
DsCreateWalkState (
    ACPI_GENERIC_OP         *Origin,
    ACPI_OBJECT_INTERNAL    *MthDesc,
    ACPI_WALK_LIST          *WalkList);

ACPI_STATUS
DsObjStackDeleteAll (
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
DsObjStackPopAndDelete (
    UINT32                  PopCount,
    ACPI_WALK_STATE         *WalkState);

void
DsDeleteWalkState (
    ACPI_WALK_STATE         *WalkState);

ACPI_WALK_STATE *
DsPopWalkState (
    ACPI_WALK_LIST          *WalkList);

ACPI_STATUS
DsResultStackPop (
    ACPI_OBJECT_INTERNAL    **Object,
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
DsResultStackPush (
    void                    *Object,
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
DsResultStackClear (
    ACPI_WALK_STATE         *WalkState);

ACPI_WALK_STATE *
DsGetCurrentWalkState (
    ACPI_WALK_LIST          *WalkList);

void
DsDeleteWalkStateCache (
    void);




#endif /* _DISPATCH_H_ */