
/*
 *  ./acpica/subsystem/include/events.h - Event subcomponent prototypes and defines
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef __EVENTS_H__
#define __EVENTS_H__

typedef struct
{
    NATIVE_UINT      Seg;
    NATIVE_UINT      Bus;
    NATIVE_UINT      DevFunc;
} PCI_HANDLER_CONTEXT;


/* Status bits. */

#define STATUS_PMTIMER                  0x0001
#define STATUS_GLOBAL                   0x0020
#define STATUS_POWER_BUTTON             0x0100
#define STATUS_SLEEP_BUTTON             0x0200
#define STATUS_RTC_ALARM                0x0400

/* Enable bits. */

#define ENABLE_PMTIMER                  0x0001
#define ENABLE_GLOBAL                   0x0020
#define ENABLE_POWER_BUTTON             0x0100
#define ENABLE_SLEEP_BUTTON             0x0200
#define ENABLE_RTC_ALARM                0x0400



/* 
 * Evfixed - Fixed event handling
 */

ACPI_STATUS
EvFixedEventInitialize (
    void);

UINT32
EvFixedEventDetect (
    void);

UINT32
EvFixedEventDispatch (
    UINT32                  Event);


/*
 * Evglock - Global Lock support
 */

ACPI_STATUS
EvAcquireGlobalLock(
    void);

void
EvReleaseGlobalLock(
    void);

ACPI_STATUS
EvInitGlobalLockHandler (
    void);


/*
 * Evgpe - GPE handling and dispatch
 */

ACPI_STATUS
EvGpeInitialize (
    void);

ACPI_STATUS
EvInitGpeControlMethods (
    void);

UINT32
EvGpeDispatch (
    UINT32                  GpeNumber);

UINT32
EvGpeDetect (
    void);


/*
 * Evnotify - Device Notify handling and dispatch
 */

void
EvNotifyDispatch (
    ACPI_HANDLE             Device, 
    UINT32                  NotifyValue);


/* 
 * Evregion - Address Space handling
 */

ACPI_STATUS
EvInstallDefaultAddressSpaceHandlers (
    void);

ACPI_STATUS
EvAddressSpaceDispatch (
    ACPI_OBJECT_INTERNAL   *RegionObj,
    UINT32                  Function,
    UINT32                  Address,
    UINT32                  BitWidth,
    UINT32                  *Value);


ACPI_STATUS
EvAddrHandlerHelper (
    ACPI_HANDLE             ObjHandle, 
    UINT32                  Level, 
    void                    *Context,
    void                    **ReturnValue);

void
EvDisassociateRegionFromHandler(
    ACPI_OBJECT_INTERNAL   *RegionObj);


ACPI_STATUS
EvAssociateRegionAndHandler(
    ACPI_OBJECT_INTERNAL    *HandlerObj,
    ACPI_OBJECT_INTERNAL    *RegionObj);


/*
 * Evregini - Region initialization and setup
 */

ACPI_STATUS
EvSystemMemoryRegionSetup (
    ACPI_OBJECT_INTERNAL    *RegionObj,
    UINT32                  Function,
    void                    *HandlerContext,
    void                    **ReturnContext);

ACPI_STATUS
EvIoSpaceRegionSetup (
    ACPI_OBJECT_INTERNAL    *RegionObj,
    UINT32                  Function,
    void                    *HandlerContext,
    void                    **ReturnContext);

ACPI_STATUS
EvPciConfigRegionSetup (
    ACPI_OBJECT_INTERNAL    *RegionObj,
    UINT32                  Function,
    void                    *HandlerContext,
    void                    **ReturnContext);

ACPI_STATUS
EvDefaultRegionSetup (
    ACPI_OBJECT_INTERNAL    *RegionObj,
    UINT32                  Function,
    void                    *HandlerContext,
    void                    **ReturnContext);

ACPI_STATUS
EvInitializeRegion (
    ACPI_OBJECT_INTERNAL    *RegionObj,
    BOOLEAN                 NsLocked);


/*
 * Evsci - SCI (System Control Interrupt) handling/dispatch
 */
    
UINT32 
EvInstallSciHandler (
    void);

ACPI_STATUS
EvRemoveSciHandler (
    void);

INT32 
EvInitializeSCI (
    INT32                   ProgramSCI);

void
EvRestoreAcpiState (
    void);

void
EvTerminate (
    void);







/* Debug support */

#ifdef ACPI_DEBUG

INT32 
EvSciCount (
    UINT32                  Event);

#define DEBUG_INCREMENT_EVENT_COUNT(a)   Gbl_EventCount[a]++;

#else

#define DEBUG_INCREMENT_EVENT_COUNT(a)
#endif


#endif  /*  __EVENTS_H__   */
