
/*
 *  ./acpica/subsystem/include/globals.h - Declarations for global variables
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef __GLOBALS_H__
#define __GLOBALS_H__

#include <config.h>
#include <actables.h>
#include <internal.h>
#include <acobject.h>


/*
 * Ensure that the globals are actually defined only once
 */
#ifdef DEFINE_ACPI_GLOBALS
#define ACPI_EXTERN
#else
#define ACPI_EXTERN extern
#endif


extern      char                        *MsgAcpiErrorBreak;


/* Runtime configuration of debug print levels */

extern      UINT32                      DebugLevel;
extern      UINT32                      DebugLayer;


/* Procedure nesting level for debug output */

extern      UINT32                      Gbl_NestingLevel;




/*
 * Table pointers.
 * Although these pointers are somewhat redundant with the global AcpiTable,
 * they are convenient because they are typed pointers.
 *
 * These tables are single-table only; meaning that there can be at most one
 * of each in the system.  Each global points to the actual table.
 *
 */
ACPI_EXTERN ROOT_SYSTEM_DESCRIPTOR_POINTER      *Gbl_RSDP;
ACPI_EXTERN ROOT_SYSTEM_DESCRIPTION_TABLE       *Gbl_RSDT;
ACPI_EXTERN FIRMWARE_ACPI_CONTROL_STRUCTURE     *Gbl_FACS;
ACPI_EXTERN FIXED_ACPI_DESCRIPTION_TABLE        *Gbl_FACP;
ACPI_EXTERN APIC_TABLE                          *Gbl_APIC;
ACPI_EXTERN ACPI_TABLE_HEADER                   *Gbl_DSDT;
ACPI_EXTERN ACPI_TABLE_HEADER                   *Gbl_SBST;
/* 
 * Since there may be multiple SSDTs and PSDTS, a single pointer is not 
 * sufficient; Therefore, there isn't one!
 */


/*
 * ACPI Table info arrays
 */
extern      ACPI_TABLE_DESC             Gbl_AcpiTables[NUM_ACPI_TABLES];
extern      ACPI_TABLE_SUPPORT          Gbl_AcpiTableData[NUM_ACPI_TABLES];

/*
 * Predefined mutex objects.  This array contains the 
 * actual OS mutex handles, indexed by the local ACPI_MUTEX_HANDLEs.
 * (The table maps local handles to the real OS handles)
 */
ACPI_EXTERN ACPI_MUTEX_INFO             Gbl_AcpiMutexInfo [NUM_MTX];
extern      ACPI_INIT_DATA              Gbl_AcpiInitData;




ACPI_EXTERN UINT8                      *Gbl_Gpe0EnableRegisterSave;
ACPI_EXTERN UINT8                      *Gbl_Gpe1EnableRegisterSave;
ACPI_EXTERN ACPI_WALK_STATE            *Gbl_BreakpointWalk;
ACPI_EXTERN ACPI_GENERIC_STATE         *Gbl_GenericStateCache;
ACPI_EXTERN ACPI_GENERIC_OP            *Gbl_ParseCache;
ACPI_EXTERN ACPI_OBJECT_INTERNAL       *Gbl_ObjectCache;
ACPI_EXTERN ACPI_WALK_STATE            *Gbl_WalkStateCache;
ACPI_EXTERN ACPI_HANDLE                 Gbl_GlobalLockSemaphore;


ACPI_EXTERN UINT32                      Gbl_GlobalLockThreadCount;
ACPI_EXTERN UINT32                      Gbl_RestoreAcpiChipset;
ACPI_EXTERN UINT32                      Gbl_OriginalMode;
ACPI_EXTERN UINT32                      Gbl_EdgeLevelSave;
ACPI_EXTERN UINT32                      Gbl_IrqEnableSave;
ACPI_EXTERN UINT32                      Gbl_RsdpOriginalLocation; 

ACPI_EXTERN UINT32                      Gbl_StateCacheRequests;
ACPI_EXTERN UINT32                      Gbl_StateCacheHits;
ACPI_EXTERN UINT32                      Gbl_ParseCacheRequests;
ACPI_EXTERN UINT32                      Gbl_ParseCacheHits;
ACPI_EXTERN UINT32                      Gbl_ObjectCacheRequests;
ACPI_EXTERN UINT32                      Gbl_ObjectCacheHits;
ACPI_EXTERN UINT32                      Gbl_WalkStateCacheRequests;
ACPI_EXTERN UINT32                      Gbl_WalkStateCacheHits;
ACPI_EXTERN UINT32                      Gbl_NsLookupCount;
ACPI_EXTERN UINT32                      Gbl_PsFindCount;


ACPI_EXTERN UINT16                      Gbl_GenericStateCacheDepth;
ACPI_EXTERN UINT16                      Gbl_ParseCacheDepth;
ACPI_EXTERN UINT16                      Gbl_ObjectCacheDepth;
ACPI_EXTERN UINT16                      Gbl_WalkStateCacheDepth;
ACPI_EXTERN UINT16                      Gbl_Pm1EnableRegisterSave;
ACPI_EXTERN UINT16                      Gbl_NextTableOwnerId;
ACPI_EXTERN UINT16                      Gbl_NextMethodOwnerId;

ACPI_EXTERN UINT8                       Gbl_DebuggerConfiguration;
ACPI_EXTERN BOOLEAN                     Gbl_GlobalLockAcquired;
ACPI_EXTERN BOOLEAN                     Gbl_GlobalLockSet; /* TBD: OBSOLETE?? */
ACPI_EXTERN BOOLEAN                     Gbl_MethodExecuting;
ACPI_EXTERN BOOLEAN                     Gbl_StepToNextCall;
ACPI_EXTERN BOOLEAN                     Gbl_AcpiHardwarePresent;
ACPI_EXTERN BOOLEAN                     Gbl_DbTerminateThreads;


ACPI_EXTERN ACPI_OBJECT_NotifyHandler   Gbl_DrvNotify;
ACPI_EXTERN ACPI_OBJECT_NotifyHandler   Gbl_SysNotify;


extern  BOOLEAN                     Gbl_Shutdown;
extern      UINT32                      Gbl_SystemFlags;
extern      UINT32                      Gbl_StartupFlags;




#define NUM_NS_TYPES                    INTERNAL_TYPE_Invalid+1
#define NUM_PREDEFINED_NAMES            9


ACPI_EXTERN NAME_TABLE_ENTRY            Gbl_RootObjStruct;
ACPI_EXTERN NAME_TABLE_ENTRY           *Gbl_RootObject;

extern      UINT8                       Gbl_NsProperties[NUM_NS_TYPES];
extern      PREDEFINED_NAMES            Gbl_PreDefinedNames [NUM_PREDEFINED_NAMES];


/* Used to detect memory leaks (DEBUG ONLY) */

#ifdef ACPI_DEBUG
ACPI_EXTERN ALLOCATION_INFO            *Gbl_HeadAllocPtr;
ACPI_EXTERN ALLOCATION_INFO            *Gbl_TailAllocPtr;
#endif





ACPI_EXTERN UINT32                      Gbl_WhenToParseMethods;
ACPI_EXTERN ACPI_WALK_LIST             *Gbl_CurrentWalkList;

/* Base of AML block, and pointer to current location in it */

ACPI_EXTERN UINT8                      *Gbl_PCodeBase;
ACPI_EXTERN UINT8                      *Gbl_PCode;

/* 
 * Length of AML block, and remaining length of current package.
 */
ACPI_EXTERN UINT32                      Gbl_PCodeBlockLen;
ACPI_EXTERN UINT32                      Gbl_PCodeLen;

ACPI_EXTERN UINT32                      Gbl_BufSeq;             /* Counts allocated Buffer descriptors */
ACPI_EXTERN INT32                       Gbl_NamedObjectErr;     /* Indicate if inc_error should be called */

/* 
 * Handle to the last method found - used during pass1 of load 
 */
ACPI_EXTERN ACPI_HANDLE                 Gbl_LastMethod;

/*
 * Table of Address Space handlers
 */

ACPI_EXTERN ADDRESS_SPACE_INFO          Gbl_AddressSpaces[ACPI_NUM_ADDRESS_SPACES];


/* Control method single step flag */

ACPI_EXTERN UINT8                       Gbl_CmSingleStep;




ACPI_EXTERN ACPI_GENERIC_OP             *Gbl_ParsedNamespaceRoot;

extern ACPI_OP_INFO                     Gbl_AmlOpInfo[];
extern UINT8                            Gbl_AmlOpInfoIndex[256];
extern char                             *Gbl_ParserId;




ACPI_EXTERN FIXED_EVENT_INFO            Gbl_FixedEventHandlers[NUM_FIXED_EVENTS];

ACPI_EXTERN ACPI_HANDLE                 Gbl_GpeObjHandle;
ACPI_EXTERN UINT32                      Gbl_GpeRegisterCount;
ACPI_EXTERN GPE_REGISTERS              *Gbl_GpeRegisters;
ACPI_EXTERN GPE_LEVEL_INFO             *Gbl_GpeInfo;

/* 
 * Gpe validation and translation table
 * Indexed by the GPE number, returns GPE_INVALID if the GPE is not supported.
 * Otherwise, returns a valid index into the global GPE table.
 *
 * This table is needed because the GPE numbers supported by block 1 do not
 * have to be contiguous with the GPE numbers supported by block 0.
 */
ACPI_EXTERN UINT8                       Gbl_GpeValid [NUM_GPE];

/* Event counter for debug only */

#ifdef ACPI_DEBUG
ACPI_EXTERN UINT32                      Gbl_EventCount[NUM_FIXED_EVENTS];   
#endif





/* Memory allocation metrics - Debug Only! */

#ifdef ACPI_DEBUG

ACPI_EXTERN UINT32            Gbl_CurrentAllocSize;
ACPI_EXTERN UINT32            Gbl_CurrentAllocCount;
ACPI_EXTERN UINT32            Gbl_RunningAllocSize;
ACPI_EXTERN UINT32            Gbl_RunningAllocCount;
ACPI_EXTERN UINT32            Gbl_MaxConcurrentAllocSize;
ACPI_EXTERN UINT32            Gbl_MaxConcurrentAllocCount;
ACPI_EXTERN UINT32            Gbl_CurrentObjectCount;
ACPI_EXTERN UINT32            Gbl_CurrentObjectSize;
ACPI_EXTERN UINT32            Gbl_MaxConcurrentObjectCount;
ACPI_EXTERN UINT32            Gbl_MaxConcurrentObjectSize;
ACPI_EXTERN UINT32            Gbl_RunningObjectCount;
ACPI_EXTERN UINT32            Gbl_RunningObjectSize;

#endif





#endif /* __GLOBALS_H__ */
