
/*
 *  ./acpica/subsystem/include/interp.h - Interpreter subcomponent prototypes and defines
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef __INTERPRETER_H__
#define __INTERPRETER_H__


#include <actypes.h>
#include <acobject.h>


#define WALK_OPERANDS       &(WalkState->Operands [WalkState->NumOperands -1])


/* Interpreter constants */

#define AML_END_OF_BLOCK            -1
#define PUSH_PKG_LENGTH             1
#define DO_NOT_PUSH_PKG_LENGTH      0


#define STACK_TOP                   0
#define STACK_BOTTOM                (UINT32) -1

/* Constants for global "WhenToParseMethods" */

#define METHOD_PARSE_AT_INIT        0x0
#define METHOD_PARSE_JUST_IN_TIME   0x1
#define METHOD_DELETE_AT_COMPLETION 0x2


ACPI_STATUS
AmlResolveOperands (
    UINT16                  Opcode,
    ACPI_OBJECT_INTERNAL    **StackPtr);


/*
 * iapi - External interpreter interfaces
 */

ACPI_STATUS
AmlLoadTable (
    ACPI_TABLE_TYPE         TableId);

ACPI_STATUS
AmlExecuteMethod (
    NAME_TABLE_ENTRY        *MethodEntry,
    ACPI_OBJECT_INTERNAL    **Params,
    ACPI_OBJECT_INTERNAL    **ReturnObjDesc);


/*
 * iecopy - Interpreter object copy support
 */

ACPI_STATUS
AmlBuildCopyInternalPackageObject (
    ACPI_OBJECT_INTERNAL    *SourceObj, 
    ACPI_OBJECT_INTERNAL    *DestObj);


/*
 * iefield - ACPI AML (p-code) execution - field manipulation
 */


ACPI_STATUS
AmlReadField (
    ACPI_OBJECT_INTERNAL    *ObjDesc, 
    void                    *Buffer,
    UINT32                  BufferLength,
    UINT32                  ByteLength,
    UINT32                  DatumLength,
    UINT32                  BitGranularity,
    UINT32                  ByteGranularity);

ACPI_STATUS
AmlWriteField (
    ACPI_OBJECT_INTERNAL    *ObjDesc, 
    void                    *Buffer,
    UINT32                  BufferLength,
    UINT32                  ByteLength,
    UINT32                  DatumLength,
    UINT32                  BitGranularity,
    UINT32                  ByteGranularity);

ACPI_STATUS
AmlSetupField (
    ACPI_OBJECT_INTERNAL    *ObjDesc, 
    ACPI_OBJECT_INTERNAL    *RgnDesc, 
    INT32                   FieldBitWidth);

ACPI_STATUS
AmlReadFieldData (
    ACPI_OBJECT_INTERNAL    *ObjDesc, 
    UINT32                  FieldByteOffset,
    UINT32                  FieldBitWidth,
    UINT32                  *Value);

ACPI_STATUS
AmlAccessNamedField (
    INT32                   Mode, 
    ACPI_HANDLE             NamedField, 
    void                    *Buffer,
    UINT32                  Length);

ACPI_STATUS
AmlSetNamedFieldValue (
    ACPI_HANDLE             NamedField,
    void                    *Buffer,
    UINT32                  Length);

ACPI_STATUS
AmlGetNamedFieldValue (
    ACPI_HANDLE             NamedField, 
    void                    *Buffer,
    UINT32                  Length);


/*
 * ieopexec - ACPI AML (p-code) execution - specific opcodes
 */

ACPI_STATUS
AmlExecCreateField (
    UINT16                  Opcode,
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
AmlExecReconfiguration (
    UINT16                  Opcode,
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
AmlExecFatal (    
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
AmlExecIndex (
    ACPI_WALK_STATE         *WalkState,
    ACPI_OBJECT_INTERNAL    **ReturnDesc);

ACPI_STATUS
AmlExecMatch (
    ACPI_WALK_STATE         *WalkState,
    ACPI_OBJECT_INTERNAL    **ReturnDesc);

ACPI_STATUS
AmlExecCreateMutex (
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
AmlExecCreateProcessor (
    ACPI_GENERIC_OP         *Op,
    ACPI_HANDLE             ProcessorNTE);

ACPI_STATUS
AmlExecCreatePowerResource (
    ACPI_GENERIC_OP         *Op,
    ACPI_HANDLE             ProcessorNTE);

ACPI_STATUS
AmlExecCreateRegion (
    UINT8                   *AmlPtr,
    UINT32                  AmlLength,
    UINT32                  RegionSpace,
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
AmlExecCreateEvent (
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
AmlExecCreateAlias (
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
AmlExecCreateMethod (
    UINT8                   *AmlPtr,
    UINT32                  AmlLength,
    UINT32                  MethodFlags,
    ACPI_HANDLE             Method);


/*
 * ieprep - ACPI AML (p-code) execution - prep utilities
 */

ACPI_STATUS
AmlPrepDefFieldValue (
    NAME_TABLE_ENTRY        *ThisEntry,
    ACPI_HANDLE             Region, 
    UINT8                   FieldFlags,
    UINT8                   FieldAttribute,
    UINT32                  FieldPosition, 
    UINT32                  FieldLength);

ACPI_STATUS
AmlPrepBankFieldValue (
    NAME_TABLE_ENTRY        *ThisEntry,
    ACPI_HANDLE             Region, 
    ACPI_HANDLE             BankReg, 
    UINT32                  BankVal,
    UINT8                   FieldFlags, 
    UINT8                   FieldAttribute,
    UINT32                  FieldPosition, 
    UINT32                  FieldLength);

ACPI_STATUS
AmlPrepIndexFieldValue (
    NAME_TABLE_ENTRY        *ThisEntry,
    ACPI_HANDLE             IndexReg, 
    ACPI_HANDLE             DataReg,
    UINT8                   FieldFlags, 
    UINT8                   FieldAttribute,
    UINT32                  FieldPosition, 
    UINT32                  FieldLength);

ACPI_STATUS
AmlPrepOperands (
    char                    *Types,
    ACPI_OBJECT_INTERNAL    **StackPtr);



/*
 * iepstack - package stack utilities
 */

UINT32
AmlPkgStackLevel (
     void);

void
AmlClearPkgStack (
    void);

ACPI_STATUS
AmlPkgPushLength (
    UINT32                  Length, 
    OPERATING_MODE          LoadExecMode);

ACPI_STATUS
AmlPkgPushExecLength (
    UINT32                  Length);

ACPI_STATUS
AmlPkgPushExec (
    UINT8                   *Code, 
    UINT32                  Len);

ACPI_STATUS
AmlPkgPopLength (
    INT32                   NoErrUnder, 
    OPERATING_MODE          LoadExecMode);

ACPI_STATUS
AmlPkgPopExecLength (
    void);

ACPI_STATUS
AmlPkgPopExec (
    void);


/*
 * iesystem - Interface to OS services
 */

UINT16 
OsThreadId (
    void);

ACPI_STATUS
OsDoNotifyOp (
    ACPI_OBJECT_INTERNAL    *Value, 
    ACPI_OBJECT_INTERNAL    *ObjDesc);

void
OsDoSuspend(
    UINT32                  Time);

void
OsDoStall (
    UINT32                  Time);

ACPI_STATUS
OsAcquireMutex(
    ACPI_OBJECT_INTERNAL    *Time, 
    ACPI_OBJECT_INTERNAL    *ObjDesc);

ACPI_STATUS
OsReleaseMutex(
    ACPI_OBJECT_INTERNAL    *ObjDesc);

ACPI_STATUS
OsSignalEvent(
    ACPI_OBJECT_INTERNAL    *ObjDesc);

ACPI_STATUS
OsWaitEvent(
    ACPI_OBJECT_INTERNAL    *Time, 
    ACPI_OBJECT_INTERNAL    *ObjDesc);

ACPI_STATUS
OsResetEvent(
    ACPI_OBJECT_INTERNAL    *ObjDesc);

ACPI_STATUS
OsLocalWaitSemaphore (
    ACPI_HANDLE             Semaphore,
    UINT32                  Timeout);


/*
 * iemonadic - ACPI AML (p-code) execution, monadic operators
 */

ACPI_STATUS
AmlExecMonadic1 (
    UINT16                  Opcode,
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
AmlExecMonadic2 (
    UINT16                  Opcode,
    ACPI_WALK_STATE         *WalkState,
    ACPI_OBJECT_INTERNAL    **ReturnDesc);

ACPI_STATUS
AmlExecMonadic2R (
    UINT16                  Opcode,
    ACPI_WALK_STATE         *WalkState,
    ACPI_OBJECT_INTERNAL    **ReturnDesc);


/*
 * iedyadic - ACPI AML (p-code) execution, dyadic operators
 */

ACPI_STATUS
AmlExecDyadic1 (
    UINT16                  Opcode,
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
AmlExecDyadic2 (
    UINT16                  Opcode,
    ACPI_WALK_STATE         *WalkState,
    ACPI_OBJECT_INTERNAL    **ReturnDesc);

ACPI_STATUS
AmlExecDyadic2R (
    UINT16                  Opcode,
    ACPI_WALK_STATE         *WalkState,
    ACPI_OBJECT_INTERNAL    **ReturnDesc);
        
ACPI_STATUS
AmlExecDyadic2S (
    UINT16                  Opcode,
    ACPI_WALK_STATE         *WalkState,
    ACPI_OBJECT_INTERNAL    **ReturnDesc);



/*
 * iresolve  - Object resolution and get value functions
 */

ACPI_STATUS
AmlResolveToValue (
    ACPI_OBJECT_INTERNAL    **StackPtr);

ACPI_STATUS
AmlResolveEntryToValue (
    NAME_TABLE_ENTRY        **StackPtr);

ACPI_STATUS
AmlResolveObjectToValue (
    ACPI_OBJECT_INTERNAL    **StackPtr);

ACPI_STATUS
AmlGetFieldUnitValue (
    ACPI_OBJECT_INTERNAL    *FieldDesc, 
    ACPI_OBJECT_INTERNAL    *ResultDesc);



/*
 * iscode - Scanner AML code manipulation routines
 */

INT32
AmlAvail (
    ACPI_SIZE               n);

INT32
AmlPeek (
    void);

INT32
AmlGetPCodeByte (
    UINT8                   *Pcode);

UINT16
AmlPeekOp (
    void);

UINT8 *
AmlConsumeBytes (
    ACPI_SIZE               Bytes);

ACPI_SIZE
AmlConsumeStreamBytes (
    ACPI_SIZE               BytesToGet, 
    UINT8                   *AmlBuffer);

void
AmlConsumePackage (
    OPERATING_MODE          LoadExecMode);

void
AmlSetPCodeInput (
    UINT8                   *Base, 
    UINT32                  Length);

ACPI_STATUS
AmlSetMethod (
    void                    *Object);

ACPI_STATUS
AmlPrepExec (
    UINT8                   *Pcode, 
    UINT32                  PcodeLength);

ACPI_HANDLE
AmlGetPCodeHandle (
    void);

void
AmlGetCurrentLocation (
    ACPI_OBJECT_INTERNAL    *MethodDesc);

void
AmlSetCurrentLocation (
    ACPI_OBJECT_INTERNAL    *MethodDesc);


/*
 * isdump - Scanner debug output routines
 */

void 
AmlShowHexValue (
    INT32                   ByteCount, 
    UINT8                   *AmlPtr, 
    INT32                   LeadSpace);

void
AmlDumpBuffer (
    ACPI_SIZE               Length);


ACPI_STATUS
AmlDumpOperand (
    ACPI_OBJECT_INTERNAL    *EntryDesc);

void
AmlDumpOperands (
    ACPI_OBJECT_INTERNAL    **Operands,
    OPERATING_MODE          InterpreterMode, 
    char                    *Ident, 
    INT32                   NumLevels, 
    char                    *Note,
    char                    *ModuleName, 
    INT32                   LineNumber);

void
AmlDumpObjectDescriptor (
    ACPI_OBJECT_INTERNAL    *Object,
    UINT32                  Flags);


void
AmlDumpNameTableEntry (
    NAME_TABLE_ENTRY        *Entry,
    UINT32                  Flags);


/*
 * isnames - interpreter/scanner name load/execute
 */

char *
AmlAllocateNameString (
    INT32                   PrefixCount,
    INT32                   NSeg);

INT32
AmlGoodChar (
    INT32                   Character);

ACPI_STATUS
AmlExecNameSegment (
    UINT8                   **InAmlAddress,
    char                    *NameString);

ACPI_STATUS
AmlGetNameString (
    ACPI_OBJECT_TYPE        DataType, 
    UINT8                   *InAmlAddress,
    char                    **OutNameString,
    UINT32                  *OutNameLength)  ; 

INT32 
AmlDecodePackageLength (
    INT32                   LastPkgLen);


ACPI_STATUS
AmlDoName (
    ACPI_OBJECT_TYPE        DataType, 
    OPERATING_MODE          LoadExecMode);


/*
 * istore - Object store support
 */

ACPI_STATUS
AmlExecStore (
    ACPI_OBJECT_INTERNAL    *op1, 
    ACPI_OBJECT_INTERNAL    *res);

ACPI_STATUS
AmlStoreObjectToObject (
    ACPI_OBJECT_INTERNAL    *ValDesc,
    ACPI_OBJECT_INTERNAL    *DestDesc);

ACPI_STATUS
AmlStoreObjectToNte (
    ACPI_OBJECT_INTERNAL    *ValDesc,
    NAME_TABLE_ENTRY        *Entry);


/*
 * isutils - interpreter/scanner utilities
 */

void
AmlEnterInterpreter (
    void);

void
AmlExitInterpreter (
    void);

BOOLEAN
AmlValidateObjectType (
    ACPI_OBJECT_TYPE        Type);

BOOLEAN
AmlAcquireGlobalLock (
    UINT32                  Rule);

ACPI_STATUS
AmlReleaseGlobalLock (
    BOOLEAN                 Locked);

void
AmlAppendOperandDiag(
    char                    *Name, 
    INT32                   Line, 
    UINT16                  OpCode, 
    ACPI_OBJECT_INTERNAL    **Operands,
    INT32                   NOperands);

UINT32 
AmlBufSeq (
    void);

INT32
AmlDigitsNeeded (
    INT32                   Value, 
    INT32                   Base);

ACPI_STATUS
AmlEisaIdToString (
    UINT32                  NumericId,
    char                    *OutString);


/*
 * ieregion - default OpRegion handlers
 */

ACPI_STATUS
AmlSystemMemorySpaceHandler (
    UINT32                  Function,
    UINT32                  Address,
    UINT32                  BitWidth,
    UINT32                  *Value,
    void                    *Context);

ACPI_STATUS
AmlSystemIoSpaceHandler (
    UINT32                  Function,
    UINT32                  Address,
    UINT32                  BitWidth,
    UINT32                  *Value,
    void                    *Context);

ACPI_STATUS
AmlPciConfigSpaceHandler (
    UINT32                  Function,
    UINT32                  Address,
    UINT32                  BitWidth,
    UINT32                  *Value,
    void                    *Context);

ACPI_STATUS
AmlEmbeddedControllerSpaceHandler (
    UINT32                  Function,
    UINT32                  Address,
    UINT32                  BitWidth,
    UINT32                  *Value,
    void                    *Context);

ACPI_STATUS
AmlSmBusSpaceHandler (
    UINT32                  Function,
    UINT32                  Address,
    UINT32                  BitWidth,
    UINT32                  *Value,
    void                    *Context);




#endif /* __INTERPRETER_H__ */
