
/*
 *  ./acpica/subsystem/include/namesp.h - Namespace subcomponent prototypes and defines
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef __NAMESPACE_H__
#define __NAMESPACE_H__

#include <actypes.h>
#include <actables.h>
#include <acobject.h>
#include <pnp.h>


/* To search the entire name space, pass this as SearchBase */

#define NS_ALL                  ((ACPI_HANDLE)0)

/* 
 * Elements of NsProperties are bit significant
 * and should be one-to-one with values of ACPI_OBJECT_TYPE
 */
#define NSP_NORMAL              0
#define NSP_NEWSCOPE            1   /* a definition of this type opens a name scope */
#define NSP_LOCAL               2   /* suppress search of enclosing scopes */


/* Definitions of the predefined namespace names  */

#define NS_ROOT                 "/   "
#define NS_ROOT_PATH            "/"
#define NS_SYSTEM_BUS           "_SB_"


/* Flags for NsLookup, NsSearchAndEnter */

#define NS_NO_UPSEARCH          0
#define NS_SEARCH_PARENT        0x01
#define NS_DONT_OPEN_SCOPE      0x02
#define NS_NO_PEER_SEARCH       0x04

#define NS_WALK_UNLOCK          TRUE
#define NS_WALK_NO_UNLOCK       FALSE





ACPI_STATUS
NsWalkNamespace (
    ACPI_OBJECT_TYPE        Type, 
    ACPI_HANDLE             StartObject, 
    UINT32                  MaxDepth,
    BOOLEAN                 UnlockBeforeCallback,
    WALK_CALLBACK           UserFunction, 
    void                    *Context, 
    void                    **ReturnValue);


NAME_TABLE_ENTRY *
NsGetNextObject (
    ACPI_OBJECT_TYPE        Type, 
    NAME_TABLE_ENTRY        *Parent, 
    NAME_TABLE_ENTRY        *Child);


ACPI_STATUS
NsDeleteNamespaceByOwner (
    UINT16                  TableId);

void
NsFreeTableEntry (
    NAME_TABLE_ENTRY        *Entry);



/* Namespace loading - nsload */

ACPI_STATUS
NsParseTable (
    ACPI_TABLE_DESC         *TableDesc,
    NAME_TABLE_ENTRY        *Scope);

ACPI_STATUS
NsLoadTable (
    ACPI_TABLE_DESC         *TableDesc,
    NAME_TABLE_ENTRY        *Entry);

ACPI_STATUS
NsLoadTableByType (
    ACPI_TABLE_TYPE         TableType);



/*
 * Top-level namespace access - nsaccess
 */


ACPI_STATUS
NsRootInitialize (
    void);

ACPI_STATUS
NsLookup (
    ACPI_GENERIC_STATE      *ScopeInfo,
    char                    *Name, 
    ACPI_OBJECT_TYPE        Type, 
    OPERATING_MODE          InterpreterMode,
 UINT32     Flags,
    ACPI_WALK_STATE   *WalkState,
    NAME_TABLE_ENTRY        **RetEntry);


/*
 * Table allocation/deallocation - nsalloc
 */

NAME_TABLE_ENTRY *
NsAllocateNameTable (
    INT32                   NteEntries);

ACPI_STATUS
NsDeleteNamespaceSubtree (
    NAME_TABLE_ENTRY        *ParentHandle);

void
NsDetachObject (
    ACPI_HANDLE             Object);

void
NsDeleteScope (
    ACPI_HANDLE             Scope);


/*
 * Namespace modification - nsmodify
 */

ACPI_STATUS
NsUnloadNamespace (
    ACPI_HANDLE             Handle);

ACPI_STATUS
NsDeleteSubtree (
    ACPI_HANDLE             StartHandle);


/* 
 * Namespace dump/print utilities - nsdump
 */

void
NsDumpTables (
    ACPI_HANDLE             SearchBase, 
    INT32                   MaxDepth);

void
NsDumpEntry (
    ACPI_HANDLE             Handle,
    UINT32                  DebugLevel);

ACPI_STATUS
NsDumpPathname (
    ACPI_HANDLE             Handle, 
    char                    *Msg, 
    UINT32                  Level, 
    UINT32                  Component);

void
NsDumpRootDevices (
    void);

void
NsDumpObjects (
    ACPI_OBJECT_TYPE        Type, 
    UINT32                  MaxDepth, 
    UINT32                  OwnderId,
    ACPI_HANDLE             StartHandle);


/*
 * Namespace evaluation functions - nseval
 */

ACPI_STATUS
NsEvaluateByHandle (
    NAME_TABLE_ENTRY        *ObjectNte, 
    ACPI_OBJECT_INTERNAL    **Params,
    ACPI_OBJECT_INTERNAL    **ReturnObject);

ACPI_STATUS
NsEvaluateByName (
    char                    *Pathname, 
    ACPI_OBJECT_INTERNAL    **Params,
    ACPI_OBJECT_INTERNAL    **ReturnObject);

ACPI_STATUS
NsEvaluateRelative (
    NAME_TABLE_ENTRY        *ObjectNte, 
    char                    *Pathname, 
    ACPI_OBJECT_INTERNAL    **Params,
    ACPI_OBJECT_INTERNAL    **ReturnObject);

ACPI_STATUS
NsExecuteControlMethod (
    NAME_TABLE_ENTRY        *MethodEntry, 
    ACPI_OBJECT_INTERNAL    **Params,
    ACPI_OBJECT_INTERNAL    **ReturnObjDesc);

ACPI_STATUS
NsGetObjectValue (
    NAME_TABLE_ENTRY        *ObjectEntry,
    ACPI_OBJECT_INTERNAL    **ReturnObjDesc);


/*
 * Parent/Child/Peer utility functions - nsfamily
 */

ACPI_NAME
NsFindParentName (
    NAME_TABLE_ENTRY        *EntryToSearch);

INT32
NsExistDownstreamSibling (
    NAME_TABLE_ENTRY        *ThisEntry, 
    INT32                   Size, 
    NAME_TABLE_ENTRY        *Appendage);


/*
 * Scope manipulation - nsscope
 */

INT32
NsOpensScope (
    ACPI_OBJECT_TYPE        Type);

char *
NsNameOfScope (
    NAME_TABLE_ENTRY        *EntryToSearch);

char *
NsNameOfCurrentScope (
    ACPI_WALK_STATE         *WalkState);

ACPI_STATUS
NsHandleToPathname (
    ACPI_HANDLE             ObjHandle,
    UINT32                  *BufSize,
    char                    *UserBuffer);

BOOLEAN
NsPatternMatch (
    NAME_TABLE_ENTRY        *ObjEntry, 
    char                    *SearchFor);
        
ACPI_STATUS
NsNameCompare (
    ACPI_HANDLE             ObjHandle, 
    UINT32                  Level, 
    void                    *Context,
    void                    **ReturnValue);

void
NsLowFindNames (
    NAME_TABLE_ENTRY        *ThisEntry, 
    char                    *SearchFor,
    INT32                   *Count, 
    ACPI_HANDLE             List[], 
    INT32                   MaxDepth);

ACPI_HANDLE *
NsFindNames (
    char                    *SearchFor, 
    ACPI_HANDLE             SearchBase, 
    INT32                   MaxDepth);

ACPI_STATUS
NsGetNte (
    char                    *Name, 
    ACPI_HANDLE             Scope,
    NAME_TABLE_ENTRY      **OutNte);

/*
 * Object management for NTEs - nsobject
 */

ACPI_STATUS
NsAttachMethod (
    ACPI_HANDLE             ObjHandle, 
    UINT8                   *PcodeAddr, 
    UINT32                  PcodeLength);

ACPI_STATUS
NsAttachObject (
    ACPI_HANDLE             ObjHandle, 
    ACPI_HANDLE             Value, 
    UINT8                   ValTyp);


void *
NsCompareValue (
    ACPI_HANDLE             ObjHandle, 
    UINT32                  Level, 
    void                    *ObjDesc);

ACPI_HANDLE
NsFindAttachedObject (
    ACPI_OBJECT_INTERNAL    *ObjDesc, 
    ACPI_HANDLE             SearchBase, 
    INT32                   MaxDepth);


/*
 * Namespace searching and entry - nssearch
 */

ACPI_STATUS
NsSearchAndEnter (
    UINT32                  EntryName, 
    ACPI_WALK_STATE   *WalkState,
    NAME_TABLE_ENTRY        *NameTable,
    OPERATING_MODE          InterpreterMode, 
    ACPI_OBJECT_TYPE        Type, 
    UINT32                  Flags,
    NAME_TABLE_ENTRY        **RetEntry);

void
NsInitializeTable (
    NAME_TABLE_ENTRY        *NewTbl, 
    NAME_TABLE_ENTRY        *ParentScope, 
    NAME_TABLE_ENTRY        *ParentEntry);

ACPI_STATUS
NsSearchOneScope (
    UINT32                  EntryName, 
    NAME_TABLE_ENTRY        *NameTable, 
    ACPI_OBJECT_TYPE        Type, 
    NAME_TABLE_ENTRY        **RetEntry, 
    NS_SEARCH_DATA          *RetInfo);


/*
 * Utility functions - nsutils
 */

BOOLEAN
NsValidRootPrefix (
    char                    Prefix);

BOOLEAN
NsValidPathSeparator (
    char                    Sep);

ACPI_OBJECT_TYPE
NsGetType (
    ACPI_HANDLE             ObjHandle);

void *
NsGetAttachedObject (
    ACPI_HANDLE             ObjHandle);

INT32
NsLocal (
    ACPI_OBJECT_TYPE        Type);

ACPI_STATUS
NsInternalizeName (
    char                    *DottedName,
    char                    **ConvertedName);

ACPI_STATUS
NsExternalizeName (
    UINT32                  InternalNameLength,
    char                    *InternalName,
    UINT32                  *ConvertedNameLength,
    char                    **ConvertedName);

INT32
IsNsObject (
    ACPI_OBJECT_INTERNAL    *pOD);

INT32
NsMarkNS(
    void);

NAME_TABLE_ENTRY *
NsConvertHandleToEntry (
    ACPI_HANDLE             Handle);

ACPI_HANDLE
NsConvertEntryToHandle(
    NAME_TABLE_ENTRY *Nte);

void
NsTerminate (
    void);



#endif /* __NAMESPACE_H__ */
