/*
 *  ./acpica/subsystem/include/parser.h - AML Parser subcomponent prototypes and defines
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#ifndef _PARSER_H_
#define _PARSER_H_




#define OP_HAS_RETURN_VALUE         1

/* variable # arguments */

#define ACPI_VAR_ARGS               ACPI_UINT32_MAX

/* maximum virtual address */

#define ACPI_MAX_AML                ((UINT8 *)(~0UL))


/* psapi - Parser external interfaces */

ACPI_STATUS
PsxLoadTable (
    UINT8                   *PcodeAddr, 
    INT32                   PcodeLength);

ACPI_STATUS
PsxExecute (
    NAME_TABLE_ENTRY        *MethodEntry,
    ACPI_OBJECT_INTERNAL    **Params,
    ACPI_OBJECT_INTERNAL    **ReturnObjDesc);




BOOLEAN
PsIsNamespaceObjectOp (
    UINT16                  Opcode);
BOOLEAN
PsIsNamespaceOp (
    UINT16                  Opcode);





/* psargs - Parse AML opcode arguments */

UINT8 *
PsGetNextPackageEnd (
    ACPI_PARSE_STATE        *ParserState);

char *
PsGetNextNamestring (
    ACPI_PARSE_STATE        *ParserState);

void
PsGetNextSimpleArg (
    ACPI_PARSE_STATE        *ParserState,
    INT32                   ArgType,        /* type of argument */
    ACPI_GENERIC_OP         *Arg);           /* (OUT) argument data */

void
PsGetNextNamepath (
    ACPI_PARSE_STATE        *ParserState,
    ACPI_GENERIC_OP         *Arg,
    UINT32                  *ArgCount,
    BOOLEAN                 MethodCall);

ACPI_GENERIC_OP *
PsGetNextField (
    ACPI_PARSE_STATE        *ParserState);

ACPI_GENERIC_OP *
PsGetNextArg (
    ACPI_PARSE_STATE        *ParserState, 
    INT32                   ArgType, 
    UINT32                  *ArgCount);


/* psopcode - AML Opcode information */

ACPI_OP_INFO *
PsGetOpcodeInfo (
    UINT16                  Opcode);

char *
PsGetOpcodeName (
    UINT16                  Opcode);


/* psparse - top level parsing routines */

void
PsDeleteParseTree (
    ACPI_GENERIC_OP         *root);

ACPI_STATUS
PsParseLoop (
    ACPI_PARSE_STATE        *ParserState,
    ACPI_WALK_STATE         *WalkState);


ACPI_STATUS
PsParseAml (
    ACPI_GENERIC_OP         *StartScope,
    UINT8                   *Aml, 
    UINT32                  AmlSize);

ACPI_STATUS
PsParseTable (
    UINT8                   *aml, 
    INT32                   amlSize,
    INTERPRETER_CALLBACK    DescendingCallback,
    INTERPRETER_CALLBACK    AscendingCallback,
    ACPI_GENERIC_OP         **RootObject);

UINT16
PsPeekOpcode (
    ACPI_PARSE_STATE        *state);



/* psscope - Scope stack management routines */


ACPI_STATUS
PsInitScope (
    ACPI_PARSE_STATE        *ParserState,
    ACPI_GENERIC_OP         *Root);

ACPI_GENERIC_OP *
PsGetParentScope (
    ACPI_PARSE_STATE        *state);

BOOLEAN
PsHasCompletedScope (
    ACPI_PARSE_STATE        *ParserState);

void
PsPopScope (
    ACPI_PARSE_STATE        *ParserState,
    ACPI_GENERIC_OP         **Op,
    UINT32                  *ArgList);

ACPI_STATUS
PsPushScope (
    ACPI_PARSE_STATE        *ParserState,
    ACPI_GENERIC_OP         *Op, 
    UINT32                  RemainingArgs,
    UINT32                  ArgCount);

void
PsCleanupScope (
    ACPI_PARSE_STATE        *state);


/* pstree - parse tree manipulation routines */

void 
PsAppendArg(
    ACPI_GENERIC_OP         *op, 
    ACPI_GENERIC_OP         *arg);

ACPI_GENERIC_OP*
PsFind (
    ACPI_GENERIC_OP         *Scope, 
    char                    *Path,
    UINT16                  Opcode,
    UINT32                  Create);

ACPI_GENERIC_OP *
PsGetArg(
    ACPI_GENERIC_OP         *op, 
    UINT32                   argn);

ACPI_GENERIC_OP *
PsGetChild (
    ACPI_GENERIC_OP         *op);

ACPI_GENERIC_OP *
PsGetDepthNext (
    ACPI_GENERIC_OP         *Origin, 
    ACPI_GENERIC_OP         *Op);



/* pswalk - parse tree walk routines */

ACPI_STATUS
PsWalkParsedAml (
    ACPI_GENERIC_OP         *StartOp,
    ACPI_GENERIC_OP         *EndOp,
    ACPI_OBJECT_INTERNAL    *MthDesc,
 NAME_TABLE_ENTRY  *StartScope,
    ACPI_OBJECT_INTERNAL    **Params,
    ACPI_OBJECT_INTERNAL    **CallerReturnDesc,
    ACPI_OWNER_ID           OwnerId,
    INTERPRETER_CALLBACK    DescendingCallback,
    INTERPRETER_CALLBACK    AscendingCallback);

ACPI_STATUS
PsGetNextWalkOp (
    ACPI_WALK_STATE         *WalkState,
    ACPI_GENERIC_OP         *Op,
    INTERPRETER_CALLBACK    AscendingCallback);



/* psutils - parser utilities */

void
PsInitOp (
    ACPI_GENERIC_OP         *op,
    UINT16                  opcode);

ACPI_GENERIC_OP * 
PsAllocOp (
    UINT16                  opcode);

void
PsFreeOp (
    ACPI_GENERIC_OP         *Op);

void
PsDeleteParseCache (
    void);

BOOLEAN
PsIsLeadingChar (
    INT32                   c);

BOOLEAN
PsIsPrefixChar (
    INT32                   c);

BOOLEAN
PsIsNamedOp (
    UINT16                  opcode);

BOOLEAN
PsIsNamedObjectOp (
    UINT16                  opcode);

BOOLEAN
PsIsDeferredOp (
    UINT16                  opcode);

BOOLEAN
PsIsBytelistOp(
    UINT16                  opcode);

BOOLEAN
PsIsFieldOp(
    UINT16                  opcode);

ACPI_NAMED_OP*
PsToNamedOp(
    ACPI_GENERIC_OP         *op);

ACPI_DEFERRED_OP *
PsToDeferredOp (
    ACPI_GENERIC_OP         *Op);

ACPI_BYTELIST_OP*
PsToBytelistOp(
    ACPI_GENERIC_OP         *op);

UINT32
PsGetName(
    ACPI_GENERIC_OP         *op);

void
PsSetName(
    ACPI_GENERIC_OP         *op, 
    UINT32                  name);


/* psdump - display parser tree */

INT32
PsSprintPath (
    char                    *BufferStart,
    UINT32                  BufferSize,
    ACPI_GENERIC_OP         *Op);

INT32
PsSprintOp (
    char                    *BufferStart, 
    UINT32                  BufferSize, 
    ACPI_GENERIC_OP         *Op);

void 
PsShow (
    ACPI_GENERIC_OP         *op);




#endif /* _PARSER_H_ */
