
/*
 *  ./acpica/subsystem/include/tables.h - ACPI table management
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef __TABLES_H__
#define __TABLES_H__

#include <actypes.h>
#include <actables.h>



/* Used in TbMapAcpiTable for size parameter if table header is to be used */

#define SIZE_IN_HEADER          0


/* Used if RSDT is loaded from a buffer */
/* TBD: is this obsolete? */

#define ACPILIB_DATA_FILE_VERSION "ADF-001"



ACPI_STATUS
TbHandleToObject (
    UINT16                  TableId,
    ACPI_TABLE_DESC         **TableDesc);


/*
 * Tbfac - FACP, FACS utilities
 */

ACPI_STATUS
TbGetTableFacs (
    char                    *BufferPtr,
    ACPI_TABLE_DESC         *TableInfo);


/*
 * Tbget - Table "get" routines
 */

ACPI_STATUS
TbGetTablePtr (
    ACPI_TABLE_TYPE         TableType, 
    UINT32                  Instance,
    ACPI_TABLE_HEADER       **TablePtrLoc);

ACPI_STATUS
TbGetTable (
    void                    *PhysicalAddress, 
    char                    *BufferPtr,
    ACPI_TABLE_DESC         *TableInfo);


/* 
 * Tbgetall - Get all firmware ACPI tables
 */

ACPI_STATUS
TbGetAllTables (
    UINT32                  NumberOfTables, 
    char                    *BufferPtr);


/*
 * Tbinstall - Table installation
 */

ACPI_STATUS
TbInstallTable (
    char                    *TablePtr,
    ACPI_TABLE_DESC         *TableInfo);

ACPI_STATUS
TbRecognizeTable (
    char                    *TablePtr,
    ACPI_TABLE_DESC         *TableInfo);

ACPI_STATUS
TbInitTableDescriptor (
    ACPI_TABLE_TYPE         TableType,
    ACPI_TABLE_DESC         *TableInfo);


/*
 * Tbremove - Table removal and deletion
 */

void
TbDeleteAcpiTables (
    void);

void
TbDeleteAcpiTable (
    ACPI_TABLE_TYPE         Type);

ACPI_TABLE_DESC *
TbDeleteSingleTable (
    ACPI_TABLE_DESC         *TableDesc);

void
TbFreeAcpiTablesOfType (
    ACPI_TABLE_DESC         *TableInfo);


/*
 * Tbrsd - RSDP, RSDT utilities
 */

ACPI_STATUS
TbGetTableRsdt (
    UINT32                  *NumberOfTables);

char *
TbScanMemoryForRsdp (
    char                    *StartAddress, 
    UINT32                  Length);

ACPI_STATUS
TbFindRsdp (
    ACPI_TABLE_DESC         *TableInfo);


/*
 * Tbutils - common table utilities
 */

BOOLEAN
TbSystemTablePointer (
    void                    *Where);

ACPI_STATUS
TbMapAcpiTable (
    void                    *PhysicalAddress,
    UINT32                  *Size,
 void     **LogicalAddress);

ACPI_STATUS
TbVerifyTableChecksum (
    void                    *TableHeader); 

UINT8
TbChecksum (
    void                    *Buffer,
    UINT32                  Length);

ACPI_STATUS
TbValidateTableHeader (
    ACPI_TABLE_HEADER       *TableHeader);


#endif /* __TABLES_H__ */
