
/*
 *  ./acpica/subsystem/interpreter/iapi.c - External interpreter interfaces
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __IAPI_C__

#include <acpi.h>
#include <interp.h>


#define _COMPONENT          INTERPRETER
        MODULE_NAME         ("iapi");


/* 
 * DEFINE_AML_GLOBALS is tested in amlcode.h
 * to determine whether certain global names should be "defined" or only
 * "declared" in the current compilation.  This enhances maintainability
 * by enabling a single header file to embody all knowledge of the names
 * in question.
 *
 * Exactly one module of any executable should #define DEFINE_GLOBALS
 * before #including the header files which use this convention.  The
 * names in question will be defined and initialized in that module,
 * and declared as extern in all other modules which #include those
 * header files.
 */

#define DEFINE_AML_GLOBALS
#include <amlcode.h>
#include <parser.h>
#include <namesp.h>




ACPI_STATUS
AmlExecuteMethod (
    NAME_TABLE_ENTRY        *MethodEntry,
    ACPI_OBJECT_INTERNAL    **Params,
    ACPI_OBJECT_INTERNAL    **ReturnObjDesc)
{
    ACPI_STATUS             Status;


    FUNCTION_TRACE ("AmlExecuteMethod");


    /* 
     * The point here is to lock the interpreter and call the low
     * level execute.
     */

    AmlEnterInterpreter ();

    Status = PsxExecute (MethodEntry, Params, ReturnObjDesc);

    AmlExitInterpreter ();

    return_ACPI_STATUS (Status);
}



