
/*
 *  ./acpica/subsystem/interpreter/isutils.c - interpreter/scanner utilities
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __ISUTILS_C__

#include <acpi.h>
#include <parser.h>
#include <interp.h>
#include <amlcode.h>
#include <namesp.h>
#include <events.h>

#define _COMPONENT          INTERPRETER
        MODULE_NAME         ("isutils");

static char                 hex[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};




void
AmlEnterInterpreter (void)
{
    FUNCTION_TRACE ("AmlEnterInterpreter");

    
    CmAcquireMutex (MTX_EXECUTE);

    return_VOID;
}



void
AmlExitInterpreter (void)
{
    FUNCTION_TRACE ("AmlExitInterpreter");


    CmReleaseMutex (MTX_EXECUTE);

    return_VOID;
}



BOOLEAN
AmlValidateObjectType (
    ACPI_OBJECT_TYPE        Type)
{

    if ((Type > ACPI_TYPE_MAX && Type < INTERNAL_TYPE_BEGIN) ||
        (Type > INTERNAL_TYPE_MAX))
    {
        return FALSE;
    }

    return TRUE;
}



void
AmlAppendOperandDiag (
    char                    *FileName, 
    INT32                   LineNum, 
    UINT16                  OpCode, 
    ACPI_OBJECT_INTERNAL    **Operands,
    INT32                   NumOperands)
{
/* TBD:    ACPI_OBJECT_INTERNAL    MthDesc; */


/* TBD: Rparser only    AmlGetCurrentLocation (&MthDesc); */

    DEBUG_PRINT (ACPI_ERROR, (" [%s:%d, opcode = %s AML offset %04x]\n",
                    FileName, LineNum, PsGetOpcodeName (OpCode), NULL /*MthDesc.Method.Pcode*/));

    if (GetDebugLevel () > 0)
    {
        DUMP_OPERANDS (Operands, IMODE_Execute, PsGetOpcodeName (OpCode),
                      NumOperands, "after PrepStack failed");
    }
}




UINT32 
AmlBufSeq (void)
{
    
    return ++Gbl_BufSeq;
}




BOOLEAN
AmlAcquireGlobalLock (
    UINT32                  Rule)
{
    BOOLEAN                 Locked = FALSE;


    FUNCTION_TRACE ("AmlAcquireGlobalLock");


    /*  Only attempt lock if the Rule says so */
    
    if (Rule == (UINT32) GLOCK_AlwaysLock)
    {   
        /*  OK to get the lock   */
        
        if (EvAcquireGlobalLock () != AE_OK)
        {
            /*  
             * lock ownership failed
             */
            DEBUG_PRINT (ACPI_ERROR, ("Get Global Lock Failed!!\n"));
        }

        else
        {
            Gbl_GlobalLockSet = TRUE;
            Locked = TRUE;
        }
    }

    return_VALUE (Locked);
}



ACPI_STATUS
AmlReleaseGlobalLock (
    BOOLEAN                 LockedByMe)
{

    FUNCTION_TRACE ("AmlReleaseGlobalLock");


    /* Only attempt unlock if the caller locked it */

    if (LockedByMe)
    {
        /* Double check against the global flag */

        if (Gbl_GlobalLockSet)
        {
            /* OK, now release the lock */

            EvReleaseGlobalLock ();
            Gbl_GlobalLockSet = FALSE;
        }

        else
        {
            /* Sorry, the lock wasn't set, according to our records */

            DEBUG_PRINT (ACPI_ERROR, ("Global lock was not set\n"));
        }
    }


    return_ACPI_STATUS (AE_OK);
}



INT32
AmlDigitsNeeded (
    INT32                   val, 
    INT32                   base)
{
    INT32                   NumDigits = 0;


    FUNCTION_TRACE ("AmlDigitsNeeded");


    if (base < 1)
    {   
        /*  impossible base */

        REPORT_ERROR ("AmlDigitsNeeded: Impossible base");
    }

    else
    {
        for (NumDigits = 1 + (val < 0) ; val /= base ; ++NumDigits)
        { ; }
    }

    return_VALUE (NumDigits);
}



UINT32
_ntohl (
    UINT32                  Value)
{
    union 
    {
        UINT32              Value;
        char                Bytes[4];
    } Out; 

    union
    {
        UINT32              Value;
        char                Bytes[4];
    } In; 


    In.Value = Value;

    Out.Bytes[0] = In.Bytes[3];
    Out.Bytes[1] = In.Bytes[2];
    Out.Bytes[2] = In.Bytes[1];
    Out.Bytes[3] = In.Bytes[0];

    return Out.Value;
}



ACPI_STATUS
AmlEisaIdToString (
    UINT32                  NumericId,
    char                    *OutString)
{
    UINT32                  id;

    
    id = _ntohl (NumericId);                    /* swap to big-endian to get contiguous bits */
    
    OutString[0] = (char) ('@' + ((id >> 26) & 0x1f));
    OutString[1] = (char) ('@' + ((id >> 21) & 0x1f));
    OutString[2] = (char) ('@' + ((id >> 16) & 0x1f));
    OutString[3] = hex[(id >> 12) & 0xf];
    OutString[4] = hex[(id >> 8) & 0xf];
    OutString[5] = hex[(id >> 4) & 0xf];
    OutString[6] = hex[id & 0xf];
    OutString[7] = 0;
    
    return AE_OK;
}

