
/*
 *  ./acpica/subsystem/methods/mtadr.c - acpi EvaluateNumeric()
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __MTADR_C__

#include <acpi.h>
#include <methods.h>
#include <namesp.h>


#define _COMPONENT          MISCELLANEOUS
        MODULE_NAME         ("mtadr");



ACPI_STATUS 
EvaluateNumeric (
    char                    *ObjectName,
    NAME_TABLE_ENTRY        *AcpiDevice, 
    UINT32                  *Address)
{
    ACPI_OBJECT_INTERNAL    *ObjDesc;
    ACPI_STATUS             Status;

    
    FUNCTION_TRACE ("EvaluateNumeric");


    /* Execute the method */

    Status = NsEvaluateRelative (AcpiDevice, ObjectName, NULL, &ObjDesc);
    if (ACPI_FAILURE (Status))
    {
        if (Status == AE_NOT_FOUND)
        {
            DEBUG_PRINT (ACPI_INFO, ("%s on %4.4s was not found\n", ObjectName,
                                        &((NAME_TABLE_ENTRY *) AcpiDevice)->Name));
        }
        else
        {
            DEBUG_PRINT (ACPI_ERROR, ("%s on %4.4s failed with status %4.4x\n", ObjectName,
                                        &((NAME_TABLE_ENTRY *) AcpiDevice)->Name, Status));
        }

        return_ACPI_STATUS (Status);
    }


    /* Did we get a return object? */

    if (!ObjDesc)
    {
        DEBUG_PRINT (ACPI_ERROR, ("No object was returned from %s\n", ObjectName));
        return_ACPI_STATUS (AE_TYPE);
    }

    /* Is the return object of the correct type? */

    if (ObjDesc->Common.Type != ACPI_TYPE_Number)
    {
        Status = AE_TYPE;
        DEBUG_PRINT (ACPI_ERROR, ("Type returned from %s was not a number: %d \n",
                                    ObjectName, ObjDesc->Common.Type));
    }
    else
    {
        /* 
         * Since the structure is a union, setting any field will set all
         * of the variables in the union
         */
        *Address = ObjDesc->Number.Value;
    }

    /* On exit, we must delete the return object */

    CmRemoveReference (ObjDesc);

    return_ACPI_STATUS (Status);
}
