
/*
 *  ./acpica/subsystem/methods/mtsta.c - acpi Execute_STA()
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __MTSTA_C__

#include <acpi.h>
#include <methods.h>
#include <namesp.h>


#define _COMPONENT          MISCELLANEOUS
        MODULE_NAME         ("mtsta");



ACPI_STATUS 
Execute_STA (
    NAME_TABLE_ENTRY        *AcpiDevice, 
    UINT32                  *Flags)
{
    ACPI_OBJECT_INTERNAL    *ObjDesc;
    ACPI_STATUS             Status;


    FUNCTION_TRACE ("Execute_STA");



    /* Execute the method */

    Status = NsEvaluateRelative (AcpiDevice, METHOD_NAME__STA, NULL, &ObjDesc);
    if (ACPI_FAILURE (Status))
    {
        if (Status == AE_NOT_FOUND)
        {
            DEBUG_PRINT (ACPI_INFO, ("_STA on %4.4s was not found\n",
                                        &((NAME_TABLE_ENTRY *) AcpiDevice)->Name));
        }

        else
        {
            DEBUG_PRINT (ACPI_ERROR, ("_STA on %4.4s failed with status %4.4x\n",
                                        &((NAME_TABLE_ENTRY *) AcpiDevice)->Name, Status));
        }

        return_ACPI_STATUS (Status);
    }


    /* Did we get a return object? */

    if (!ObjDesc)
    {
        DEBUG_PRINT (ACPI_ERROR, ("No object was returned from _ADR\n"));
        return_ACPI_STATUS (AE_TYPE);
    }

    /* Is the return object of the correct type? */

    if (ObjDesc->Common.Type != ACPI_TYPE_Number)
    {
        Status = AE_TYPE;
        DEBUG_PRINT (ACPI_ERROR, ("Type returned from _STA was not a number: %d \n",
                                    ObjDesc->Common.Type));
    }

    else
    {
        /* Extract the status flags */

        *Flags = ObjDesc->Number.Value;
    }


    /* On exit, we must delete the return object */

    CmRemoveReference (ObjDesc);

    return_ACPI_STATUS (Status);
}
