
/*
 *  ./acpica/subsystem/methods/mtuid.c - acpi Execute_UID()
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __MTUID_C__

#include <acpi.h>
#include <methods.h>
#include <namesp.h>


#define _COMPONENT          MISCELLANEOUS
        MODULE_NAME         ("mtuid");



ACPI_STATUS 
Execute_UID (
    NAME_TABLE_ENTRY        *AcpiDevice, 
    DEVICE_ID               *Uid)
{
    ACPI_OBJECT_INTERNAL    *ObjDesc;
    ACPI_STATUS             Status;



    /* Execute the method */

    Status = NsEvaluateRelative (AcpiDevice, METHOD_NAME__UID, NULL, &ObjDesc);
    if (ACPI_FAILURE (Status))
    {
        if (Status == AE_NOT_FOUND)
        {
            DEBUG_PRINT (ACPI_INFO, ("_UID on %4.4s was not found\n",
                                        &((NAME_TABLE_ENTRY *) AcpiDevice)->Name));
        }

        else
        {
            DEBUG_PRINT (ACPI_ERROR, ("_UID on %4.4s failed with status %4.4x\n",
                                        &((NAME_TABLE_ENTRY *) AcpiDevice)->Name, Status));
        }

     return (Status);
    }

    /* Did we get a return object? */

    if (!ObjDesc)
    {
        DEBUG_PRINT (ACPI_ERROR, ("No object was returned from _ADR\n"));
        return (AE_TYPE);
    }

    /*
     *  A _UID can return either a Number (32 bit compressed EISA ID) or a string
     */

    if ((ObjDesc->Common.Type != ACPI_TYPE_Number) &&
        (ObjDesc->Common.Type != ACPI_TYPE_String))
    {
        Status = AE_TYPE;
        DEBUG_PRINT (ACPI_ERROR, ("Type returned from _UID was not a number or string: %d \n",
                                    ObjDesc->Common.Type));
    }

    else
    {
        if (ObjDesc->Common.Type == ACPI_TYPE_Number)
        {
            /* Convert the Numeric HID to string */

            Uid->Data.Number = ObjDesc->Number.Value;
        }

        else
        {
            /* Copy the String HID from the returned object */

            Uid->Data.StringPtr = ObjDesc->String.Pointer;
            Uid->Type = STRING_PTR_DEVICE_ID;
        }
    }


    /* On exit, we must delete the return object */

    CmRemoveReference (ObjDesc);

 return (Status);
}  

