
/*
 *  ./acpica/subsystem/namespace/nsfamily.c - Utilities for accessing parents and siblings
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __NSFAMILY_C__

#include <acpi.h>
#include <namesp.h>

#define _COMPONENT          NAMESPACE
        MODULE_NAME         ("nsfamily");




ACPI_NAME
NsFindParentName (
    NAME_TABLE_ENTRY        *EntryToSearch) 
{
    NAME_TABLE_ENTRY        *ParentEntry;


    FUNCTION_TRACE ("FindParentName");


    if (EntryToSearch)
    {
        /* Valid entry.  Get the parent Nte */

        ParentEntry = EntryToSearch->ParentEntry;
        if (ParentEntry)
        {
            DEBUG_PRINT (TRACE_EXEC, ("Parent Entry: %p, [%4.4s]\n", 
                            ParentEntry, &ParentEntry->Name));

            if (ParentEntry->Name)
            {
                return_VALUE (ParentEntry->Name);
            }
        }

        DEBUG_PRINT (TRACE_EXEC,
                        ("FindParentName: unable to find parent of (%4.4s)\n",
                        &EntryToSearch->Name));
    }


    return_VALUE ((ACPI_NAME) *(UINT32 *) "????");
}


INT32
NsExistDownstreamSibling (
    NAME_TABLE_ENTRY        *ThisEntry, 
    INT32                   Size, 
    NAME_TABLE_ENTRY        *Appendage)
{

    if (!ThisEntry)
    {
        return FALSE;
    }

    if (ThisEntry->Name)
    {
        return TRUE;
    }
        
    if (ThisEntry->NextEntry)
    {
        return TRUE;
    }

    return FALSE;
}

