
/*
 *  ./acpica/subsystem/namespace/nsscope.c - Scope manipulation
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __NSSCOPE_C__

#include <acpi.h>
#include <amlcode.h>
#include <interp.h>
#include <namesp.h>


#define _COMPONENT          NAMESPACE
        MODULE_NAME         ("nsscope");




INT32
NsOpensScope (
    ACPI_OBJECT_TYPE        Type)
{
    FUNCTION_TRACE ("NsOpensScope");


    if (!CmValidObjectType (Type))
    {
        /* type code out of range  */

        REPORT_WARNING ("NsOpensScope: Invalid Object Type");
        return_VALUE (NSP_NORMAL);
    }

    return_VALUE (((INT32) Gbl_NsProperties[Type]) & NSP_NEWSCOPE);
}



ACPI_STATUS
NsGetNte (
    char                    *Pathname, 
    ACPI_HANDLE             InScope, 
    NAME_TABLE_ENTRY        **OutNte)
{
    ACPI_GENERIC_STATE      ScopeInfo;
    ACPI_STATUS             Status;
    NAME_TABLE_ENTRY        *ObjEntry = NULL;
    char                    *InternalPath = NULL;


    FUNCTION_TRACE_PTR ("NsGetNte", Pathname);

    ScopeInfo.Scope.Entry = InScope;

    /* Ensure that the namespace has been initialized */

    if (!Gbl_RootObject->Scope)
    {
        return_ACPI_STATUS (AE_NO_NAMESPACE);
    }

    if (!Pathname)
    {
        return_ACPI_STATUS (AE_BAD_PARAMETER);
    }


    /* Convert path to internal representation */

    Status = NsInternalizeName (Pathname, &InternalPath);
    if (ACPI_FAILURE (Status))
    {
        return_ACPI_STATUS (Status);
    }


    CmAcquireMutex (MTX_NAMESPACE);

    /* NS_ALL means start from the root */
    
    if (NS_ALL == ScopeInfo.Scope.Entry)
    {
        ScopeInfo.Scope.Entry = Gbl_RootObject->Scope;
    }

    else
    {
        ScopeInfo.Scope.Entry = NsConvertHandleToEntry (InScope);
        if (!ScopeInfo.Scope.Entry)
        {
            Status = AE_BAD_PARAMETER;
            goto UnlockAndExit;
        }
    }

    /* Lookup the name in the namespace */

    Status = NsLookup (&ScopeInfo, InternalPath, ACPI_TYPE_Any, IMODE_Execute, 
                                NS_NO_UPSEARCH | NS_DONT_OPEN_SCOPE, NULL, &ObjEntry);
    if (Status != AE_OK)
    {
        DEBUG_PRINT (ACPI_INFO, ("NsGetNte: %s, %s\n",
                        InternalPath, CmFormatException (Status)));
    }

    /* Return what was wanted - the NTE that matches the name */

    *OutNte = ObjEntry;


UnlockAndExit:

    CmReleaseMutex (MTX_NAMESPACE);

    /* Cleanup */

    CmFree (InternalPath);

    return_ACPI_STATUS (Status);
}


