
/*
 *  ./acpica/subsystem/parser/psapi.c - Parser external interfaces
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __PSAPI_C__

#include <acpi.h>
#include <parser.h>
#include <dispatch.h>
#include <interp.h>
#include <amlcode.h>
#include <namesp.h>


#define _COMPONENT          PARSER
        MODULE_NAME         ("psapi");


char    *Gbl_ParserId = "Non-recursive AML Parser";




ACPI_STATUS
PsxExecute (
    NAME_TABLE_ENTRY        *MethodEntry,
    ACPI_OBJECT_INTERNAL    **Params,
    ACPI_OBJECT_INTERNAL    **ReturnObjDesc)
{
    ACPI_STATUS             Status;
    ACPI_OBJECT_INTERNAL    *ObjDesc;
    UINT32                  i;


    FUNCTION_TRACE ("PsxExecute");


    /* Validate the NTE and get the attached object */

    if (!MethodEntry)
    {
        return_ACPI_STATUS (AE_NULL_ENTRY);
    }

    ObjDesc = NsGetAttachedObject (MethodEntry);
    if (!ObjDesc)
    {
        return_ACPI_STATUS (AE_NULL_OBJECT);
    }

    /* Parse method if necessary, wait on concurrency semaphore */

    Status = DsBeginMethodExecution (MethodEntry, ObjDesc);
    if (ACPI_FAILURE (Status))
    {
        return_ACPI_STATUS (Status);
    }

    if (Params)
    {
        /* The caller "owns" the parameters, so give them an extra reference */

        for (i = 0; Params[i]; i++)
        {
            CmAddReference (Params[i]);
        }
    }

    /* 
     * Method is parsed and ready to execute
     * The walk of the parse tree is where we actually execute the method
     */

    DEBUG_PRINT (ACPI_INFO, ("PsxExecute: **** Begin Method Execution **** Entry=%p obj=%p\n",
                    MethodEntry, ObjDesc));

    Status = PsWalkParsedAml (ObjDesc->Method.ParserOp, ObjDesc->Method.ParserOp, ObjDesc, MethodEntry->Scope, Params,
                                ReturnObjDesc, ObjDesc->Method.OwningId, DsExecBeginOp, DsExecEndOp);
    
    if (Params)
    {
        /* Take away the extra reference that we gave the parameters above */

        for (i = 0; Params[i]; i++)
        {
            CmUpdateObjectReference (Params[i], REF_DECREMENT);
        }
    }


    /* 
     * Normal exit is with Status == AE_RETURN_VALUE when a ReturnOp has been executed,
     * or with Status == AE_PENDING at end of AML block (end of Method code)
     */

    if (*ReturnObjDesc)
    {
        DEBUG_PRINT (ACPI_INFO, ("Method returned ObjDesc=%X\n", *ReturnObjDesc));
        DUMP_STACK_ENTRY (*ReturnObjDesc);

        Status = AE_CTRL_RETURN_VALUE;
    }


    return_ACPI_STATUS (Status);
}




