/*
 *  ./acpica/subsystem/resources/rsaddr16.c - RsAddress16Resource
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __RSADDR16_C__

#include <acpi.h>

#define _COMPONENT          RESOURCE_MANAGER
        MODULE_NAME         ("rsaddr16");



ACPI_STATUS
RsAddress16Resource (
    UINT8                   *ByteStreamBuffer,
    UINT32                  *BytesConsumed,
    UINT8                   **OutputBuffer,
    UINT32                  *StructureSize)
{
    UINT8                   *Buffer = ByteStreamBuffer;
    RESOURCE                *OutputStruct = (RESOURCE *) * OutputBuffer;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;        
    UINT32                  Index;
    UINT32                  StructSize = sizeof(ADDRESS16_RESOURCE) + RESOURCE_LENGTH_NO_DATA;
    

    FUNCTION_TRACE ("RsAddress16Resource");

    /*
     * Point past the Descriptor to get the number of bytes consumed
     */ 
    Buffer += 1;

    Temp16 = *(UINT16 *)Buffer;

    *BytesConsumed = Temp16 + 3;

    OutputStruct->Id = Address16;

    OutputStruct->Length = StructSize;

    /*
     * Get the Resource Type (Byte3)
     */ 
    Buffer += 2;
    Temp8 = *Buffer;

    /* Values 0-2 are valid */
    if (Temp8 > 2)
    {
        return_ACPI_STATUS (AE_ERROR);
    }

    OutputStruct->Data.Address16.ResourceType = Temp8 & 0x03;

    /*
     * Get the General Flags (Byte4)
     */ 
    Buffer += 1;
    Temp8 = *Buffer;

    /*
     * Producer / Consumer
     */
    OutputStruct->Data.Address16.ProducerConsumer = Temp8 & 0x01;
    
    /*
     * Decode
     */
    OutputStruct->Data.Address16.Decode = (Temp8 >> 1) & 0x01;

    /*
     * Min Address Fixed
     */
    OutputStruct->Data.Address16.MinAddressFixed = (Temp8 >> 2) & 0x01;

    /*
     * Max Address Fixed
     */
    OutputStruct->Data.Address16.MaxAddressFixed = (Temp8 >> 3) & 0x01;

    /*
     * Get the Type Specific Flags (Byte5)
     */ 
    Buffer += 1;
    Temp8 = *Buffer;

    if (MEMORY_RANGE == OutputStruct->Data.Address16.ResourceType)
    {
        OutputStruct->Data.Address16.Attribute.Memory.ReadWriteAttribute = (UINT16) (Temp8 & 0x01);
        OutputStruct->Data.Address16.Attribute.Memory.CacheAttribute = (UINT16) ((Temp8 >> 1) & 0x0F);
    }

    else
    {
        if (IO_RANGE == OutputStruct->Data.Address16.ResourceType)
        {
            OutputStruct->Data.Address16.Attribute.Io.RangeAttribute = (UINT16) (Temp8 & 0x03);
        }

        else
        {
            /* BUS_NUMBER_RANGE == Address32Data->ResourceType */
            /* Nothing needs to be filled in */
        }
    }

    /*
     * Get Granularity (Bytes 6-7)
     */ 
    Buffer += 1;
    Temp16 = *(UINT16 *) Buffer;

    OutputStruct->Data.Address16.Granularity = Temp16;

    /*
     * Get MinAddressRange (Bytes 8-9)
     */ 
    Buffer += 2;
    Temp16 = *(UINT16 *) Buffer;

    OutputStruct->Data.Address16.MinAddressRange = Temp16;

    /*
     * Get MaxAddressRange (Bytes 10-11)
     */ 
    Buffer += 2;
    Temp16 = *(UINT16 *) Buffer;

    OutputStruct->Data.Address16.MaxAddressRange = Temp16;

    /*
     * Get AddressTranslationOffset (Bytes 12-13)
     */ 
    Buffer += 2;
    Temp16 = *(UINT16 *) Buffer;

    OutputStruct->Data.Address16.AddressTranslationOffset = Temp16;

    /*
     * Get AddressLength (Bytes 14-15)
     */ 
    Buffer += 2;
    Temp16 = *(UINT16 *) Buffer;

    OutputStruct->Data.Address16.AddressLength = Temp16;

    /*
     * Resource Source Index (if present)
     */
    Buffer += 2;

    /*
     * This will leave us pointing to the Resource Source Index
     *  If it is present, then save it off and calculate the
     *  pointer to where the null terminated string goes:
     *  Each Interrupt takes 32-bits + the 5 bytes of the 
     *  stream that are default.
     */
    if (*BytesConsumed > 16)
    {
        /* Dereference the Index */

        Temp8 = *Buffer;

        OutputStruct->Data.Address16.ResourceSourceIndex = (UINT32)Temp8;

        /* Point to the String */

        Buffer += 1;
        
        /* Copy the string into the buffer */

        Index = 0;

        while(0x00 != *Buffer)
        {
            OutputStruct->Data.Address16.ResourceSource[Index] = *Buffer;
            Buffer += 1;
            Index += 1;
        }

        /*
         * Add the terminating null
         */
        OutputStruct->Data.Address16.ResourceSource[Index] = 0x00;

        OutputStruct->Data.Address16.ResourceSourceStringLength = Index + 1;

        /*
         * In order for the StructSize to fall on a 32-bit boundry,
         *  calculate the length of the string and expand the 
         *  StructSize to the next 32-bit boundry.
         */
        Temp8 = (UINT8) (Index + 1);

        if(0 == Temp8 % 4)
        {
            StructSize += Temp8;
        }
        else
        {
            StructSize += ((Temp8 / 4) * 4) + 4;
        }
    }
    else
    {
        OutputStruct->Data.Address16.ResourceSourceIndex = 0x00;
        OutputStruct->Data.Address16.ResourceSourceStringLength = 0;
        OutputStruct->Data.Address16.ResourceSource[0] = 0x00;
    }
    
    /*
     * Set the Length parameter
     */
    OutputStruct->Length = StructSize;

    /*
     * Return the final size of the structure
     */
    *StructureSize = StructSize;

    return_ACPI_STATUS (AE_OK);
}



ACPI_STATUS
RsAddress16Stream (
    RESOURCE                *LinkedList,
    UINT8                   **OutputBuffer,
    UINT32                  *BytesConsumed)
{
    UINT8                   *Buffer = *OutputBuffer;
    UINT16                  *LengthField;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;
    UINT8                   *TempPointer = NULL;
    

    FUNCTION_TRACE ("RsAddress16Stream");

    /*
     * The descriptor field is static
     */ 
    *Buffer = 0x88;

    Buffer += 1;

    /*
     * Set a pointer to the Length field - to be filled in later
     */

    LengthField = (UINT16 *) Buffer;

    Buffer += 2;

    /*
     * Set the Resource Type (Memory, Io, BusNumber)
     */
    Temp8 = (UINT8) (LinkedList->Data.Address16.ResourceType & 0x03);

    *Buffer = Temp8;

    Buffer += 1;
    
    /*
     * Set the general flags
     */
    Temp8 = (UINT8) (LinkedList->Data.Address16.ProducerConsumer & 0x01);

    Temp8 |= (LinkedList->Data.Address16.Decode & 0x01) << 1;

    Temp8 |= (LinkedList->Data.Address16.MinAddressFixed & 0x01) << 2;

    Temp8 |= (LinkedList->Data.Address16.MaxAddressFixed & 0x01) << 3;

    *Buffer = Temp8;

    Buffer += 1;
    
    /*
     * Set the type specific flags
     */
    Temp8 = 0;
    
    if (MEMORY_RANGE == LinkedList->Data.Address16.ResourceType)
    {
        Temp8 = (UINT8) (LinkedList->Data.Address16.Attribute.Memory.ReadWriteAttribute & 0x01);
    
        Temp8 |= (LinkedList->Data.Address16.Attribute.Memory.CacheAttribute & 0x0F) << 1;
    }
    else if (IO_RANGE == LinkedList->Data.Address16.ResourceType)
    {
        Temp8 = (UINT8) (LinkedList->Data.Address16.Attribute.Io.RangeAttribute & 0x03);
    }

    *Buffer = Temp8;

    Buffer += 1;
    
    /*
     * Set the address space granularity
     */
    Temp16 = (UINT16) LinkedList->Data.Address16.Granularity;

    *(UINT16 *) Buffer = Temp16;


    Buffer += 2;

    /*
     * Set the address range minimum
     */
    Temp16 = (UINT16) LinkedList->Data.Address16.MinAddressRange;

    *(UINT16 *) Buffer = Temp16;

    Buffer += 2;

    /*
     * Set the address range maximum
     */
    Temp16 = (UINT16) LinkedList->Data.Address16.MaxAddressRange;

    *(UINT16 *) Buffer = Temp16;

    Buffer += 2;

    /*
     * Set the address translation offset
     */
    Temp16 = (UINT16) LinkedList->Data.Address16.AddressTranslationOffset;

    *(UINT16 *) Buffer = Temp16;

    Buffer += 2;

    /*
     * Set the address length
     */
    Temp16 = (UINT16) LinkedList->Data.Address16.AddressLength;

    *(UINT16 *) Buffer = Temp16;

    Buffer += 2;

    /*
     * Resource Source Index and Resource Source are optional
     */
    if(0 != LinkedList->Data.Address16.ResourceSourceStringLength)
    {
        Temp8 = (UINT8) LinkedList->Data.Address16.ResourceSourceIndex;

        *Buffer = Temp8;

        Buffer += 1;

        TempPointer = Buffer;
        
        /*
         * Copy the string
         */
        STRCPY (TempPointer, LinkedList->Data.Address16.ResourceSource);

        /*
         * Buffer needs to be set to the length of the sting + one for the 
         *  terminating null
         */
        Buffer += (STRLEN (LinkedList->Data.Address16.ResourceSource) + 1);
    }
    
    /*
     * Return the number of bytes consumed in this operation
     */
    *BytesConsumed = (UINT32) ((NATIVE_UINT) Buffer - (NATIVE_UINT) *OutputBuffer);

    /*
     * Set the length field to the number of bytes consumed
     *  minus the header size (3 bytes)
     */
    *LengthField = (UINT16) (*BytesConsumed - 3);

    return_ACPI_STATUS (AE_OK);
}

