/*
 *  ./acpica/subsystem/resources/rsaddr32.c - RsAddress32Resource function
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __RSADDR32_C__

#include <acpi.h>

#define _COMPONENT          RESOURCE_MANAGER
        MODULE_NAME         ("rsaddr32");



ACPI_STATUS
RsAddress32Resource (
    UINT8                   *ByteStreamBuffer,
    UINT32                  *BytesConsumed,
    UINT8                   **OutputBuffer,
    UINT32                  *StructureSize)
{
    UINT8                   *Buffer = ByteStreamBuffer;
    RESOURCE                *OutputStruct = (RESOURCE *) * OutputBuffer;
    UINT32                  Temp32 = 0;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;        
    UINT32                  StructSize = sizeof (ADDRESS32_RESOURCE) + RESOURCE_LENGTH_NO_DATA;
    UINT32                  Index;
    

    FUNCTION_TRACE ("RsAddress32Resource");

    /*
     * Point past the Descriptor to get the number of bytes consumed
     */ 
    Buffer += 1;

    Temp16 = *(UINT16 *) Buffer;

    *BytesConsumed = Temp16 + 3;

    OutputStruct->Id = Address32;

    /*
     * Get the Resource Type (Byte3)
     */ 
    Buffer += 2;
    Temp8 = *Buffer;

    /* Values 0-2 are valid */
    if(Temp8 > 2)
    {
        return_ACPI_STATUS (AE_ERROR);
    }

    OutputStruct->Data.Address32.ResourceType = Temp8 & 0x03;

    /*
     * Get the General Flags (Byte4)
     */ 
    Buffer += 1;
    Temp8 = *Buffer;

    /*
     * Producer / Consumer
     */
    OutputStruct->Data.Address32.ProducerConsumer = Temp8 & 0x01;
    
    /*
     * Decode
     */
    OutputStruct->Data.Address32.Decode = (Temp8 >> 1) & 0x01;

    /*
     * Min Address Fixed
     */
    OutputStruct->Data.Address32.MinAddressFixed = (Temp8 >> 2) & 0x01;

    /*
     * Max Address Fixed
     */
    OutputStruct->Data.Address32.MaxAddressFixed = (Temp8 >> 3) & 0x01;

    /*
     * Get the Type Specific Flags (Byte5)
     */ 
    Buffer += 1;
    Temp8 = *Buffer;

    if (MEMORY_RANGE == OutputStruct->Data.Address32.ResourceType)
    {
        OutputStruct->Data.Address32.Attribute.Memory.ReadWriteAttribute = (UINT16) (Temp8 & 0x01);
        OutputStruct->Data.Address32.Attribute.Memory.CacheAttribute = (UINT16) ((Temp8 >> 1) & 0x0F);
    }

    else
    {
        if (IO_RANGE == OutputStruct->Data.Address32.ResourceType)
        {
            OutputStruct->Data.Address32.Attribute.Io.RangeAttribute = (UINT16) (Temp8 & 0x03);
        }

        else
        {
            /* BUS_NUMBER_RANGE == OutputStruct->Data.Address32.ResourceType */
            /* Nothing needs to be filled in */
        }
    }

    /*
     * Get Granularity (Bytes 6-9)
     */ 
    Buffer += 1;
    Temp32 = *(UINT32 *) Buffer;

    OutputStruct->Data.Address32.Granularity = Temp32;

    /*
     * Get MinAddressRange (Bytes 10-13)
     */ 
    Buffer += 4;
    Temp32 = *(UINT32 *) Buffer;

    OutputStruct->Data.Address32.MinAddressRange = Temp32;

    /*
     * Get MaxAddressRange (Bytes 14-17)
     */ 
    Buffer += 4;
    Temp32 = *(UINT32 *) Buffer;

    OutputStruct->Data.Address32.MaxAddressRange = Temp32;

    /*
     * Get AddressTranslationOffset (Bytes 18-21)
     */ 
    Buffer += 4;
    Temp32 = *(UINT32 *) Buffer;

    OutputStruct->Data.Address32.AddressTranslationOffset = Temp32;

    /*
     * Get AddressLength (Bytes 22-25)
     */ 
    Buffer += 4;
    Temp32 = *(UINT32 *) Buffer;

    OutputStruct->Data.Address32.AddressLength = Temp32;

    /*
     * Resource Source Index (if present)
     */
    Buffer += 4;

    /*
     * This will leave us pointing to the Resource Source Index
     *  If it is present, then save it off and calculate the
     *  pointer to where the null terminated string goes:
     *  Each Interrupt takes 32-bits + the 5 bytes of the 
     *  stream that are default.
     */
    if (*BytesConsumed > 26)
    {
        /* Dereference the Index */

        Temp8 = *Buffer;
        OutputStruct->Data.Address32.ResourceSourceIndex = (UINT32)Temp8;

        /* Point to the String */

        Buffer += 1;
        
        /* Copy the string into the buffer */

        Index = 0;

        while (0x00 != *Buffer)
        {
            OutputStruct->Data.Address32.ResourceSource[Index] = *Buffer;
            Buffer += 1;
            Index += 1;
        }

        /*
         * Add the terminating null
         */
        OutputStruct->Data.Address32.ResourceSource[Index] = 0x00;

        OutputStruct->Data.Address32.ResourceSourceStringLength = Index + 1;

        /*
         * In order for the StructSize to fall on a 32-bit boundry,
         *  calculate the length of the string and expand the 
         *  StructSize to the next 32-bit boundry.
         */
        Temp8 = (UINT8) (Index + 1);

        if (0 == Temp8 % 4)
        {
            StructSize += Temp8;
        }

        else
        {
            StructSize += ((Temp8 / 4) * 4) + 4;
        }
    }

    else
    {
        OutputStruct->Data.Address32.ResourceSourceIndex = 0x00;
        OutputStruct->Data.Address32.ResourceSourceStringLength = 0;
        OutputStruct->Data.Address32.ResourceSource[0] = 0x00;
    }
    
    /*
     * Set the Length parameter
     */
    OutputStruct->Length = StructSize;

    /*
     * Return the final size of the structure
     */
    *StructureSize = StructSize;

    return_ACPI_STATUS (AE_OK);
}



ACPI_STATUS
RsAddress32Stream (
    RESOURCE                *LinkedList,
    UINT8                   **OutputBuffer,
    UINT32                  *BytesConsumed)
{
    UINT8                   *Buffer = *OutputBuffer;
    UINT16                  *LengthField;
    UINT32                  Temp32 = 0;
    UINT8                   Temp8 = 0;
    UINT8                   *TempPointer = NULL;
    

    FUNCTION_TRACE ("RsAddress32Stream");

    /*
     * The descriptor field is static
     */ 
    *Buffer = 0x87;

    Buffer += 1;

    /*
     * Set a pointer to the Length field - to be filled in later
     */

    LengthField = (UINT16 *)Buffer;

    Buffer += 2;

    /*
     * Set the Resource Type (Memory, Io, BusNumber)
     */
    Temp8 = (UINT8) (LinkedList->Data.Address32.ResourceType & 0x03);

    *Buffer = Temp8;

    Buffer += 1;
    
    /*
     * Set the general flags
     */
    Temp8 = (UINT8) (LinkedList->Data.Address32.ProducerConsumer & 0x01);

    Temp8 |= (LinkedList->Data.Address32.Decode & 0x01) << 1;

    Temp8 |= (LinkedList->Data.Address32.MinAddressFixed & 0x01) << 2;

    Temp8 |= (LinkedList->Data.Address32.MaxAddressFixed & 0x01) << 3;

    *Buffer = Temp8;

    Buffer += 1;
    
    /*
     * Set the type specific flags
     */
    Temp8 = 0;
    
    if(MEMORY_RANGE == LinkedList->Data.Address32.ResourceType)
    {
        Temp8 = (UINT8) (LinkedList->Data.Address32.Attribute.Memory.ReadWriteAttribute & 0x01);
    
        Temp8 |= (LinkedList->Data.Address32.Attribute.Memory.CacheAttribute & 0x0F) << 1;
    }

    else if (IO_RANGE == LinkedList->Data.Address32.ResourceType)
    {
        Temp8 = (UINT8) (LinkedList->Data.Address32.Attribute.Io.RangeAttribute & 0x03);
    }

    *Buffer = Temp8;

    Buffer += 1;
    
    /*
     * Set the address space granularity
     */
    Temp32 = LinkedList->Data.Address32.Granularity;

    *(UINT32 *) Buffer = Temp32;

    Buffer += 4;

    /*
     * Set the address range minimum
     */
    Temp32 = LinkedList->Data.Address32.MinAddressRange;

    *(UINT32 *) Buffer = Temp32;

    Buffer += 4;

    /*
     * Set the address range maximum
     */
    Temp32 = LinkedList->Data.Address32.MaxAddressRange;

    *(UINT32 *) Buffer = Temp32;

    Buffer += 4;

    /*
     * Set the address translation offset
     */
    Temp32 = LinkedList->Data.Address32.AddressTranslationOffset;

    *(UINT32 *) Buffer = Temp32;

    Buffer += 4;

    /*
     * Set the address length
     */
    Temp32 = LinkedList->Data.Address32.AddressLength;

    *(UINT32 *) Buffer = Temp32;

    Buffer += 4;

    /*
     * Resource Source Index and Resource Source are optional
     */
    if (0 != LinkedList->Data.Address32.ResourceSourceStringLength)
    {
        Temp8 = (UINT8) LinkedList->Data.Address32.ResourceSourceIndex;

        *Buffer = Temp8;

        Buffer += 1;

        TempPointer = Buffer;
        
        /*
         * Copy the string
         */
        STRCPY (TempPointer, LinkedList->Data.Address32.ResourceSource);

        /*
         * Buffer needs to be set to the length of the sting + one for the 
         *  terminating null
         */
        Buffer += (STRLEN (LinkedList->Data.Address32.ResourceSource) + 1);
    }
    
    /*
     * Return the number of bytes consumed in this operation
     */
    *BytesConsumed = (UINT32) ((NATIVE_UINT) Buffer - (NATIVE_UINT) *OutputBuffer);

    /*
     * Set the length field to the number of bytes consumed
     *  minus the header size (3 bytes)
     */
    *LengthField = (UINT16) (*BytesConsumed - 3);

    return_ACPI_STATUS (AE_OK);
}

