/*
 *  ./acpica/subsystem/resources/rscrbyst.c - RsCreateByteStream function
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __RSCRRESL_C__

#include <acpi.h>
#include <resource.h>

#define _COMPONENT          RESOURCE_MANAGER
        MODULE_NAME         ("rscrbyst");



ACPI_STATUS
RsCreateByteStream (
    RESOURCE                *LinkedListBuffer,
    UINT8                   *OutputBuffer,
    UINT32                  *OutputBufferLength)
{ 
    ACPI_STATUS             Status = AE_UNKNOWN_STATUS;
    UINT32                  ByteStreamSizeNeeded = 0;
    

    FUNCTION_TRACE ("RsCreateByteStream");

    DEBUG_PRINT (VERBOSE_INFO, ("RsCreateByteStream: LinkedListBuffer = %p\n", LinkedListBuffer));

    /*
     * Validate parameters:
     *  
     *  1. If LinkedListBuffer is NULL, or
     *  2. If OutputBuffer is NULL and OutputBufferLength is not zero
     *  
     *  Return an error
     */
    if (!LinkedListBuffer ||
       (!OutputBuffer && 0 != *OutputBufferLength))
    {
        return_ACPI_STATUS (AE_BAD_PARAMETER);
    }

    /*
     * Pass the LinkedListBuffer into a module that can calculate
     *  the buffer size needed for the byte stream.
     */
    Status = RsCalculateByteStreamLength (LinkedListBuffer, &ByteStreamSizeNeeded);
    
    DEBUG_PRINT (VERBOSE_INFO, ("RsCreateByteStream: Status = %d; ByteStreamSizeNeeded = %d\n", 
                    Status, ByteStreamSizeNeeded));

    /*
     * Exit with the error passed back
     */
    if (AE_OK != Status)
    {
        return_ACPI_STATUS (Status);
    }

    /*
     * If the linked list will fit into the available buffer
     *  call to fill in the list
     */

    if (ByteStreamSizeNeeded <= *OutputBufferLength)
    {
        /*
         * Zero out the return buffer before proceeding
         */
        MEMSET (OutputBuffer, 0x00, *OutputBufferLength);

        Status = RsListToByteStream (LinkedListBuffer, ByteStreamSizeNeeded, &OutputBuffer);

        /*
         * Exit with the error passed back
         */
        if (AE_OK != Status)
        {
            return_ACPI_STATUS (Status);
        }

        DEBUG_PRINT (VERBOSE_INFO, ("RsListToByteStream: OutputBuffer = %p\n", OutputBuffer));
    }
    else
    {
        *OutputBufferLength = ByteStreamSizeNeeded;
        return_ACPI_STATUS (AE_BUFFER_OVERFLOW);
    }

    return_ACPI_STATUS (AE_OK);

}    /*    RsCreateResourceList    */

