/*
 *  ./acpica/subsystem/resources/rsdepend.c - RsStartDependentFunctionsResource,
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __RSDEPEND_C__

#include <acpi.h>

#define _COMPONENT          RESOURCE_MANAGER
        MODULE_NAME         ("rsdepend");



ACPI_STATUS
RsStartDependentFunctionsResource (
    UINT8                   *ByteStreamBuffer,
    UINT32                  *BytesConsumed,
    UINT8                   **OutputBuffer,
    UINT32                  *StructureSize)
{
    UINT8                   *Buffer = ByteStreamBuffer;
    RESOURCE                *OutputStruct = (RESOURCE *) * OutputBuffer;
    UINT8                   Temp8 = 0;        
    UINT32                  StructSize = sizeof (START_DEPENDENT_FUNCTIONS_RESOURCE) + RESOURCE_LENGTH_NO_DATA;
    

    FUNCTION_TRACE ("RsStartDependentFunctionsResource");

    /*
     * The number of bytes consumed are contained in the descriptor (Bits:0-1)
     */ 
    Temp8 = *Buffer;

    *BytesConsumed = (Temp8 & 0x01) + 1;

    OutputStruct->Id = StartDependentFunctions;

    /*
     * Point to Byte 1 if it is used
     */
    if (2 == *BytesConsumed)
    {
        Buffer += 1;
        Temp8 = *Buffer;

        /*
         * Check Compatibility priority
         */
        OutputStruct->Data.StartDependentFunctions.CompatibilityPriority = Temp8 & 0x03;

        if (3 == OutputStruct->Data.StartDependentFunctions.CompatibilityPriority)
        {
            return_ACPI_STATUS (AE_ERROR);
        }

        /*
         * Check Performance/Robustness preference
         */
        OutputStruct->Data.StartDependentFunctions.PerformanceRobustness = (Temp8 >> 2) & 0x03;

        if (3 == OutputStruct->Data.StartDependentFunctions.PerformanceRobustness)
        {
            return_ACPI_STATUS (AE_ERROR);
        }
    }

    else
    {
        OutputStruct->Data.StartDependentFunctions.CompatibilityPriority = ACCEPTABLE_CONFIGURATION;
        OutputStruct->Data.StartDependentFunctions.PerformanceRobustness = ACCEPTABLE_CONFIGURATION;
    }

    /*
     * Set the Length parameter
     */
    OutputStruct->Length = StructSize;

    /*
     * Return the final size of the structure
     */
    *StructureSize = StructSize;

    return_ACPI_STATUS (AE_OK);
}



ACPI_STATUS
RsEndDependentFunctionsResource (
    UINT8                   *ByteStreamBuffer,
    UINT32                  *BytesConsumed,
    UINT8                   **OutputBuffer,
    UINT32                  *StructureSize)
{
    RESOURCE                *OutputStruct = (RESOURCE *) * OutputBuffer;
    UINT32                  StructSize = RESOURCE_LENGTH;
  
    
    FUNCTION_TRACE ("RsEndDependentFunctionsResource");

    /*
     * The number of bytes consumed is static
     */ 
    *BytesConsumed = 1;

    /*
     *  Fill out the structure
     */
    OutputStruct->Id = EndDependentFunctions;

    /*
     * Set the Length parameter
     */
    OutputStruct->Length = StructSize;

    /*
     * Return the final size of the structure
     */
    *StructureSize = StructSize;

    return_ACPI_STATUS (AE_OK);
}


ACPI_STATUS
RsStartDependentFunctionsStream (
    RESOURCE                *LinkedList,
    UINT8                   **OutputBuffer,
    UINT32                  *BytesConsumed)
{
    UINT8                   *Buffer = *OutputBuffer;
    UINT8                   Temp8 = 0;        
    

    FUNCTION_TRACE ("RsStartDependentFunctionsStream");

    /*
     * The descriptor field is set based upon whether a byte is needed
     *  to contain Priority data.
     */ 
    if (ACCEPTABLE_CONFIGURATION == 
            LinkedList->Data.StartDependentFunctions.CompatibilityPriority &&
        ACCEPTABLE_CONFIGURATION ==
            LinkedList->Data.StartDependentFunctions.PerformanceRobustness)
    {
        *Buffer = 0x30;
    }
    else
    {
        *Buffer = 0x31;

        Buffer += 1;

        /*
         * Set the Priority Byte Definition
         */
        Temp8 = 0;

        Temp8 = (UINT8) ((LinkedList->Data.StartDependentFunctions.PerformanceRobustness & 0x03) << 2);

        Temp8 |= (LinkedList->Data.StartDependentFunctions.CompatibilityPriority & 0x03);

        *Buffer = Temp8;
    }

    Buffer += 1;
    
    /*
     * Return the number of bytes consumed in this operation
     */
    *BytesConsumed = (UINT32) ((NATIVE_UINT) Buffer - (NATIVE_UINT) *OutputBuffer);

    return_ACPI_STATUS (AE_OK);
}



ACPI_STATUS
RsEndDependentFunctionsStream (
    RESOURCE                *LinkedList,
    UINT8                   **OutputBuffer,
    UINT32                  *BytesConsumed
    )
{
    UINT8                   *Buffer = *OutputBuffer;
    

    FUNCTION_TRACE ("RsEndDependentFunctionsStream");

    /*
     * The descriptor field is static
     */ 
    *Buffer = 0x38;

    Buffer += 1;

    /*
     * Return the number of bytes consumed in this operation
     */
    *BytesConsumed = (UINT32) ((NATIVE_UINT) Buffer - (NATIVE_UINT) *OutputBuffer);

    return_ACPI_STATUS (AE_OK);
}

