/*
 *  ./acpica/subsystem/resources/rsdma.c - RsDmaResource,
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __RSDMA_C__

#include <acpi.h>

#define _COMPONENT          RESOURCE_MANAGER
        MODULE_NAME         ("rsdma");



ACPI_STATUS
RsDmaResource (
    UINT8                   *ByteStreamBuffer,
    UINT32                  *BytesConsumed,
    UINT8                   **OutputBuffer,
    UINT32                  *StructureSize)
{
    UINT8                   *Buffer = ByteStreamBuffer;
    RESOURCE                *OutputStruct = (RESOURCE *) * OutputBuffer;
    UINT8                   Temp8 = 0;        
    UINT8                   Index;
    UINT32                  StructSize = sizeof(DMA_RESOURCE) + RESOURCE_LENGTH_NO_DATA;
   
    
    FUNCTION_TRACE ("RsDmaResource");

    /*
     * The number of bytes consumed are Constant
     */ 
    *BytesConsumed = 3;

    OutputStruct->Id = Dma;

    /*
     * Point to the 8-bits of Byte 1
     */ 
    Buffer += 1;

    Temp8 = *Buffer;

    OutputStruct->Data.Dma.NumberOfChannels = 0;

    /* Decode the IRQ bits */

    for (Index = 0; Index < 8; Index++)
    {
        if ((Temp8 >> Index) & 0x01)
        {
            OutputStruct->Data.Dma.Channels[OutputStruct->Data.Dma.NumberOfChannels] = Index;
            OutputStruct->Data.Dma.NumberOfChannels += 1;
        }
    }

    /*
     * Calculate the structure size based upon the number of interrupts
     */
    StructSize += (OutputStruct->Data.Dma.NumberOfChannels - 1) * 4;

    /*
     * Point to Byte 2
     */
     Buffer += 1;

     Temp8 = *Buffer;

    /*
     * Check for transfer preference (Bits[1:0])
     */
    OutputStruct->Data.Dma.Transfer = Temp8 & 0x03;

    if (0x03 == OutputStruct->Data.Dma.Transfer)
    {
        return_ACPI_STATUS (AE_BAD_DATA);
    }

    /*
     * Get bus master preference (Bit[2])
     */
    OutputStruct->Data.Dma.BusMaster = (Temp8 >> 2) & 0x01;

    /*
     * Get channel speed support (Bits[6:5])
     */
    OutputStruct->Data.Dma.Type = (Temp8 >> 5) & 0x03;

    /*
     * Set the Length parameter
     */
    OutputStruct->Length = StructSize;

    /*
     * Return the final size of the structure
     */
    *StructureSize = StructSize;

    return_ACPI_STATUS (AE_OK);
}



ACPI_STATUS
RsDmaStream (
    RESOURCE                *LinkedList,
    UINT8                   **OutputBuffer,
    UINT32                  *BytesConsumed)
{
    UINT8                   *Buffer = *OutputBuffer;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;        
    UINT8                   Index;
    

    FUNCTION_TRACE ("RsDmaStream");


    /*
     * The descriptor field is static
     */ 
    *Buffer = 0x2A;

    Buffer += 1;

    Temp8 = 0;

    /*
     * Loop through all of the Channels and set the mask bits
     */
    for (Index = 0; Index < LinkedList->Data.Dma.NumberOfChannels; Index++)
    {
        Temp16 = (UINT16) LinkedList->Data.Dma.Channels[Index];
        Temp8 |= 0x1 << Temp16;
    }

    *Buffer = Temp8;

    Buffer += 1;

    /*
     * Set the DMA Info
     */
    Temp8 = (UINT8) ((LinkedList->Data.Dma.Type & 0x03) << 5);

    Temp8 |= ((LinkedList->Data.Dma.BusMaster & 0x01) << 2);

    Temp8 |= (LinkedList->Data.Dma.Transfer & 0x03);

    *Buffer = Temp8;

    Buffer += 1;
    
    /*
     * Return the number of bytes consumed in this operation
     */
    *BytesConsumed = (UINT32) ((NATIVE_UINT) Buffer - (NATIVE_UINT) *OutputBuffer);

    return_ACPI_STATUS (AE_OK);
}

