
/*
 *  ./acpica/subsystem/resources/rsdump.c - Functions do dump out the resource structures.
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#define __RSDUMP_C__

#include <acpi.h>

#define _COMPONENT          RESOURCE_MANAGER
        MODULE_NAME         ("rsdump");



void
RsDumpIrq (
 RESOURCE_DATA   *Data)
{
    IRQ_RESOURCE    *IrqData = (IRQ_RESOURCE*) Data;
    UINT8     Index = 0;

    OsdPrintf ("\tIRQ Resource\n");
    
    OsdPrintf ("\t\t%s Triggered\n", LEVEL_SENSITIVE == IrqData->EdgeLevel ? "Level" : "Edge");
 OsdPrintf ("\t\tActive %s\n", ACTIVE_LOW == IrqData->ActiveHighLow ? "Low" : "High");
    OsdPrintf ("\t\t%s\n", SHARED == IrqData->SharedExclusive ? "Shared" : "Exclusive");
 OsdPrintf ("\t\t%d Interrupts ( ", IrqData->NumberOfInterrupts);

    for (Index = 0; Index < IrqData->NumberOfInterrupts; Index++)
    {
        OsdPrintf ("%d ", IrqData->Interrupts[Index]);
    }
    
    OsdPrintf (")\n");
    return;
}



void
RsDumpDma (
 RESOURCE_DATA   *Data)
{
    DMA_RESOURCE   *DmaData = (DMA_RESOURCE*) Data;
    UINT8     Index = 0;

    OsdPrintf ("\tDMA Resource\n");

    switch (DmaData->Type)
    {
    case COMPATIBILITY:
        OsdPrintf ("\t\tCompatibility mode\n");
        break;

    case TYPE_A:
        OsdPrintf ("\t\tType A\n");
        break;

    case TYPE_B:
        OsdPrintf ("\t\tType B\n");
        break;

    case TYPE_F:
        OsdPrintf ("\t\tType F\n");
        break; 

    default:
        OsdPrintf ("\t\tInvalid DMA type\n");
        break;
    }
    
    OsdPrintf ("\t\t%sBus Master\n", BUS_MASTER == DmaData->BusMaster ? "" : "Not a ");

    switch (DmaData->Transfer)
    {
    case TRANSFER_8:
        OsdPrintf ("\t\t8-bit only transfer\n");
        break;

    case TRANSFER_8_16:
        OsdPrintf ("\t\t8 and 16-bit transfer\n");
        break;

    case TRANSFER_16:
        OsdPrintf ("\t\t16 bit only transfer\n");
        break;

    default:
        OsdPrintf ("\t\tInvalid transfer preference\n");
        break;
    }

    OsdPrintf ("\t\tNumber of Channels: %d ( ", DmaData->NumberOfChannels);

    for (Index = 0; Index < DmaData->NumberOfChannels; Index++)
    {
        OsdPrintf ("%d ", DmaData->Channels[Index]);
    }
    
    OsdPrintf (")\n");
    return;
}



void
RsDumpStartDependentFunctions (
 RESOURCE_DATA    *Data)
{
    START_DEPENDENT_FUNCTIONS_RESOURCE *SdfData = (START_DEPENDENT_FUNCTIONS_RESOURCE*) Data;

    OsdPrintf ("\tStart Dependent Functions Resource\n");

    switch (SdfData->CompatibilityPriority)
    {
    case GOOD_CONFIGURATION:
        OsdPrintf ("\t\tGood configuration\n");
        break;

    case ACCEPTABLE_CONFIGURATION:
        OsdPrintf ("\t\tAcceptable configuration\n");
        break;

    case SUB_OPTIMAL_CONFIGURATION:
        OsdPrintf ("\t\tSub-optimal configuration\n");
        break;

    default:
        OsdPrintf ("\t\tInvalid compatibility priority\n");
        break; 
    }
    
    switch(SdfData->PerformanceRobustness)
    {
    case GOOD_CONFIGURATION:
        OsdPrintf ("\t\tGood configuration\n");
        break;

    case ACCEPTABLE_CONFIGURATION:
        OsdPrintf ("\t\tAcceptable configuration\n");
        break;

    case SUB_OPTIMAL_CONFIGURATION:
        OsdPrintf ("\t\tSub-optimal configuration\n");
        break;

    default:
        OsdPrintf ("\t\tInvalid performance robustness preference\n");
        break; 
    }

    return;
}



void
RsDumpIo (
 RESOURCE_DATA  *Data)
{
    IO_RESOURCE   *IoData = (IO_RESOURCE*) Data;

    OsdPrintf ("\tIo Resource\n");
    OsdPrintf ("\t\t%d bit decode\n", DECODE_16 == IoData->IoDecode ? 16 : 10);
    OsdPrintf ("\t\tRange minimum base: 0x%08x\n", IoData->MinBaseAddress);
    OsdPrintf ("\t\tRange maximum base: 0x%08x\n", IoData->MaxBaseAddress);
    OsdPrintf ("\t\tAlignment: 0x%08x\n", IoData->Alignment);
    OsdPrintf ("\t\tRange Length: 0x%08x\n", IoData->RangeLength);

    return;
}



void
RsDumpFixedIo (
 RESOURCE_DATA   *Data)
{
    FIXED_IO_RESOURCE  *FixedIoData = (FIXED_IO_RESOURCE*) Data;

    OsdPrintf ("\tFixed Io Resource\n");
    OsdPrintf ("\t\tRange base address: 0x%08x", FixedIoData->BaseAddress);
    OsdPrintf ("\t\tRange length: 0x%08x", FixedIoData->RangeLength);

    return;
}



void
RsDumpVendorSpecific (
 RESOURCE_DATA   *Data)
{
    VENDOR_RESOURCE   *VendorData = (VENDOR_RESOURCE*) Data;
    UINT16     Index = 0;

    OsdPrintf ("\tVendor Specific Resource\n");
    OsdPrintf ("\t\tLength: 0x%08x\n", VendorData->Length);
    
    for (Index = 0; Index < VendorData->Length; Index++)
    {
        OsdPrintf ("\t\tByte %d: 0x%08x\n", Index, VendorData->Reserved[Index]);
    }
 
    return;
}



void
RsDumpMemory24 (
 RESOURCE_DATA   *Data)
{
    MEMORY24_RESOURCE   *Memory24Data = (MEMORY24_RESOURCE*) Data;

    OsdPrintf ("\t24-Bit Memory Range Resource\n");

    OsdPrintf ("\t\tRead%s\n",
     READ_WRITE_MEMORY == Memory24Data->ReadWriteAttribute ? "/Write" : " only");
    OsdPrintf ("\t\tRange minimum base: 0x%08x\n", Memory24Data->MinBaseAddress);
    OsdPrintf ("\t\tRange maximum base: 0x%08x\n", Memory24Data->MaxBaseAddress);
    OsdPrintf ("\t\tAlignment: 0x%08x\n", Memory24Data->Alignment);
    OsdPrintf ("\t\tRange length: 0x%08x\n", Memory24Data->RangeLength);
 
    return;
}



void
RsDumpMemory32 (
 RESOURCE_DATA   *Data)
{
    MEMORY32_RESOURCE  *Memory32Data = (MEMORY32_RESOURCE*) Data;

    OsdPrintf ("\t32-Bit Memory Range Resource\n");

    OsdPrintf ("\t\tRead%s\n",
     READ_WRITE_MEMORY == Memory32Data->ReadWriteAttribute ? "/Write" 
     : " only");
    OsdPrintf ("\t\tRange minimum base: 0x%08x\n", Memory32Data->MinBaseAddress);
    OsdPrintf ("\t\tRange maximum base: 0x%08x\n", Memory32Data->MaxBaseAddress);
    OsdPrintf ("\t\tAlignment: 0x%08x\n", Memory32Data->Alignment);
    OsdPrintf ("\t\tRange length: 0x%08x\n", Memory32Data->RangeLength);
    
    return;
}



void
RsDumpFixedMemory32 (
 RESOURCE_DATA   *Data)
{
    FIXED_MEMORY32_RESOURCE *FixedMemory32Data = (FIXED_MEMORY32_RESOURCE*) Data;

    OsdPrintf ("\t32-Bit Fixed Location Memory Range Resource\n");
    OsdPrintf ("\t\tRead%s\n",
     READ_WRITE_MEMORY == FixedMemory32Data->ReadWriteAttribute ? "/Write" : " Only");
    OsdPrintf ("\t\tRange base address: 0x%08x\n", FixedMemory32Data->RangeBaseAddress);
    OsdPrintf ("\t\tRange length: 0x%08x\n", FixedMemory32Data->RangeLength);

    return;
}



void
RsDumpAddress16 (
 RESOURCE_DATA   *Data)
{
    ADDRESS16_RESOURCE *Address16Data = (ADDRESS16_RESOURCE*) Data;

    OsdPrintf ("\t16-Bit Address Space Resource\n");    
    OsdPrintf ("\t\tResource Type: ");

    switch (Address16Data->ResourceType)
    {
    case MEMORY_RANGE:
        
        OsdPrintf ("Memory Range\n");
        
        switch (Address16Data->Attribute.Memory.CacheAttribute)
        {
        case NON_CACHEABLE_MEMORY:
            OsdPrintf ("\t\tType Specific: Noncacheable memory\n");
            break;

        case CACHABLE_MEMORY:
            OsdPrintf ("\t\tType Specific: Cacheable memory\n");
            break;

        case WRITE_COMBINING_MEMORY:
            OsdPrintf ("\t\tType Specific: Write-combining memory\n");
            break;

        case PREFETCHABLE_MEMORY:
            OsdPrintf ("\t\tType Specific: Prefetchable memory\n");
            break;

        default:
            OsdPrintf ("\t\tType Specific: Invalid cache attribute\n");
            break;
        }
        
        OsdPrintf ("\t\tType Specific: Read%s\n",
         READ_WRITE_MEMORY == Address16Data->Attribute.Memory.ReadWriteAttribute ?
         "/Write" : " Only");
        break;

    case IO_RANGE:

        OsdPrintf ("I/O Range\n");
        
        switch (Address16Data->Attribute.Io.RangeAttribute)
        {
        case NON_ISA_ONLY_RANGES:
            OsdPrintf ("\t\tType Specific: Non-ISA Io Addresses\n");
            break;

        case ISA_ONLY_RANGES:
            OsdPrintf ("\t\tType Specific: ISA Io Addresses\n");
            break;

        case ENTIRE_RANGE:
            OsdPrintf ("\t\tType Specific: ISA and non-ISA Io Addresses\n");
            break;

        default:
            OsdPrintf ("\t\tType Specific: Invalid range attribute\n");
            break;
        } 
        break;

    case BUS_NUMBER_RANGE:

        OsdPrintf ("Bus Number Range\n");
        break;

    default:

        OsdPrintf ("Invalid resource type. Exiting.\n");
        return;
    }

    OsdPrintf ("\t\tResource %s\n",
     CONSUMER == Address16Data->ProducerConsumer ? "Consumer" : "Producer");
    OsdPrintf ("\t\t%s decode\n",
  SUB_DECODE == Address16Data->Decode ? "Subtractive" : "Positive");
    OsdPrintf ("\t\tMin address is %s fixed\n",
     ADDRESS_FIXED == Address16Data->MinAddressFixed ? "" : "not");
    OsdPrintf ("\t\tMax address is %s fixed\n",
     ADDRESS_FIXED == Address16Data->MaxAddressFixed ? "" : "not");
    OsdPrintf ("\t\tGranularity: 0x%08x\n", Address16Data->Granularity);
    OsdPrintf ("\t\tAddress range min: 0x%08x\n", Address16Data->MinAddressRange);
    OsdPrintf ("\t\tAddress range max: 0x%08x\n", Address16Data->MaxAddressRange);
    OsdPrintf ("\t\tAddress translation offset: 0x%08x\n", Address16Data->AddressTranslationOffset);
    OsdPrintf ("\t\tAddress Length: 0x%08x\n", Address16Data->AddressLength);

    if (0xFF != Address16Data->ResourceSourceIndex)
    {
        OsdPrintf ("\t\tResource Source Index: %d\n", Address16Data->ResourceSourceIndex);
        OsdPrintf ("\t\tResource Source: %s\n", Address16Data->ResourceSource);
    }

    return;
}



void
RsDumpAddress32 (
 RESOURCE_DATA   *Data)
{
    ADDRESS32_RESOURCE *Address32Data = (ADDRESS32_RESOURCE*) Data;

    OsdPrintf ("\t32-Bit Address Space Resource\n");
    
    switch (Address32Data->ResourceType)
    {
    case MEMORY_RANGE:

        OsdPrintf ("\t\tResource Type: Memory Range\n");

        switch (Address32Data->Attribute.Memory.CacheAttribute)
        {
        case NON_CACHEABLE_MEMORY:
            OsdPrintf ("\t\tType Specific: Noncacheable memory\n");
            break;

        case CACHABLE_MEMORY:
            OsdPrintf ("\t\tType Specific: Cacheable memory\n");
            break;

        case WRITE_COMBINING_MEMORY:
            OsdPrintf ("\t\tType Specific: Write-combining memory\n");
            break;

        case PREFETCHABLE_MEMORY:
            OsdPrintf ("\t\tType Specific: Prefetchable memory\n");
            break;

        default:
            OsdPrintf ("\t\tType Specific: Invalid cache attribute\n");
            break;
        }
        
        OsdPrintf ("\t\tType Specific: Read%s\n",
         READ_WRITE_MEMORY == Address32Data->Attribute.Memory.ReadWriteAttribute ?
         "/Write" : " Only");
        break;

    case IO_RANGE:

        OsdPrintf ("\t\tResource Type: Io Range\n");
        
        switch (Address32Data->Attribute.Io.RangeAttribute)
            {
            case NON_ISA_ONLY_RANGES:
                OsdPrintf ("\t\tType Specific: Non-ISA Io Addresses\n");
                break;

            case ISA_ONLY_RANGES:
                OsdPrintf ("\t\tType Specific: ISA Io Addresses\n");
                break;

            case ENTIRE_RANGE:
                OsdPrintf ("\t\tType Specific: ISA and non-ISA Io Addresses\n");
                break;

            default:
                OsdPrintf ("\t\tType Specific: Invalid Range attribute");
                break;
            }
        break;

    case BUS_NUMBER_RANGE:

        OsdPrintf ("\t\tResource Type: Bus Number Range\n");
        break;

    default:

        OsdPrintf ("\t\tInvalid Resource Type..exiting.\n");
        return;
    }

    OsdPrintf ("\t\tResource %s\n",
     CONSUMER == Address32Data->ProducerConsumer ? "Consumer" : "Producer");
    OsdPrintf ("\t\t%s decode\n",
     SUB_DECODE == Address32Data->Decode ? "Subtractive" : "Positive");
    OsdPrintf ("\t\tMin address is %s fixed\n",
     ADDRESS_FIXED == Address32Data->MinAddressFixed ? "" : "not ");
    OsdPrintf ("\t\tMax address is %s fixed\n",
     ADDRESS_FIXED == Address32Data->MaxAddressFixed ? "" : "not ");
    OsdPrintf ("\t\tGranularity: 0x%08x\n", Address32Data->Granularity);
    OsdPrintf ("\t\tAddress range min: 0x%08x\n", Address32Data->MinAddressRange);
    OsdPrintf ("\t\tAddress range max: 0x%08x\n", Address32Data->MaxAddressRange);
    OsdPrintf ("\t\tAddress translation offset: 0x%08x\n", Address32Data->AddressTranslationOffset);
    OsdPrintf ("\t\tAddress Length: 0x%08x\n", Address32Data->AddressLength);

    if(0xFF != Address32Data->ResourceSourceIndex)
    {
        OsdPrintf ("\t\tResource Source Index: %d\n", Address32Data->ResourceSourceIndex);
        OsdPrintf ("\t\tResource Source: %s\n", Address32Data->ResourceSource);
    }

    return;
}



void
RsDumpExtendedIrq (
 RESOURCE_DATA   *Data)
{
    EXTENDED_IRQ_RESOURCE  *ExtIrqData = (EXTENDED_IRQ_RESOURCE*) Data;
    UINT8     Index = 0;

    OsdPrintf ("\tExtended IRQ Resource\n");
    
    OsdPrintf ("\t\tResource %s\n",
     CONSUMER == ExtIrqData->ProducerConsumer ? "Consumer" : "Producer");
    OsdPrintf ("\t\t%s\n",
     LEVEL_SENSITIVE == ExtIrqData->EdgeLevel ? "Level" : "Edge");
    OsdPrintf ("\t\tActive %s\n",
     ACTIVE_LOW == ExtIrqData->ActiveHighLow ? "low" : "high");
    OsdPrintf ("\t\t%s\n",
     SHARED == ExtIrqData->SharedExclusive ? "Shared" : "Exclusive");
    
    OsdPrintf ("\t\tInterrupts : %d ( ", ExtIrqData->NumberOfInterrupts);
    
    for (Index = 0; Index < ExtIrqData->NumberOfInterrupts; Index++)
    {
        OsdPrintf ("%d ", ExtIrqData->Interrupts[Index]);
    }
 
 OsdPrintf (")\n");

    if(0xFF != ExtIrqData->ResourceSourceIndex)
    {
        OsdPrintf ("\t\tResource Source Index: %d", ExtIrqData->ResourceSourceIndex);
        OsdPrintf ("\t\tResource Source: %s", ExtIrqData->ResourceSource);
    }
    
    return;
}



void
RsDumpResourceList (
 RESOURCE   *Resource)
{
 if (DebugLevel & TRACE_RESOURCES && _COMPONENT & DebugLayer)
 {
  INT8   Count = 0;
        BOOLEAN         Done = FALSE;
  
     while (!Done)
     {
      OsdPrintf ("\tResource structure %x.\n", Count++);
      
         switch (Resource->Id)
         {
         case Irq:
             RsDumpIrq (&Resource->Data);
             break;
 
         case Dma:
             RsDumpDma (&Resource->Data);
             break;

         case StartDependentFunctions:
             RsDumpStartDependentFunctions (&Resource->Data);
             break;

         case EndDependentFunctions:
                OsdPrintf ("\tEndDependentFunctions Resource\n");
             /* RsDumpEndDependentFunctions (Resource->Data);*/
             break;

         case Io:
             RsDumpIo (&Resource->Data);
             break;

         case FixedIo:
             RsDumpFixedIo (&Resource->Data);
             break;

         case VendorSpecific:
             RsDumpVendorSpecific (&Resource->Data);
             break;

         case EndTag:
             /*RsDumpEndTag (Resource->Data);*/
                OsdPrintf ("\tEndTag Resource\n");
                Done = TRUE;
             break;

         case Memory24:
             RsDumpMemory24 (&Resource->Data);
             break; 

         case Memory32:
             RsDumpMemory32 (&Resource->Data);
             break; 

         case FixedMemory32:
             RsDumpFixedMemory32 (&Resource->Data);
             break;

         case Address16:
             RsDumpAddress16 (&Resource->Data);
             break;

         case Address32:
             RsDumpAddress32 (&Resource->Data);
             break;

         case ExtendedIrq:
             RsDumpExtendedIrq (&Resource->Data);
             break;

         default:
             OsdPrintf ("Invalid resource type\n");
             break;
        
         } 
         
         Resource = (RESOURCE *) ((NATIVE_UINT) Resource + (NATIVE_UINT) Resource->Length);
     }
 }
 
    return;
}


void
RsDumpIRQList (
 UINT8 *RouteTable)
{
 if (DebugLevel & TRACE_RESOURCES && _COMPONENT & DebugLayer)
 {
  UINT8               *Buffer = RouteTable;
        INT8       Count = 0;
        BOOLEAN             Done = FALSE;
        PCI_ROUTING_TABLE   *PrtElement;

        PrtElement = (PCI_ROUTING_TABLE *)Buffer;

        while (!Done)
     {
            OsdPrintf ("\tPCI IRQ Routing Table structure %x.\n", Count++);
      
            OsdPrintf ("\t\tAddress: 0x%x\n", PrtElement->Data.Address);
            OsdPrintf ("\t\tPin: 0x%x\n", PrtElement->Data.Pin);
            OsdPrintf ("\t\tSource: %s\n", PrtElement->Data.Source);
            OsdPrintf ("\t\tSourceIndex: 0x%x\n", PrtElement->Data.SourceIndex);

            Buffer += PrtElement->Length;

            PrtElement = (PCI_ROUTING_TABLE *)Buffer;

            if(0 == PrtElement->Length)
            {
                Done = TRUE;
            }
     }
 }
 
    return;
}

