/*
 *  ./acpica/subsystem/resources/rsgetbss.c - RsGetByteStreamStart function
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __RSGETBSS_C__

#include <acpi.h>

#define _COMPONENT          RESOURCE_MANAGER
        MODULE_NAME         ("rsgetbss");



ACPI_STATUS
RsGetByteStreamStart (
    UINT8                   *ByteStreamBuffer,
    UINT8                   **ByteStreamStart,
    UINT32                  *SizeNeeded)
{
    ACPI_STATUS             Status = AE_OK;
    UINT8                   *StartPointer = ByteStreamBuffer;
    UINT8                   Temp8 = 0;
    UINT8                   Index;


    FUNCTION_TRACE ("RsGetByteStreamStart");

    /*
     * Validate parameters
     *  If either pointer is NULL or stream opcode
     *  the first UINT8 is not a buffer opcode, return an error.
     */
    if (!ByteStreamBuffer ||
        !ByteStreamStart  ||
        0x11 != *StartPointer)
    {
        *SizeNeeded = 0;
        return_ACPI_STATUS (AE_BAD_DATA);
    }

    /*
     * Point to the next element (PackageLength)
     */
    ++StartPointer;

    Temp8 = (UINT8) *StartPointer;

    /*
     * Length is held in the high two bits
     */
    Temp8 >>= 6;
    
    /*
     * Push our start pointer past the package length
     *  because it is not too interesting anyway.
     */
    for (Index = 0; Index <= Temp8; Index++)
    {
        ++StartPointer;
    }

    /*
     * This should leave us pointing at a Prefix opcode - 
     *  either Byte (0x0a), Word (0x0b) or Dword (0x0c)
     */
    Temp8 = (UINT8) *StartPointer;

    ++StartPointer;

    switch(Temp8)
    {
    case 0x0a:
        *SizeNeeded = (UINT8) *StartPointer;
        *ByteStreamStart = StartPointer + 1;
        break;

    case 0x0b:
        *SizeNeeded = (UINT16) *StartPointer;
        *ByteStreamStart = StartPointer + 2;
        break;

    case 0x0c:
        *SizeNeeded = (UINT32) *StartPointer;
        *ByteStreamStart = StartPointer + 4;
        break;

    default: /* An error has occurred */
        *SizeNeeded = 0;
        *ByteStreamStart = NULL;
        Status = AE_BAD_DATA;
        break;
    }

    return_ACPI_STATUS (Status);
}

