/*
 *  ./acpica/subsystem/resources/rsio.c - RsIoResource, 
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __RSIO_C__

#include <acpi.h>

#define _COMPONENT          RESOURCE_MANAGER
        MODULE_NAME         ("rsio");



ACPI_STATUS
RsIoResource (
    UINT8                   *ByteStreamBuffer,
    UINT32                  *BytesConsumed,
    UINT8                   **OutputBuffer,
    UINT32                  *StructureSize)
{
    UINT8                   *Buffer = ByteStreamBuffer;
    RESOURCE                *OutputStruct = (RESOURCE *) * OutputBuffer;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;        
    UINT32                  StructSize = sizeof (IO_RESOURCE) + RESOURCE_LENGTH_NO_DATA;
  
    
    FUNCTION_TRACE ("RsIoResource");

    /*
     * The number of bytes consumed are Constant
     */ 
    *BytesConsumed = 8;

    OutputStruct->Id = Io;

    /*
     * Check Decode
     */
    Buffer += 1;
    Temp8 = *Buffer;

    OutputStruct->Data.Io.IoDecode = Temp8 & 0x01;

    /*
     * Check MinBase Address
     */ 
    Buffer += 1;
    Temp16 = *(UINT16 *)Buffer;

    OutputStruct->Data.Io.MinBaseAddress = Temp16;

    /*
     * Check MaxBase Address
     */ 
    Buffer += 2;
    Temp16 = *(UINT16 *)Buffer;

    OutputStruct->Data.Io.MaxBaseAddress = Temp16;

    /*
     * Check Base alignment
     */
    Buffer += 2;
    Temp8 = *Buffer;

    OutputStruct->Data.Io.Alignment = Temp8;

    /*
     * Check RangeLength
     */
    Buffer += 1;
    Temp8 = *Buffer;

    OutputStruct->Data.Io.RangeLength = Temp8;

    /*
     * Set the Length parameter
     */
    OutputStruct->Length = StructSize;

    /*
     * Return the final size of the structure
     */
    *StructureSize = StructSize;

    return_ACPI_STATUS (AE_OK);
}



ACPI_STATUS
RsFixedIoResource (
    UINT8                   *ByteStreamBuffer,
    UINT32                  *BytesConsumed,
    UINT8                   **OutputBuffer,
    UINT32                  *StructureSize)
{
    UINT8                   *Buffer = ByteStreamBuffer;
    RESOURCE                *OutputStruct = (RESOURCE *) * OutputBuffer;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;        
    UINT32                  StructSize = sizeof (FIXED_IO_RESOURCE) + RESOURCE_LENGTH_NO_DATA;
    

    FUNCTION_TRACE ("RsFixedIoResource");

    /*
     * The number of bytes consumed are Constant
     */ 
    *BytesConsumed = 4;

    OutputStruct->Id = FixedIo;

    /*
     * Check Range Base Address
     */ 
    Buffer += 1;
    Temp16 = *(UINT16 *) Buffer;

    OutputStruct->Data.FixedIo.BaseAddress = Temp16;

    /*
     * Check RangeLength
     */
    Buffer += 2;
    Temp8 = *Buffer;

    OutputStruct->Data.FixedIo.RangeLength = Temp8;

    /*
     * Set the Length parameter
     */
    OutputStruct->Length = StructSize;

    /*
     * Return the final size of the structure
     */
    *StructureSize = StructSize;

    return_ACPI_STATUS (AE_OK);
}



ACPI_STATUS
RsIoStream (
    RESOURCE                *LinkedList,
    UINT8                   **OutputBuffer,
    UINT32                  *BytesConsumed)
{
    UINT8                   *Buffer = *OutputBuffer;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;        
    

    FUNCTION_TRACE ("RsIoStream");

    /*
     * The descriptor field is static
     */ 
    *Buffer = 0x47;

    Buffer += 1;

    /*
     * Io Information Byte
     */
    Temp8 = (UINT8) (LinkedList->Data.Io.IoDecode & 0x01);

    *Buffer = Temp8;

    Buffer += 1;

    /*
     * Set the Range minimum base address
     */
    Temp16 = (UINT16) LinkedList->Data.Io.MinBaseAddress;

    *(UINT16 *)Buffer = Temp16;

    Buffer += 2;

    /*
     * Set the Range maximum base address
     */
    Temp16 = (UINT16) LinkedList->Data.Io.MaxBaseAddress;

    *(UINT16 *)Buffer = Temp16;

    Buffer += 2;

    /*
     * Set the base alignment
     */
    Temp8 = (UINT8) LinkedList->Data.Io.Alignment;

    *Buffer = Temp8;

    Buffer += 1;

    /*
     * Set the range length
     */
    Temp8 = (UINT8) LinkedList->Data.Io.RangeLength;

    *Buffer = Temp8;

    Buffer += 1;
    
    /*
     * Return the number of bytes consumed in this operation
     */
    *BytesConsumed = (UINT32) ((NATIVE_UINT) Buffer - (NATIVE_UINT) *OutputBuffer);

    return_ACPI_STATUS (AE_OK);
}



ACPI_STATUS
RsFixedIoStream (
    RESOURCE                *LinkedList,
    UINT8                   **OutputBuffer,
    UINT32                  *BytesConsumed)
{
    UINT8                   *Buffer = *OutputBuffer;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;        
    

    FUNCTION_TRACE ("RsFixedIoStream");

    /*
     * The descriptor field is static
     */ 
    *Buffer = 0x4B;

    Buffer += 1;

    /*
     * Set the Range base address
     */
    Temp16 = (UINT16) LinkedList->Data.FixedIo.BaseAddress;

    *(UINT16 *)Buffer = Temp16;

    Buffer += 2;

    /*
     * Set the range length
     */
    Temp8 = (UINT8) LinkedList->Data.FixedIo.RangeLength;

    *Buffer = Temp8;

    Buffer += 1;
    
    /*
     * Return the number of bytes consumed in this operation
     */
    *BytesConsumed = (UINT32) ((NATIVE_UINT) Buffer - (NATIVE_UINT) *OutputBuffer);

    return_ACPI_STATUS (AE_OK);
}

