/*
 *  ./acpica/subsystem/resources/rslst2by.c - RsListToByteStream function
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __RSLST2BY_C__

#include <acpi.h>
#include <resource.h>

#define _COMPONENT          RESOURCE_MANAGER
        MODULE_NAME         ("rslst2by");



ACPI_STATUS
RsListToByteStream (
    RESOURCE                *LinkedList,
    UINT32                  ByteStreamSizeNeeded,
    UINT8                   **OutputBuffer)
{
    ACPI_STATUS             Status = AE_UNKNOWN_STATUS;
    UINT8                   *Buffer = *OutputBuffer;
    UINT32                  BytesConsumed = 0;
    BOOLEAN                 Done = FALSE;
    

    FUNCTION_TRACE ("RsListToByteStream");

    while (!Done)
    {
        switch (LinkedList->Id)
        {
        case Irq:
            /*
             * IRQ Resource
             */
            Status = RsIrqStream (LinkedList, &Buffer, &BytesConsumed);
            break;

        case Dma:
            /*
             * DMA Resource
             */
            Status = RsDmaStream (LinkedList, &Buffer, &BytesConsumed);
            break;

        case StartDependentFunctions:
            /*
             * Start Dependent Functions Resource
             */
            Status = RsStartDependentFunctionsStream (LinkedList, &Buffer, &BytesConsumed);
            break;

        case EndDependentFunctions:
            /*
             * End Dependent Functions Resource
             */
            Status = RsEndDependentFunctionsStream (LinkedList, &Buffer, &BytesConsumed);
            break;
        
        case Io:
            /*
             * IO Port Resource
             */
            Status = RsIoStream (LinkedList, &Buffer, &BytesConsumed);
            break;
        
        case FixedIo:
            /*
             * Fixed IO Port Resource
             */
            Status = RsFixedIoStream (LinkedList, &Buffer, &BytesConsumed);
            break;
        
        case VendorSpecific:
            /*
             * Vendor Defined Resource
             */
            Status = RsVendorStream (LinkedList, &Buffer, &BytesConsumed);
            break;
        
        case EndTag:
            /*
             * End Tag 
             */
            Status = RsEndTagStream (LinkedList, &Buffer, &BytesConsumed);
            
            /*
             * An End Tag indicates the end of the Resource Template
             */
            Done = TRUE;
            break;
        
        case Memory24:
            /*
             * 24-Bit Memory Resource
             */
            Status = RsMemory24Stream (LinkedList, &Buffer, &BytesConsumed);
            break;
        
        case Memory32:
            /*
             * 32-Bit Memory Range Resource
             */
            Status = RsMemory32RangeStream (LinkedList, &Buffer, &BytesConsumed);
            break;
        
        case FixedMemory32:
            /*
             * 32-Bit Fixed Memory Resource
             */
            Status = RsFixedMemory32Stream (LinkedList, &Buffer, &BytesConsumed);
            break;
        
        case Address16:
            /*
             * 16-Bit Address Descriptor Resource
             */
            Status = RsAddress16Stream (LinkedList, &Buffer, &BytesConsumed);
            break;
        
        case Address32:
            /*
             * 32-Bit Address Descriptor Resource
             */
            Status = RsAddress32Stream (LinkedList, &Buffer, &BytesConsumed);
            break;

        case ExtendedIrq:
            /*
             * Extended IRQ Resource
             */
            Status = RsExtendedIrqStream (LinkedList, &Buffer, &BytesConsumed);
            break;
        
        default:
            /*
             * If we get here, everything is out of sync,
             *  so exit with an error
             */
            return_ACPI_STATUS (AE_BAD_DATA);
            break;

        } /* switch (LinkedList->Id) */
        
        /*
         * Set the Buffer to point to the open byte
         */
        Buffer += BytesConsumed;

        /*
         * Point to the next object
         */
        LinkedList = (RESOURCE *) ((NATIVE_UINT) LinkedList + (NATIVE_UINT) LinkedList->Length);
    }
    
    return_ACPI_STATUS  (AE_OK);

} /* RsListToByteStream */

