/*
 *  ./acpica/subsystem/resources/rsmem24.c - RsMemory24Resource
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __RSMEM24_C__

#include <acpi.h>

#define _COMPONENT          RESOURCE_MANAGER
        MODULE_NAME         ("rsmem24");



ACPI_STATUS
RsMemory24Resource (
    UINT8                   *ByteStreamBuffer,
    UINT32                  *BytesConsumed,
    UINT8                   **OutputBuffer,
    UINT32                  *StructureSize)
{
    UINT8                   *Buffer = ByteStreamBuffer;
    RESOURCE                *OutputStruct = (RESOURCE *) * OutputBuffer;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;        
    UINT32                  StructSize = sizeof (MEMORY24_RESOURCE) + RESOURCE_LENGTH_NO_DATA;
    

    FUNCTION_TRACE ("RsMemory24Resource");

    /*
     * Point past the Descriptor to get the number of bytes consumed
     */ 
    Buffer += 1;

    Temp16 = *(UINT16 *)Buffer;

    Buffer += 2;

    *BytesConsumed = Temp16 + 3;

    OutputStruct->Id = Memory24;

    /*
     * Check Byte 3 the Read/Write bit
     */ 
    Temp8 = *Buffer;

    Buffer += 1;

    OutputStruct->Data.Memory24.ReadWriteAttribute = Temp8 & 0x01;
    
    /*
     * Get MinBaseAddress (Bytes 4-5)
     */
    Temp16 = *(UINT16 *)Buffer;

    Buffer += 2;

    OutputStruct->Data.Memory24.MinBaseAddress = Temp16;

    /*
     * Get MaxBaseAddress (Bytes 6-7)
     */
    Temp16 = *(UINT16 *)Buffer;

    Buffer += 2;

    OutputStruct->Data.Memory24.MaxBaseAddress = Temp16;

    /*
     * Get Alignment (Bytes 8-9)
     */
    Temp16 = *(UINT16 *)Buffer;

    Buffer += 2;
    
    OutputStruct->Data.Memory24.Alignment = Temp16;

    /*
     * Get RangeLength (Bytes 10-11)
     */
    Temp16 = *(UINT16 *)Buffer;

    OutputStruct->Data.Memory24.RangeLength = Temp16;
    
    /*
     * Set the Length parameter
     */
    OutputStruct->Length = StructSize;

    /*
     * Return the final size of the structure
     */
    *StructureSize = StructSize;

    return_ACPI_STATUS (AE_OK);
}



ACPI_STATUS
RsMemory24Stream (
    RESOURCE                *LinkedList,
    UINT8                   **OutputBuffer,
    UINT32                  *BytesConsumed)
{
    UINT8                   *Buffer = *OutputBuffer;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;        
    

    FUNCTION_TRACE ("RsMemory24Stream");

    /*
     * The descriptor field is static
     */ 
    *Buffer = 0x81;

    Buffer += 1;

    /*
     * The length field is static
     */
    Temp16 = 0x09;

    *(UINT16 *) Buffer = Temp16;

    Buffer += 2;

    /*
     * Set the Information Byte
     */
    Temp8 = (UINT8) (LinkedList->Data.Memory24.ReadWriteAttribute & 0x01);

    *Buffer = Temp8;

    Buffer += 1;
    
    /*
     * Set the Range minimum base address
     */
    Temp16 = (UINT16) LinkedList->Data.Memory24.MinBaseAddress;

    *(UINT16 *) Buffer = Temp16;

    Buffer += 2;

    /*
     * Set the Range maximum base address
     */
    Temp16 = (UINT16) LinkedList->Data.Memory24.MaxBaseAddress;

    *(UINT16 *) Buffer = Temp16;

    Buffer += 2;

    /*
     * Set the base alignment
     */
    Temp16 = (UINT16) LinkedList->Data.Memory24.Alignment;

    *(UINT16 *) Buffer = Temp16;

    Buffer += 2;

    /*
     * Set the range length
     */
    Temp16 = (UINT16) LinkedList->Data.Memory24.RangeLength;

    *(UINT16 *) Buffer = Temp16;

    Buffer += 2;
    
    /*
     * Return the number of bytes consumed in this operation
     */
    *BytesConsumed = (UINT32) ((NATIVE_UINT) Buffer - (NATIVE_UINT) *OutputBuffer);

    return_ACPI_STATUS (AE_OK);
}

