/*
 *  ./acpica/subsystem/resources/rsmem32.c - RsMemory32RangeResource,
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __RSMEM32_C__

#include <acpi.h>

#define _COMPONENT          RESOURCE_MANAGER
        MODULE_NAME         ("rsmem32");



ACPI_STATUS
RsMemory32RangeResource (
    UINT8                   *ByteStreamBuffer,
    UINT32                  *BytesConsumed,
    UINT8                   **OutputBuffer,
    UINT32                  *StructureSize)
{
    UINT8                   *Buffer = ByteStreamBuffer;
    RESOURCE                *OutputStruct = (RESOURCE *) * OutputBuffer;
    UINT32                  Temp32 = 0;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;        
    UINT32                  StructSize = sizeof (MEMORY32_RESOURCE) + RESOURCE_LENGTH_NO_DATA;
    

    FUNCTION_TRACE ("RsMemory32RangeResource");

    /*
     * Point past the Descriptor to get the number of bytes consumed
     */ 
    Buffer += 1;

    Temp16 = *Buffer;

    Buffer += 2;

    *BytesConsumed = Temp16 + 3;

    OutputStruct->Id = Memory32;

    /*
     *  Point to the place in the output buffer where the data portion will begin.
     *  1. Set the RESOURCE_DATA * Data to point to it's own address, then
     *  2. Set the pointer to the next address.
     *  NOTE: OutputStruct->Data is cast to UINT8, otherwise, this addition adds
     *      4 * sizeof(RESOURCE_DATA) instead of 4 * sizeof(UINT8) 
     */

    /*
     * Check Byte 3 the Read/Write bit
     */ 
    Temp8 = *Buffer;

    Buffer += 1;

    OutputStruct->Data.Memory32.ReadWriteAttribute = Temp8 & 0x01;
  
    /*
     * Get MinBaseAddress (Bytes 4-7)
     */
    Temp32 = *(UINT32 *)Buffer;

    Buffer += 4;

    OutputStruct->Data.Memory32.MinBaseAddress = Temp32;

    /*
     * Get MaxBaseAddress (Bytes 8-11)
     */
    Temp32 = *(UINT32 *)Buffer;

    Buffer += 4;

    OutputStruct->Data.Memory32.MaxBaseAddress = Temp32;

    /*
     * Get Alignment (Bytes 12-15)
     */
    Temp32 = *(UINT32 *)Buffer;

    Buffer += 4;

    OutputStruct->Data.Memory32.Alignment = Temp32;

    /*
     * Get RangeLength (Bytes 16-19)
     */
    Temp32 = *(UINT32 *)Buffer;

    OutputStruct->Data.Memory32.RangeLength = Temp32;
    
    /*
     * Set the Length parameter
     */
    OutputStruct->Length = StructSize;

    /*
     * Return the final size of the structure
     */
    *StructureSize = StructSize;

    return_ACPI_STATUS (AE_OK);
}



ACPI_STATUS
RsFixedMemory32Resource (
    UINT8                   *ByteStreamBuffer,
    UINT32                  *BytesConsumed,
    UINT8                   **OutputBuffer,
    UINT32                  *StructureSize)
{
    UINT8                   *Buffer = ByteStreamBuffer;
    RESOURCE                *OutputStruct = (RESOURCE *) * OutputBuffer;
    UINT32                  Temp32 = 0;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;        
    UINT32                  StructSize = sizeof (FIXED_MEMORY32_RESOURCE) + RESOURCE_LENGTH_NO_DATA;
    

    FUNCTION_TRACE ("RsFixedMemory32Resource");

    /*
     * Point past the Descriptor to get the number of bytes consumed
     */ 
    Buffer += 1;

    Temp16 = *Buffer;

    Buffer += 2;

    *BytesConsumed = Temp16 + 3;

    OutputStruct->Id = FixedMemory32;

    /*
     * Check Byte 3 the Read/Write bit
     */ 
    Temp8 = *Buffer;

    Buffer += 1;

    OutputStruct->Data.FixedMemory32.ReadWriteAttribute = Temp8 & 0x01;

    /*
     * Get RangeBaseAddress (Bytes 4-7)
     */
    Temp32 = *(UINT32 *)Buffer;

    Buffer += 4;

    OutputStruct->Data.FixedMemory32.RangeBaseAddress = Temp32;

    /*
     * Get RangeLength (Bytes 8-11)
     */
    Temp32 = *(UINT32 *)Buffer;

    OutputStruct->Data.FixedMemory32.RangeLength = Temp32;
  
    /*
     * Set the Length parameter
     */
    OutputStruct->Length = StructSize;

    /*
     * Return the final size of the structure
     */
    *StructureSize = StructSize;

    return_ACPI_STATUS (AE_OK);
}



ACPI_STATUS
RsMemory32RangeStream (
    RESOURCE                *LinkedList,
    UINT8                   **OutputBuffer,
    UINT32                  *BytesConsumed)
{
    UINT8                   *Buffer = *OutputBuffer;
    UINT32                  Temp32 = 0;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;        
   
    
    FUNCTION_TRACE ("RsMemory32RangeStream");

    /*
     * The descriptor field is static
     */ 
    *Buffer = 0x85;

    Buffer += 1;

    /*
     * The length field is static
     */
    Temp16 = 0x11;

    *(UINT16 *) Buffer = Temp16;

    Buffer += 2;

    /*
     * Set the Information Byte
     */
    Temp8 = (UINT8) (LinkedList->Data.Memory32.ReadWriteAttribute & 0x01);

    *Buffer = Temp8;

    Buffer += 1;
    
    /*
     * Set the Range minimum base address
     */
    Temp32 = LinkedList->Data.Memory32.MinBaseAddress;

    *(UINT32 *) Buffer = Temp32;

    Buffer += 4;

    /*
     * Set the Range maximum base address
     */
    Temp32 = LinkedList->Data.Memory32.MaxBaseAddress;

    *(UINT32 *) Buffer = Temp32;

    Buffer += 4;

    /*
     * Set the base alignment
     */
    Temp32 = LinkedList->Data.Memory32.Alignment;

    *(UINT32 *) Buffer = Temp32;

    Buffer += 4;

    /*
     * Set the range length
     */
    Temp32 = LinkedList->Data.Memory32.RangeLength;

    *(UINT32 *) Buffer = Temp32;

    Buffer += 4;
    
    /*
     * Return the number of bytes consumed in this operation
     */
    *BytesConsumed = (UINT32) ((NATIVE_UINT) Buffer - (NATIVE_UINT) *OutputBuffer);

    return_ACPI_STATUS (AE_OK);
}



ACPI_STATUS
RsFixedMemory32Stream (
    RESOURCE                *LinkedList,
    UINT8                   **OutputBuffer,
    UINT32                  *BytesConsumed)
{
    UINT8                   *Buffer = *OutputBuffer;
    UINT32                  Temp32 = 0;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;        
    

    FUNCTION_TRACE ("RsFixedMemory32Stream");

    /*
     * The descriptor field is static
     */ 
    *Buffer = 0x86;

    Buffer += 1;

    /*
     * The length field is static
     */
    Temp16 = 0x09;

    *(UINT16 *) Buffer = Temp16;

    Buffer += 2;

    /*
     * Set the Information Byte
     */
    Temp8 = (UINT8) (LinkedList->Data.FixedMemory32.ReadWriteAttribute & 0x01);

    *Buffer = Temp8;

    Buffer += 1;
    
    /*
     * Set the Range base address
     */
    Temp32 = LinkedList->Data.FixedMemory32.RangeBaseAddress;

    *(UINT32 *) Buffer = Temp32;

    Buffer += 4;

    /*
     * Set the range length
     */
    Temp32 = LinkedList->Data.FixedMemory32.RangeLength;

    *(UINT32 *) Buffer = Temp32;

    Buffer += 4;
    
    /*
     * Return the number of bytes consumed in this operation
     */
    *BytesConsumed = (UINT32) ((NATIVE_UINT) Buffer - (NATIVE_UINT) *OutputBuffer);

    return_ACPI_STATUS (AE_OK);
}

