/*
 *  ./acpica/subsystem/resources/rstag.c - RsEndTagResource
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __RSTAG_C__

#include <acpi.h>

#define _COMPONENT          RESOURCE_MANAGER
        MODULE_NAME         ("rstag");



ACPI_STATUS
RsEndTagResource (
    UINT8                   *ByteStreamBuffer,
    UINT32                  *BytesConsumed,
    UINT8                   **OutputBuffer,
    UINT32                  *StructureSize)
{
    RESOURCE                *OutputStruct = (RESOURCE *) * OutputBuffer;
    UINT32                  StructSize = RESOURCE_LENGTH;
    

    FUNCTION_TRACE ("RsEndTagResource");

    /*
     * The number of bytes consumed is static
     */ 
    *BytesConsumed = 2;

    /*
     *  Fill out the structure
     */
    OutputStruct->Id = EndTag;

    /*
     * Set the Length parameter
     */
    OutputStruct->Length = 0;

    /*
     * Return the final size of the structure
     */
    *StructureSize = StructSize;

    return_ACPI_STATUS (AE_OK);
}



ACPI_STATUS
RsEndTagStream (
    RESOURCE                *LinkedList,
    UINT8                   **OutputBuffer,
    UINT32                  *BytesConsumed)
{
    UINT8                   *Buffer = *OutputBuffer;
    UINT8                   Temp8 = 0;        
    

    FUNCTION_TRACE ("RsEndTagStream");

    /*
     * The descriptor field is static
     */ 
    *Buffer = 0x79;

    Buffer += 1;

    /*
     * Set the Checksum - zero means that the resource data is treated as if
     *  the checksum operation succeeded (ACPI Spec 1.0b Section 6.4.2.8)
     */
    Temp8 = 0;

    *Buffer = Temp8;

    Buffer += 1;
    
    /*
     * Return the number of bytes consumed in this operation
     */
    *BytesConsumed = (UINT32) ((NATIVE_UINT) Buffer - (NATIVE_UINT) *OutputBuffer);

    return_ACPI_STATUS (AE_OK);
}

