/*
 *  ./acpica/subsystem/resources/rsvendor.c - RsVendorResource,
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __RSVENDOR_C__

#include <acpi.h>

#define _COMPONENT          RESOURCE_MANAGER
        MODULE_NAME         ("rsvendor");



ACPI_STATUS
RsVendorResource (
    UINT8                   *ByteStreamBuffer,
    UINT32                  *BytesConsumed,
    UINT8                   **OutputBuffer,
    UINT32                  *StructureSize)
{
    UINT8                   *Buffer = ByteStreamBuffer;
    RESOURCE                *OutputStruct = (RESOURCE *) * OutputBuffer;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;
    UINT8                   Index;
    UINT32                  StructSize = sizeof (VENDOR_RESOURCE) + RESOURCE_LENGTH_NO_DATA;
    

    FUNCTION_TRACE ("RsVendorResource");

    /*
     * Dereference the Descriptor to find if this is a large or small item.
     */ 
    Temp8 = *Buffer;
    
    if (Temp8 & 0x80)
    {
        /*
         * Large Item
         */
        /* Point to the length field */

        Buffer += 1;
        
        /* Dereference */

        Temp16 = *(UINT16 *)Buffer;
        
        /* Calculate bytes consumed */

        *BytesConsumed = Temp16 + 3;
        
        /* Point to the first vendor byte */

        Buffer += 2;
    }

    else
    {
        /*
         * Small Item
         */

        /* Dereference the size */

        Temp16 = (UINT8)(*Buffer & 0x07);
        
        /* Calculate bytes consumed */

        *BytesConsumed = Temp16 + 1;
        
        /* Point to the first vendor byte */

        Buffer += 1;
    }

    OutputStruct->Id = VendorSpecific;
    
    OutputStruct->Data.VendorSpecific.Length = Temp16;

    for (Index = 0; Index < Temp16; Index++)
    {
        OutputStruct->Data.VendorSpecific.Reserved[Index] = *Buffer;
        Buffer += 1;
    }

    /*
     * In order for the StructSize to fall on a 32-bit boundry,
     *  calculate the length of the vendor string and expand the 
     *  StructSize to the next 32-bit boundry.
     */
    if (0 == Temp16 % 4)
    {
        StructSize += Temp16;
    }

    else
    {
        StructSize += ((Temp16 / 4) * 4) + 4;
    }
 
    /*
     * Set the Length parameter
     */
    OutputStruct->Length = StructSize;

    /*
     * Return the final size of the structure
     */
    *StructureSize = StructSize;

    return_ACPI_STATUS (AE_OK);
}



ACPI_STATUS
RsVendorStream (
    RESOURCE                *LinkedList,
    UINT8                   **OutputBuffer,
    UINT32                  *BytesConsumed)
{
    UINT8                   *Buffer = *OutputBuffer;
    UINT16                  Temp16 = 0;
    UINT8                   Temp8 = 0;        
    UINT8                   Index;
    

    FUNCTION_TRACE ("RsVendorStream");

    /*
     * Dereference the length to find if this is a large or small item.
     */ 
    
    if(LinkedList->Data.VendorSpecific.Length > 7)
    {
        /*
         * Large Item
         */
        /*
         * Set the descriptor field and length bytes
         */ 
        *Buffer = 0x84;

        Buffer += 1;

        Temp16 = (UINT16) LinkedList->Data.VendorSpecific.Length;

        *(UINT16 *)Buffer = Temp16;

        Buffer += 2;
    }

    else
    {
        /*
         * Small Item
         */

        /*
         * Set the descriptor field
         */ 
        Temp8 = 0x70;

        Temp8 |= LinkedList->Data.VendorSpecific.Length;
        
        *Buffer = Temp8;

        Buffer += 1;
    }

    /*
     * Loop through all of the Vendor Specific fields
     */
    for (Index = 0; Index < LinkedList->Data.VendorSpecific.Length; Index++)
    {
        Temp8 = LinkedList->Data.VendorSpecific.Reserved[Index];
        *Buffer = Temp8;
        Buffer += 1;
    }

    /*
     * Return the number of bytes consumed in this operation
     */
    *BytesConsumed = (UINT32) ((NATIVE_UINT) Buffer - (NATIVE_UINT) *OutputBuffer);

    return_ACPI_STATUS (AE_OK);
}

