
/*
 *  ./acpica/subsystem/tables/tbfac.c - ACPI FAC* (FACP, FACS) utilities
 *
 *  Copyright (C) 2000 R. Byron Moore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define __TBFAC_C__

#include <acpi.h>
#include <hardware.h>
#include <tables.h>


#define _COMPONENT          TABLE_MANAGER
        MODULE_NAME         ("tbfac");




ACPI_STATUS
TbGetTableFacs (
    char                    *BufferPtr, 
    ACPI_TABLE_DESC         *TableInfo)
{
    void                    *TablePtr = NULL;
    UINT32                  Size;
    UINT8                   Allocation;
 ACPI_STATUS    Status = AE_OK;


    FUNCTION_TRACE ("TbGetTableFacs");


    /* Must have a valid FACP pointer */

    if (!Gbl_FACP)
    {
        return_ACPI_STATUS (AE_NO_ACPI_TABLES);
    }
    
    Size = sizeof (FIRMWARE_ACPI_CONTROL_STRUCTURE);
    if (BufferPtr)
    {
        /*
         * Getting table from a file -- allocate a buffer and
         * read the table.
         */
        TablePtr = CmAllocate (Size);
        if(!TablePtr)
        {
            return_ACPI_STATUS (AE_NO_MEMORY);
        }

        MEMCPY (TablePtr, BufferPtr, Size);
        
        /* Save allocation type */

        Allocation = ACPI_MEM_ALLOCATED;
    }
    
    else
    {
        /* Just map the physical memory to our address space */

        Status = TbMapAcpiTable ((void *) Gbl_FACP->FirmwareCtrl, &Size, &TablePtr);     
  if (ACPI_FAILURE(Status))
  {
      return_ACPI_STATUS (Status);
  }
        
        /* Save allocation type */

        Allocation = ACPI_MEM_MAPPED;
    }


    /* Return values */

    TableInfo->Pointer      = TablePtr;
    TableInfo->Length       = Size;
    TableInfo->Allocation   = Allocation;
    TableInfo->BasePointer  = TablePtr;
    
    return_ACPI_STATUS (Status);
}

