/*
 *  acenv.h - OS-specific defines
 *
 *  Copyright (C) 1999-2000 Andrew Henroid
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _ACENV_H_
#define _ACENV_H_

#include <ac-config.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

/* Enable the AML debugger and extra debug information */
#define ACPI_DEBUG 1

/* Uncomment to not access system resources (I/O ports, etc.) */
/* #define ACPI_STANDALONE 1 */

/*
 * Don't touch anything past this point
 */

/* Multi-thread support needed for debugger */
#ifdef ACPI_DEBUG
#undef ACPI_APPLICATION
#else
#define ACPI_APPLICATION 1
#endif

/* GDB style debugger, who'd want anything else? */
#ifdef ACPI_DEBUG
#define ACPI_GDB_STYLE
#endif

/* Use native Linux ACPI event handling */
#define ACPI_OSD_EVENTS 1

/* Use native Linux string library */
#define SYSTEM_CLIB_FUNCTIONS 1

/* String functions */
#define STRSTR(s1,s2)           strstr((char *) (s1), (char *) (s2))
#define STRUPR(s)               strupr((char *) (s))
#define STRLEN(s)               strlen((char *) (s))
#define STRCPY(d,s)             strcpy((char *) (d), (char *) (s))
#define STRNCPY(d,s,n)          strncpy((char *) (d), (char *) (s), (n))
#define STRNCMP(d,s,n)          strncmp((char *) (d), (char *) (s), (n))
#define STRCMP(d,s)             strcmp((char *) (d), (char *) (s))
#define STRCAT(d,s)             strcat((char *) (d), (char *) (s))
#define STRNCAT(d,s,n)          strncat((char *) (d), (char *) (s), (n))
#define STRTOUL(d,s,n)          strtoul((char *) (d), (char **) (s), (n))
#define MEMCPY                  memcpy
#define MEMSET                  memset
#define TOUPPER                 toupper
#define TOLOWER                 tolower

/* Interrupts and debugging */
#define causeinterrupt(level)
#define BREAKPOINT3
#define disable()   /* asm ("cli") */
#define enable()    /* asm("sti") */
#define halt()      /* asm("hlt") */
#define wbinvd()    /* asm("wbinvd") */

/* Global lock */
#define ASM_AcquireGL(GLptr, Acq) \
  do { \
	int dummy; \
	asm("1: movl           (%1),%%eax;" \
	       "movl           %%eax,%%edx;" \
	       "andl           %2,%%edx;" \
	       "btsl           $0x1,%%edx;" \
	       "adcl           $0x0,%%edx;" \
	       "lock; cmpxchgl %%edx,(%1);" \
	       "jnz            1b;" \
	       "cmpb           $0x3,%%dl;" \
	       "sbbl           %%eax,%%eax" \
	       :"=a"(Acq),"=c"(dummy):"c"(GLptr),"i"(~1L):"dx"); \
  } while(0)
		
#define ASM_ReleaseGL(GLptr, Acq) \
  do { \
        int dummy; \
        asm("1: movl           (%1),%%eax;" \
               "movl           %%eax,%%edx;" \
               "andl           %2,%%edx;" \
               "lock; cmpxchgl %%edx,(%1);" \
               "jnz            1b;" \
               "andl           $0x1,%%eax" \
               :"=a"(Acq),"=c"(dummy):"c"(GLptr),"i"(~3L):"dx"); \
  } while(0)

#endif /* _ACENV_H_ */
