/*
 *  acpid.h - ACPI daemon
 *
 *  Copyright (C) 1999-2000 Andrew Henroid
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _ACPID_H_
#define _ACPID_H_

#include <unistd.h>
#include <syslog.h>
#include <stdarg.h>
#include "acapi.h"
#include "acpiosd.h"
#include "acexcep.h"

#define ACPI_PROC "/proc/sys/acpi"

struct acpid_request;

/*
 * acpid.c
 */
void acpid_log(int priority, char *fmt,...);
unsigned long acpid_read_sysctl(const char *name);
int acpid_write_sysctl(const char *name, unsigned long value);
void acpid_exit(int sig);

/*
 * bm.c
 */
int acpid_bm_init(void);
int acpid_battery_request(struct acpid_request *rqst);

/*
 * debug.c
 */
int acpid_debug_init(void);
extern int acpid_debug_output;
void acpid_debug_printf(const char *str);
int acpid_debug_request(int fd);

/*
 * event.c
 */
int acpid_event_init(void);
int acpid_event_request(int fd);
void acpid_event_exit(void);

/*
 * ipc.c
 */
int acpid_server_detect(void);
int acpid_server_init(void);
int acpid_event_loop(int event, int server, int debug);

/*
 * pci.c
 */
int acpid_pci_init(void);
int acpid_pci_read(int bus, int devfn, int addr, void *val, int size);
int acpid_pci_write(int bus, int devfn, int addr, void *val, int size);

#endif /* _ACPID_H_ */
