/*
 *  ipc.h - Client-server communications
 *
 *  Copyright (C) 2000 Andrew Henroid
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _IPC_H_
#define _IPC_H_

/* always 8 bytes of protocol number */
#define ACPID_PROTOCOL "00000001"
#define ACPID_SERVER_PATH "/tmp/.acpi-unix"
#define ACPID_MAX_REQUEST 512
#define ACPID_REQUEST_DATA_SIZE \
	(ACPID_MAX_REQUEST - (sizeof(struct acpid_request) + 1))

/*
 * Request types
 *
 * Use explicit enumeration in order to keep protocol stable and
 * to create family masks.
 * A family is a set of requests that are handled by a single function.
 * A family mask is a bitmask that when or'ed with a request type
 * will determine whether the request type falls in that family.
 *
 * Values in this enumeration have the format
 * ACPID_<FAMILY>_<VERB>[_<OBJECT>]
 * or
 * ACPID_<FAMILY>_MASK
 */
enum
{
	ACPID_PID_GET = 0,

	ACPID_BATTERY_GET_COUNT = 0x1,
	ACPID_BATTERY_GET_INFO = 0x2,
	ACPID_BATTERY_GET_STATUS = 0x3,
	ACPID_BATTERY_MASK = 0x4,
};

/*
 * Status values
 */
enum
{
	ACPID_STATUS_ERROR= -1,
	ACPID_STATUS_OK = 0,
};

/*
 * Request packet
 */
struct acpid_request
{
	size_t size;  /* total packet size */
	int type;     /* request type */
	int status;   /* request status */
	char data[1]; /* request data (hangs off end) */
};

#endif /* _IPC_H_ */
