/*
 *  list.h - List primitives
 *
 *  Copyright (C) 2000 Andrew Henroid
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _LIST_H_
#define _LIST_H_

struct list_entry
{
	struct list_entry *next;
	struct list_entry *prev;
};

#define LIST_HEAD(name) \
        struct list_entry name = {&name, &name}

#define LIST_ENTRY(entry, type, field) \
        ((type *)((char *)(entry) - (int)(&((type *) 0)->field)))

static inline void
list_init(struct list_entry *list)
{
	list->next = list;
	list->prev = list;
}

static inline void
list_append(struct list_entry *list, struct list_entry *entry)
{
	struct list_entry *prev = list->prev;
	entry->next = list;
	entry->prev = prev;
	prev->next = entry;
	list->prev = entry;
}

static inline void
list_remove(struct list_entry *entry)
{
	struct list_entry *next = entry->next;
	struct list_entry *prev = entry->prev;
	prev->next = next;
	next->prev = prev;
}

#endif // _LIST_H_

